#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif

#include "libhdr.h"

#define	STDIN	0
#define	STDOUT	1
#define   READ	     0
#define   WRITE	1

static pid_t childpid = 0;

static void dupto (const int from,const int to,const char *msg)
/*@globals errno, stderr; @*/
/*@modifies fileSystem, errno, *stderr; @*/
{
	int err;

	if (from != to)
	{
		err=close(to);
		if (err < 0 && errno != EBADF)
		{
			(void)fprintf(stderr,"dupto(): Cannot close file descriptor\n");
			exit(EXIT_FAILURE);
		}
		err=dup(from);
		if (err != to)
		{
			(void)fprintf(stderr,"dupto(): Cannot dup %s\n",msg);
			exit(EXIT_FAILURE);
		}
		if(close(from)<0)
		{
			(void)fprintf(stderr,"dupto(): cannot close file descriptor\n");
			exit(EXIT_FAILURE);
		}
	}
}

int child_open(const int read_flag,const char *fname,const char *filterprog)
/*@globals errno, stderr; @*/
/*@modifies fileSystem, errno, *stderr; @*/
{
	int ppipe[2]={-1,-1},fd= -1,fd1;
	
	struct stat sbuf;

	if(!fname) return fd;
	if(read_flag==READ) if(stat(fname,&sbuf)) return fd;
	if(pipe(ppipe)<0)
	{
		(void)fprintf(stderr,"child_open(): Can't open pipe\n");
		return fd;
	}

	childpid = fork ();
	if(childpid<0)
	{
		(void)fprintf(stderr, "child_open(): cannot fork\n");
		return fd;
	}
	if(childpid>0)
	{
		if (read_flag==READ)
		{
			fd = ppipe[READ];
			if(close(ppipe[WRITE])<0)
			{
				(void)fprintf(stderr, "child_open(): cannot close pipe\n");
				exit(EXIT_FAILURE);
			}
		}
		else
		{
			fd = ppipe[WRITE];
			if(close(ppipe[READ])<0)
			{
				(void)fprintf(stderr, "child_open(): cannot close pipe\n");
				exit(EXIT_FAILURE);
			}
		}
	}
	else
	{
		if (read_flag==READ)
		{
			dupto (ppipe[WRITE], STDOUT, "(child) pipe to stdout");
			if(close(ppipe[READ])<0)
			{
				(void)fprintf(stderr, "child_open(): cannot close pipe\n");
				exit(EXIT_FAILURE);
			}
			if(fname)
			{
				fd1=open(fname,O_RDONLY,0666);
				if (fd1<0)
				{
					(void)fprintf(stderr, "child_open(): cannot open file %s\n",fname);
					exit(EXIT_FAILURE);
				}
				dupto(fd1,STDIN,"file to stdin");
			}
		}
		else
		{
			dupto (ppipe[READ], STDIN, "(child) pipe to stdin");
			if(close(ppipe[WRITE])<0)
			{
				(void)fprintf(stderr, "child_open(): cannot close pipe\n");
				exit(EXIT_FAILURE);
			}
			if(fname)
			{
				fd1=creat(fname,0666);
				if (fd1<0)
				{
					(void)fprintf(stderr, "child_open(): cannot open file %s\n",fname);
					exit(EXIT_FAILURE);
				}
				dupto(fd1,STDOUT,"file to stdout");
			}
		}
		
		if (read_flag==READ) (void)execlp (filterprog,filterprog,"-d",(char *)0);
		else (void)execlp(filterprog,filterprog,(char *) 0);
		(void)fprintf(stderr, "child_open(): cannot exec %s\n",filterprog);
		_exit(EXIT_FAILURE);
	}
	return fd;
}

#if !HAVE_POPEN
/* In case the system popen() command does not exist */
/*@null@*/ /*@dependent@*/ FILE *popen(const char *command,const char *type)
/*@globals errno, stderr; @*/
/*@modifies fileSystem, errno, *stderr; @*/
{
	int ppipe[2]={-1,-1},read_flag,fd;
	FILE *fptr=0;
	
	if(!command) return fptr;
	if(!strcmp("r",type)) read_flag=READ;
	else if(!strcmp("w",type)) read_flag=WRITE;
	else return fptr;
	
	if(pipe(ppipe)<0)
	{
		(void)fprintf(stderr,"lk_popen(): Can't open pipe\n");
		return fptr;
	}

	childpid = fork ();
	if(childpid<0)
	{
		(void)fprintf(stderr, "lk_popen(): cannot fork\n");
		return fptr;
	}
	if(childpid>0)
	{
		if (read_flag==READ)
		{
			fd = ppipe[READ];
			if(close(ppipe[WRITE])<0)
			{
				(void)fprintf(stderr, "lk_popen(): cannot close pipe\n");
				exit(EXIT_FAILURE);
			}
		}
		else
		{
			fd = ppipe[WRITE];
			if(close(ppipe[READ])<0)
			{
				(void)fprintf(stderr, "lk_popen(): cannot close pipe\n");
				exit(EXIT_FAILURE);
			}
		}
		fptr=fdopen(fd,type);
		if(!fptr)
		{
			(void)fprintf(stderr, "lk_popen(): fdopen() failed\n");
			exit(EXIT_FAILURE);
		}
	}
	else
	{
		if (read_flag==READ)
		{
			dupto (ppipe[WRITE], STDOUT, "(child) pipe to stdout");
			if(close(ppipe[READ])<0)
			{
				(void)fprintf(stderr, "lk_popen(): cannot close pipe\n");
				exit(EXIT_FAILURE);
			}
		}
		else
		{
			dupto (ppipe[READ], STDIN, "(child) pipe to stdin");
			if(close(ppipe[WRITE])<0)
			{
				(void)fprintf(stderr, "lk_popen(): cannot close pipe\n");
				exit(EXIT_FAILURE);
			}
		}
		(int)execl("/bin/sh","sh","-c",command,(char *)0);
		(void)fprintf(stderr, "lk_popen(): cannot exec %s\n",command);
		_exit(EXIT_FAILURE);
	}
	return fptr;
}

#endif
