/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *             Simon Heath - University of Washington                       *
 *                                                                          *
 *                       July 1997                                          *
 *                                                                          *
 * alloc_loki.c:                                                            *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <stdio.h>
#include <stdlib.h>

#include "utils.h"
#include "loki.h"

int n_genetic_groups=1;

/* n_tloci refers to how much storage for trait loci we currently have, not
 * how many are actually in the model */
int n_tloci=8;
struct TraitLocus *tlocus=0;

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "AllocLokiStruct"
void AllocLokiStruct(void)
{
	int i,j,k,*tp1;
	double **tpp,*tp;
	signed char *tcp,**tcpp;
	
	if(n_markers) {
		if(!(tcp=malloc((size_t)n_markers*3*ped_size))) ABT_FUNC(MMsg);
		for(i=0;i<n_markers*3*ped_size;i++) tcp[i]= -2;
		for(i=0;i<n_markers;i++) {
			marker[i].seg[0]=tcp;
			marker[i].seg[1]=tcp+ped_size;
			marker[i].m_flag=tcp+2*ped_size;
			tcp+=3*ped_size;
		}
		for(j=i=0;i<n_markers;i++) j+=marker[i].n_alleles;
		if(j) {
			if(!(tpp=malloc(sizeof(void *)*n_genetic_groups*n_markers))) ABT_FUNC(MMsg);
			if(!(tcpp=malloc(sizeof(void *)*n_genetic_groups*n_markers))) ABT_FUNC(MMsg);
			if(!(tcp=malloc((size_t)j*n_genetic_groups*n_markers))) ABT_FUNC(MMsg);
			if(!(tp=malloc(sizeof(double)*j*n_genetic_groups*n_markers))) ABT_FUNC(MMsg);
			if(!(tp1=malloc(sizeof(int)*n_genetic_groups*n_markers))) ABT_FUNC(MMsg);
			RemBlock=AddRemem(tpp,RemBlock);
			RemBlock=AddRemem(tcpp,RemBlock);
			RemBlock=AddRemem(tcp,RemBlock);
			RemBlock=AddRemem(tp,RemBlock);
			RemBlock=AddRemem(tp1,RemBlock);
			for(i=0;i<n_markers;i++) {
				if(!marker[i].n_alleles) {
					marker[i].freq=0;
					marker[i].freq_set=0;
					marker[i].count_flag=0;
					continue;
				}
				marker[i].freq=tpp;
				marker[i].freq_set=tcpp;
				marker[i].count_flag=tp1;
				tcpp+=n_genetic_groups;
				tpp+=n_genetic_groups;
				tp1+=n_genetic_groups;
				for(j=0;j<n_genetic_groups;j++) {
					marker[i].count_flag[j]=0;
					marker[i].freq[j]=tp;
					marker[i].freq_set[j]=tcp;
					tp+=marker[i].n_alleles;
					tcp+=marker[i].n_alleles;
					for(k=0;k<marker[i].n_alleles;k++) {
						marker[i].freq[j][k]=0.0;
						marker[i].freq_set[j][k]=0;
					}
				}
			}
		} else for(i=0;i<n_markers;i++) {
			marker[i].freq=0;
			marker[i].freq_set=0;
		}
		for(i=0;i<n_markers;i++) {
			marker[i].pos_set=0;
			marker[i].pos[0]=marker[i].pos[1]=0.0;
		}
	}
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "get_new_traitlocus"
int get_new_traitlocus(const int n_all)
{
	int i,j,k;
	double *tp,**tpp;
	
	if(!tlocus) return -1;
	if(n_all<2) return -1;
	for(i=0;i<n_tloci;i++) if(!tlocus[i].flag) break;
	if(i==n_tloci)	{
		if(i<max_tloci) {
			n_tloci*=1.5;
			if(n_tloci>max_tloci) n_tloci=max_tloci;
			if(!(tlocus=realloc(tlocus,sizeof(struct TraitLocus)*n_tloci))) ABT_FUNC(MMsg);
			k=n_tloci-i;
			for(j=i;j<n_tloci;j++) {
				tlocus[j].freq=0;
				tlocus[j].flag=0;
				tlocus[j].seg[0]=0;
				tlocus[j].gt=0;
			}
		} else return -1;
	}
	j=n_all*(n_all+1)/2-1;
	if(!(tpp=malloc(sizeof(void *)*n_genetic_groups))) ABT_FUNC(MMsg);
	if(!(tp=malloc(sizeof(double)*(j+n_genetic_groups*n_all)))) ABT_FUNC(MMsg);
	tlocus[i].freq=tpp;
	tlocus[i].eff=tp;
	tp+=j;
	for(k=0;k<n_genetic_groups;k++) tlocus[i].freq[k]=tp+n_all*k;
	tlocus[i].n_alleles=n_all;
	if(!(tlocus[i].seg[0]=calloc((size_t)ped_size*2,1))) ABT_FUNC(MMsg);
	tlocus[i].seg[1]=tlocus[i].seg[0]+ped_size;
	for(j=0;j<ped_size;j++) tlocus[i].seg[0][j]=tlocus[i].seg[1][j]= -2;
	if(!(tlocus[i].gt=calloc(sizeof(int)*ped_size,1))) ABT_FUNC(MMsg);
	return i;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "delete_traitlocus"
void delete_traitlocus(const int tl)
{
	if(tl<0 || tl>=n_tloci) ABT_FUNC("Internal error - illegal trait locus\n");
	tlocus[tl].flag=0;
	if(tlocus[tl].seg[0]) free(tlocus[tl].seg[0]);
	if(tlocus[tl].gt) free(tlocus[tl].gt);
	if(tlocus[tl].freq) free(tlocus[tl].freq);
	if(tlocus[tl].eff) free(tlocus[tl].eff);
	tlocus[tl].seg[0]=0;
	tlocus[tl].gt=0;
	tlocus[tl].freq=0;
	tlocus[tl].eff=0;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "AllocEffects"
void AllocEffects(void)
{
	int i,j,k,k1,type,*tip;
	struct Variable *var;
	double *tp;
	struct id_data *data;
	
	j=0;
	type=model.var.type;
	k=model.var.var_index;
	for(i=0;i<ped_size;i++) {	
		id_array[i].res=id_array[i].cens=id_array[i].vv=0;
		if(type&ST_CONSTANT)	{
			if(id_array[i].data) data=id_array[i].data+k;
			else data=0;
			if(data && data->flag) {
				j+=1+use_student_t;
				if((type&ST_CENSORED)&&(data->flag&2)) j++;
			}
		} else if(id_array[i].n_rec) {
			j+=id_array[i].n_rec*(1+use_student_t);
			if(type&ST_CENSORED) j+=id_array[i].n_rec;
		}
	}
	if(j)	{
		if(!(tp=malloc(sizeof(double)*j))) ABT_FUNC(MMsg);
		RemBlock=AddRemem(tp,RemBlock);
		for(i=0;i<j;i++) tp[i]=0.0;
		for(i=0;i<ped_size;i++)	{	
			if(type&ST_CONSTANT)	{
				if(id_array[i].data) data=id_array[i].data+k;
				else data=0;
				if(data && data->flag) {
					id_array[i].res=tp++;
					if(use_student_t) id_array[i].vv=tp++;
					if((type&ST_CENSORED)&&(data->flag&2)) id_array[i].cens=tp++;
				}
			} else {
				if(id_array[i].n_rec) {
					id_array[i].res=tp;
					tp+=id_array[i].n_rec;
					if(use_student_t) {
						id_array[i].vv=tp;
						tp+=id_array[i].n_rec;
					}
					if(type&ST_CENSORED)	{
						id_array[i].cens=tp;
						tp+=id_array[i].n_rec;
					}
				}
			}
		}
	}
	for(k=i=0;i<model.n_terms;i++) {
		model.term[i].df=1;
		if(model.term[i].n_vars>1) ABT_FUNC("Sorry - interaction terms not currently supported\n");
		type=model.term[i].vars[0].type;
		j=model.term[i].vars[0].var_index;
		if(type&ST_TRAITLOCUS) model.term[i].df=2;
		else if(type&ST_ID) model.term[i].df=ped_size;
		else if(type&ST_MARKER) {
			k1=marker[j].n_alleles;
			model.term[i].df=k1*(k1+1)/2-1;
			marker[j].mterm=model.term+i;
		} else {
			if(type&ST_CONSTANT) var=id_variable+j;
			else var=nonid_variable+j;
			if(type&ST_FACTOR) {
				if(type&ST_RANDOM) model.term[i].df=var->n_levels;
				else model.term[i].df=var->n_levels-1;
			}
		}
		if(!(type&ST_TRAITLOCUS)) k+=model.term[i].df;
	}
	if(n_markers) {
		if(!(tip=calloc((size_t)n_markers*ped_size,sizeof(int)))) ABT_FUNC(MMsg);
		RemBlock=AddRemem(tip,RemBlock);
		for(i=0;i<n_markers;i++) {
			marker[i].gt=tip;
			tip+=ped_size;
		}
	}
	if(!k) return;
	if(!(tp=malloc(sizeof(double)*k))) ABT_FUNC(MMsg);
	RemBlock=AddRemem(tp,RemBlock);
	for(i=0;i<k;i++) tp[i]=0.0;
	for(i=0;i<model.n_terms;i++) if(!(model.term[i].vars[0].type&ST_TRAITLOCUS)) {
		model.term[i].eff=tp;
		tp+=model.term[i].df;
	}
}
