#ifndef lint
static char const 
yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28 2000/01/17 02:04:06 bde Exp $";
#endif
#include <stdlib.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
static int yygrowstack();
#define YYPREFIX "yy"
#line 2 "control_parse.y"
/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *             Simon Heath - University of Washington                       *
 *                                                                          *
 *                       March 1997                                         *
 *                                                                          *
 * control_parse.y:                                                         *
 *                                                                          *
 * yacc source for control file parser.                                     *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <string.h>

#include "utils.h"

#line 29 "control_parse.y"
typedef union 
{
	char *string;
	struct scan_var *var;
	int value;
	double rvalue;
	struct format_clause *format_clause;
	struct fformat *fformat;
	struct format_atom *f_atom;
	struct model_list *model_list;
	struct var_list *var_list;
	struct var_element *element;
	struct express *express;
} YYSTYPE;
#line 75 "control_parse.y"
#include "scan.h"
#include "scanner.h"

static struct format_atom *make_f_atom(int,int);
static struct format_clause *add_f_atom(struct format_clause *,struct format_atom *);
static struct format_clause *add_f_list(struct format_clause *,struct format_clause *,int);
static struct format *setup_format(struct format_clause *);
static struct model_list *add_to_model(struct model_list *,struct var_list *);
static struct var_list *add_to_var_list(struct var_list *,struct scan_var *,struct express *express);
static struct var_list *add_var_lists(struct var_list *,struct var_list *);
static struct var_element *get_element(struct scan_var *var,struct express *express);
static struct var_element *assign_var(struct scan_var *var,struct express *ix,struct express *express);
static struct express *alloc_express(void);
static struct express *do_express_op(struct express *ex1,struct express *ex2,int op);
static struct express *do_logical_op(struct express *ex1,struct express *ex2,int op);
static struct fformat *add_fformat(struct fformat *f1,struct fformat *f2);
static struct fformat *create_fformat(char *s,int fg);
static void begin_looping(struct var_element *element, struct express *exp1, struct express *exp2);
static void free_vlist(struct var_list *vlist);
static void do_ped_com(struct var_list *vlist);
static void add_restriction(struct var_list *);
static void add_censored(struct var_element *,const int fg);
static void do_file_com(char *fname,struct format_clause *fc,struct fformat *ff,struct var_list *vlist);
static void set_locus_element(struct var_element *);
static void set_haplo_element(struct var_element *,struct var_element *);
static void do_link_com(char *s,int type, struct var_list *);
static void do_missing_com(char *,struct var_list *);
static void change_type(int,struct var_list *);
static void do_model_com(struct model_list *,struct scan_var *var,struct express *express);
static void add_operation(void *,int,int);
static void set_array_var(struct scan_var *var,struct express *express);
static void check_element_add_op(struct var_element *);
static void enter_loop(void);
static void start_loopclause(void);
static void do_while_com(struct express *express);
static void print_exp(struct express *express);
static void new_command(void);
static void set_sex(struct var_element *elem,struct express *exp1,struct express *exp2);
static void set_group(struct var_element *elem);
static int count_var_list(struct var_list *vlist),shell_flag;
static struct format_atom *f_atom_list;
static struct var_element *pedlist[4];
static int f_atom_n,f_atom_size=32,pedflag;
struct operation *Affected;
struct scan_var *root_var;
struct InFile *Infiles;
struct Link *links;
struct Miss *Miss;
struct Restrict *Restrictions;
struct Censor *Censored;
struct model *Models;
static struct operation *Op_List;
struct Marker *markers,*traitlocus;
static struct var_element *hap_list[2];
struct express *sex_exp[2];
struct var_element *sex_elem,*group_elem;
static char *string_copy(char *s1,char *s2);

int scan_error,scan_error_n,scan_warn_n;
int max_scan_errors=30,max_scan_warnings=30,n_markers,iflag;
char *LogFile,*Filter,*rsformat,*fsformat,*OutputFile;
int loop_level,loop_ptr[MAX_LOOP],loop_stat[MAX_LOOP],loop_record,loop_stack_size=256;
int loop_main_ptr,in_loopclause,loop_clause_end,loop_clause_step,loop_clause_ptr;
int syst_var[NUM_SYSTEM_VAR];
int family_id;
struct var_element *loop_clause_element;
struct token_store *loop_stack;
	  
#line 127 "y.tab.c"
#define YYERRCODE 256
#define FILEC 257
#define MARKER 258
#define LOCUS 259
#define TRAIT 260
#define RANDOM 261
#define PEDIGREE 262
#define LOG 263
#define MODEL 264
#define FILTER 265
#define LINK 266
#define MISSING 267
#define FACTOR 268
#define BREAK 269
#define DOLOOP 270
#define WHILE 271
#define USE 272
#define WHERE 273
#define ORSYMBOL 274
#define ANDSYMBOL 275
#define NEQSYMBOL 276
#define LEQSYMBOL 277
#define GEQSYMBOL 278
#define NOTSYMBOL 279
#define LOGICAL 280
#define SHELL 281
#define ARRAY 282
#define PRINTEXP 283
#define INCLUDE 284
#define LOOP_CLAUSE_START 285
#define LOOP_CLAUSE_END 286
#define CONSTANT 287
#define RSFORMAT 288
#define FSFORMAT 289
#define CENSORED 290
#define GROUP 291
#define SET 292
#define GENDER 293
#define AFFECTED 294
#define OUTPUT 295
#define STRING 296
#define VARIABLE 297
#define ASSIGN 298
#define ARRAY_VAR 299
#define INTEGER 300
#define SYSTEM_VAR 301
#define REAL 302
#define UMINUS 303
const short yylhs[] = {                                        -1,
    0,    0,    0,    0,    0,   21,   21,   21,   21,   21,
   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,
   21,   21,   21,   21,   21,   21,   21,   21,   21,   21,
   21,   21,   21,   35,   17,   17,   18,   18,   18,   18,
   18,   18,   18,   18,   18,   18,   46,   46,   40,   50,
   24,   43,   44,   48,   41,   19,   19,   19,   19,   19,
   19,   19,   19,   19,   19,   39,   52,   52,   47,   47,
   38,   53,   53,   36,   37,   33,   33,   34,   34,   51,
   51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
   51,   51,   51,   51,   51,   51,   51,   51,   22,   22,
   22,    3,    3,    4,    4,    4,    4,    5,    5,    6,
    6,    7,    7,    7,    7,    7,   25,   49,   31,   31,
   31,   26,   27,   27,   28,   42,   32,   20,   20,   20,
   29,   29,   29,   23,   30,   30,   45,    8,    8,    8,
    8,   12,   12,   12,   12,   13,   13,   13,   16,   16,
   54,   54,   55,   56,   56,   56,   14,   14,   57,   58,
   58,   15,   15,    9,    9,    9,    9,   10,   10,   10,
   10,   11,   11,   11,   11,    2,    2,    2,    1,    1,
    1,    1,
};
const short yylen[] = {                                         2,
    1,    1,    2,    3,    3,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    3,    6,    1,    1,    1,    1,
    3,    3,    3,    3,    2,    3,    3,    3,    1,    0,
    3,    6,    5,    5,    4,    1,    3,    3,    3,    3,
    3,    3,    3,    3,    2,    2,    1,    3,    2,    2,
    2,    4,    6,    2,    2,    6,    5,    6,    4,    1,
    1,    1,    1,    3,    3,    3,    3,    3,    2,    3,
    3,    3,    3,    3,    3,    3,    3,    2,    4,    7,
    7,    1,    4,    1,    3,    4,    6,    1,    3,    2,
    2,    1,    2,    1,    2,    1,    2,    2,    2,    3,
    4,    2,    3,    3,    2,    2,    2,    1,    4,    4,
    2,    4,    2,    2,    4,    7,    2,    3,    3,    1,
    1,    3,    3,    3,    3,    1,    1,    4,    1,    4,
    1,    3,    2,    3,    5,    3,    1,    3,    1,    8,
   10,    4,    1,    0,    1,    1,    4,    1,    1,    3,
    3,    1,    1,    3,    3,    1,    3,    3,    1,    1,
    3,    3,
};
const short yydefred[] = {                                      0,
    2,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   49,    0,    0,    0,    0,    0,   50,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   34,    0,    1,    6,    7,    8,    9,   10,
   11,   12,   13,   14,   15,   16,   17,   18,   19,   20,
   21,   22,   23,   24,   25,   26,   27,   28,   29,   30,
   31,   32,   33,    0,  179,  149,    0,    0,    0,    0,
  180,    0,    0,  146,    0,  159,    0,  172,  173,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   39,   38,   37,    0,    0,
   40,    0,    0,    0,    0,    0,    0,    0,  137,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  176,
    0,    0,    0,    0,    0,    0,    0,  116,    0,    0,
    0,  114,    0,    0,  108,  104,    0,    0,    0,    0,
  151,    0,    0,    0,  157,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   82,
   80,   81,    0,    0,   83,    0,    0,    0,   45,    0,
    0,    0,    0,    0,    0,    0,    0,   48,   47,    0,
    0,    0,    0,    5,    4,    0,    0,    0,    0,    0,
    0,    0,    0,  115,    0,  113,    0,    0,    0,    0,
  181,  182,  165,    0,  168,    0,  169,    0,  153,    0,
    0,  174,  175,    0,    0,    0,    0,    0,    0,  129,
  130,    0,   65,    0,    0,    0,    0,    0,    0,    0,
    0,   55,    0,    0,   98,   89,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   46,    0,    0,   43,   44,    0,    0,    0,
    0,    0,    0,  177,    0,  178,  103,  150,    0,    0,
  105,    0,    0,  109,    0,    0,    0,    0,  152,  148,
    0,  158,  162,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   59,   60,    0,   61,   62,   77,    0,
   88,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   86,   87,    0,   72,    0,    0,    0,   53,    0,
    0,  106,    0,    0,    0,    0,  170,  171,  156,    0,
  154,    0,    0,    0,  144,  145,  142,  143,    0,   76,
    0,    0,   52,    0,    0,    0,    0,  167,    0,    0,
    0,   73,  107,  155,    0,  160,    0,    0,  161,
};
const short yydgoto[] = {                                      32,
   69,  123,   70,  133,  134,  135,  136,  216,  205,  206,
   77,  217,   78,  146,   79,  101,   33,  155,  156,   34,
   35,   36,   37,   38,   39,   40,   41,   42,   43,   44,
   45,   46,   47,   48,   49,   50,   51,   52,   53,   54,
   55,   56,   57,   58,   59,   60,   61,   62,   63,  104,
  166,  103,   95,  140,  141,  209,   80,   81,
};
const short yysindex[] = {                                    549,
    0,  328, -248, -221,  110,  110,  192,   16,  192,   39,
  192,  110,    0,  134,  131,  143,  -87,  106,    0,  110,
  192,  192,   28,   28, -168,   28,  -50,  192,  142,  110,
  110,  -40,    0,  135,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  196,    0,    0,  203,  313,  234,  238,
    0,   28,  110,    0,  263,    0,  282,    0,    0,   54,
   37,  282,  234,  287,  323,  234, -102,   94,  282,   99,
  329,  -38,  121,  335,  340,    0,    0,    0,  106,  106,
    0,  552,  348,  192,  282,  234,  234,  149,    0,  129,
  169,  106,  339,  234,  106,  106,  282,  282,  620,    0,
    0,  354,  216,  282,  373,  192,  106,    0,  192,  192,
  141,    0,   79,   65,    0,    0,  223,  127,  336,  406,
    0,  282,  106,  110,    0,  427,   54,   54,  106,  385,
  387,  110,  282,   99,  552,  136,  121,  450,  121,    0,
    0,    0,  121,  121,    0,  166,  106,  205,    0,  573,
  106,  106,  106,  106,  106,  234,  460,    0,    0,  623,
  121,  552,  610,    0,    0,  106,  211,  110,  230,  278,
  617,  234,  234,    0,  -83,    0,   40,  -97,  -97,  -25,
    0,    0,    0,  469,    0,  470,    0, -135,    0,   28,
  635,    0,    0, -121,  -28,  485,  179,  188,  679,    0,
    0,  282,    0,   99,   99,   99,   99,   99,   99,   99,
   99,    0,  177,  121,    0,    0,  425,  121,  121,  121,
  121,  121,  121,  121,  121,  121,  121,  121,  121,  264,
  701,  498,    0,   98,   98,    0,    0,  552,  121,  106,
  432,  482,  720,    0,  282,    0,    0,    0,  144,  305,
    0,  502,  508,    0,  106,  127,  473,   97,    0,    0,
  255,    0,    0,   54,   54,   54,   54,   54,   54,  510,
  206,  451,   66,    0,    0,   66,    0,    0,    0,  456,
    0,  515,  525,  546,  641,  641,  546,  641,  641,  159,
  159,    0,    0,  110,    0,  106,  463,  552,    0,  106,
    0,    0,  -83,  127,  127,  740,    0,    0,    0,   28,
    0,  530,  179,  188,    0,    0,    0,    0,   54,    0,
  282,  753,    0,  552,  150,  470,  470,    0,  497,  106,
  485,    0,    0,    0,  596,    0,  106,  825,    0,
};
const short yyrindex[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  140,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  534,    0,
    0,    0,    0,    0,    2,    0,    1,    0,    0,    0,
    7,    3,    4,    0,    0,   10,    0,   25,   26,    0,
    0,    0,    0,    0,   73,    0,    0,    0,    0,    0,
    0,    5,   77,    0,   80,   83,   85,    0,    0,    0,
    0,    0,    0,   87,    0,    0,   88,   89,   90,    0,
   12,   21,   95,   96,    0,    0,    0,    0,    0,    0,
  148,    0,    0,    0,    0,    0,    0,   22,    0,  101,
    0,  104,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  105,    0,  171,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  107,    0,    0,    0,    0,
    0,   23,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   93,  100,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   24,    0,  108,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  111,   41,   43,    0,    0,
    0,  112,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  113,
    0,    0,    0,    9,   31,    0,    0,   35,    0,    0,
    0,    0,    0,    0,  114,    0,    0,    0,    0,  148,
    0,    0,    0,    0,    0,   22,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   -5,  221,   86,    0,    0,  289,    0,    0,    0,    0,
    0,   81,  270,  374,  273,  306,  415,  397,  480,  256,
  494,    0,    0,    0,    0,    0,    0,  116,    0,    0,
   20,    0,    0,   22,   22,    0,    0,    0,    0,    0,
    0,    0,   59,   60,    0,    0,    0,    0,    0,    0,
  117,    0,    0,   62,    0,  118,  119,    0,    0,    0,
  120,    0,    0,    0,    0,    0,    0,    0,    0,
};
const short yygindex[] = {                                      0,
  589,  403,  139, -181,    0,  404,  408,  268,  327,   47,
   27,  334,  176,  465,  435,    6,  341,  -18,  405,    0,
  504,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  616,    0,    0,    0,  411,    0,    0,    0,
};
#define YYTABLESIZE 922
const short yytable[] = {                                     102,
  125,  147,  122,  117,   67,  144,  163,   71,   41,  134,
   72,  146,   71,  269,   71,  284,   71,  150,  151,  148,
  147,  164,   35,  166,  119,  127,   71,   71,  108,  109,
   42,  112,   82,   71,   68,   63,  132,   73,   89,  125,
  140,   92,  141,  147,  147,  147,  105,  147,   67,   41,
  163,   41,   41,   41,  149,  146,  117,  118,  139,  138,
  124,   36,  150,  148,  147,  164,   35,  166,   41,   41,
   41,   42,   71,   42,   42,   42,   66,  139,   68,  126,
  169,  170,   69,  140,   70,  141,  118,   75,   74,    3,
   42,   42,   42,  180,  133,  131,  182,  183,  165,  142,
  123,  139,  138,  124,  120,   36,   51,   99,  191,   71,
  135,  121,   79,  132,  153,   54,   78,  100,  101,  136,
  277,   96,  197,  200,  211,  230,   58,  231,  110,   87,
  219,   71,  111,   76,   71,   71,  137,  152,  100,  173,
  330,  345,  202,   99,  174,  100,   58,  281,  251,   76,
   99,  111,  254,  255,  256,  257,  258,  199,  110,  132,
  164,   66,  165,   67,  165,  163,   76,  263,  165,  165,
   76,  198,  128,   90,   76,   74,  232,   75,  222,  128,
  195,  116,   93,   64,  322,  111,  165,  197,  112,  331,
  353,  112,  110,  197,  125,  230,  229,  231,   65,   66,
  248,   67,  115,   71,   71,  249,  250,  248,  246,   94,
  247,   56,  249,  278,  265,  139,  131,  299,  248,  246,
  286,  247,  113,  249,  287,  244,  243,  245,  119,  288,
   56,   56,   56,  289,  158,  126,  244,  243,  245,  165,
  112,  318,  127,  165,  165,  165,  165,  165,  165,  165,
  165,  165,  165,  165,  165,  145,  326,  283,  187,  188,
  196,   64,  129,  130,  165,  230,  229,  231,   63,  125,
  147,  122,  117,   67,  147,  163,  137,   41,  134,  163,
  146,  138,   41,   41,   41,   41,   41,  147,  148,  147,
  164,   35,  166,  119,  127,  128,   84,  342,   84,   42,
   84,  344,  143,   68,   42,   42,   42,   42,   42,  140,
   97,  141,   84,   92,   85,   84,   84,   84,  267,  212,
  137,  147,  145,  218,   66,  144,   67,  139,  138,   57,
   36,  355,   92,   92,   92,  349,  272,  273,  358,  270,
  341,   71,  227,  228,  323,   66,   93,  148,  126,   57,
   74,   69,   75,   70,   96,  118,   75,   74,    3,   58,
   58,   58,  149,  133,  131,   93,   93,   93,  157,  123,
  346,  347,  124,  120,  167,   51,   99,  154,  181,  135,
  121,   79,  132,  168,   54,   78,  100,  101,  136,  282,
   74,  175,   75,  186,   96,   66,  194,   67,   97,  159,
   98,   96,   66,   91,   67,   97,   74,   98,   75,  224,
  225,  226,  227,  228,   91,  189,  160,   66,   68,   67,
  161,  177,  162,  203,  196,  204,  208,   74,  178,   75,
  120,  121,  132,  122,   91,  128,  128,   94,  128,  238,
  239,  240,  241,  242,   56,   56,   56,   56,   56,  210,
  238,  239,  240,  241,  242,   90,   94,   94,   94,  282,
  334,  335,  336,  337,  338,  301,  248,  246,  179,  247,
  214,  249,  319,  248,  246,   90,  247,  220,  249,  221,
  225,  226,  227,  228,  244,  243,  245,   65,   66,  234,
   67,  244,  243,  245,   64,   64,  340,  248,  246,  259,
  247,  252,  249,  343,  248,  246,  264,  247,  275,  249,
  230,  229,  231,  276,  218,  244,  243,  245,  201,   66,
   95,   67,  244,  243,  245,  120,   66,  285,   67,   84,
   84,   84,   84,   84,   85,  314,   85,  316,   85,   95,
   95,   95,  320,   97,   97,  324,   92,   92,   92,   92,
   92,  325,   29,   85,   85,   85,  248,  246,  223,  247,
  194,  249,   57,   57,   57,  329,  248,  246,  128,  247,
  339,  249,  207,  350,  244,  243,  245,  102,  213,   93,
   93,   93,   93,   93,  244,  243,  245,  248,  246,  354,
  247,  266,  249,  173,  171,   83,  172,   86,  174,   88,
  129,  130,  327,  274,  271,  244,  351,  245,   64,  106,
  107,  215,  131,  253,  173,  171,  114,  172,  333,  174,
  279,  332,  185,   65,   66,    0,   67,    0,  291,  292,
  293,  294,  295,  296,  297,  298,  356,  173,  171,  357,
  172,    0,  174,    0,    0,    0,    0,   91,   91,   91,
  262,  173,  171,    0,  172,    0,  174,  268,  173,  171,
    0,  172,    0,  174,  173,  171,  260,  172,    0,  174,
   94,   94,   94,   94,   94,  280,  173,  171,    0,  172,
    0,  174,  248,  246,    0,  247,    0,  249,   90,   90,
   90,    0,  176,    0,    0,    0,    0,    0,  238,  239,
  240,  241,  242,    0,    0,  238,  239,  240,  241,  242,
  328,    0,    0,    0,  190,    0,    0,  192,  193,  290,
  173,  171,    0,  172,    0,  174,  226,  227,  228,  238,
  239,  240,  241,  242,    0,    0,  238,  239,  240,  241,
  242,  315,  173,  171,    0,  172,    0,  174,    0,    0,
    0,    0,    0,   95,   95,   95,   95,   95,  207,  207,
  321,  173,  171,    0,  172,    0,  174,   85,   85,   85,
   85,   85,  233,    0,  235,    0,    0,    0,  236,  237,
  348,  173,  171,    0,  172,    0,  174,    0,    0,  239,
  240,  241,  242,  352,  173,  171,  261,  172,    0,  174,
  240,  241,  242,    0,    1,    2,    3,    0,    4,    5,
    6,    7,    8,    9,   10,   11,   12,    0,   13,   14,
   15,   16,  241,  242,    0,    0,    0,    0,    0,    0,
   17,   18,   19,    0,    0,   20,   21,   22,   23,   24,
   25,   26,   27,   28,    0,    0,   29,    0,   30,  300,
   31,    0,    0,  302,  303,  304,  305,  306,  307,  308,
  309,  310,  311,  312,  313,  359,  173,  171,    0,  172,
    0,  174,    0,    0,  317,  184,    2,    3,    0,    4,
    5,    6,    7,    8,    9,   10,   11,   12,    0,   13,
   14,   15,   16,    0,    0,    0,    0,    0,    0,    0,
    0,   17,   18,   19,    0,    0,   20,   21,   22,   23,
   24,   25,   26,   27,   28,    0,    0,   29,    0,   30,
    0,   31,
};
const short yycheck[] = {                                      18,
    0,    0,    0,    0,    0,   44,    0,    2,    0,    0,
  259,    0,    7,  195,    9,   44,   11,  120,  121,    0,
    0,    0,    0,    0,    0,    0,   21,   22,   23,   24,
    0,   26,    6,   28,    0,   41,  120,  259,   12,   34,
    0,   15,    0,   42,   43,   44,   20,   46,   44,   41,
   44,   43,   44,   45,   43,   44,   30,   31,    0,    0,
   34,    0,   43,   44,   44,   44,   44,   44,   60,   61,
   62,   41,    0,   43,   44,   45,    0,   72,   44,    0,
   99,  100,    0,   43,    0,   43,    0,    0,    0,    0,
   60,   61,   62,  112,    0,    0,  115,  116,   93,   73,
    0,   43,   43,    0,    0,   44,    0,    0,  127,  104,
    0,    0,    0,    0,   88,    0,    0,    0,    0,    0,
  256,   41,   44,   59,  143,   60,   41,   62,  297,   91,
  149,  126,  301,   40,  129,  130,   43,   44,   40,   42,
   44,  323,  137,   45,   47,   40,   61,  269,  167,   40,
   45,   59,  171,  172,  173,  174,  175,   93,   59,  120,
   40,  297,  157,  299,  159,   45,   40,  186,  163,  164,
   40,   93,  256,   40,   40,  297,   41,  299,  152,   40,
   40,   40,   40,  281,   41,   93,  181,   44,   41,   93,
   41,   44,   93,   44,  189,   60,   61,   62,  296,  297,
   42,  299,   61,  198,  199,   47,   41,   42,   43,  297,
   45,   41,   47,  208,  188,  210,  300,   41,   42,   43,
   42,   45,  273,   47,   46,   60,   61,   62,  269,   42,
   60,   61,   62,   46,  273,   40,   60,   61,   62,  234,
   93,  260,   40,  238,  239,  240,  241,  242,  243,  244,
  245,  246,  247,  248,  249,   80,  275,  286,   43,   44,
  120,   41,  288,  289,  259,   60,   61,   62,  274,  269,
  269,  269,  269,  269,  273,  269,   43,  269,  269,  273,
  269,   44,  274,  275,  276,  277,  278,  286,  269,  269,
  269,  269,  269,  269,  269,  256,   41,  316,   43,  269,
   45,  320,   40,  269,  274,  275,  276,  277,  278,  269,
   41,  269,  297,   41,  299,   60,   61,   62,   41,  144,
   43,  285,  147,  148,  297,   44,  299,  269,  269,   41,
  269,  350,   60,   61,   62,  330,  198,  199,  357,  300,
  314,  269,  277,  278,   40,  269,   41,   61,  269,   61,
  297,  269,  299,  269,  274,  269,  269,  269,  269,  274,
  275,  276,   40,  269,  269,   60,   61,   62,   40,  269,
  324,  325,  269,  269,   40,  269,  269,  279,   40,  269,
  269,  269,  269,   44,  269,  269,  269,  269,  269,  214,
  297,   44,  299,   40,  296,  297,  256,  299,  300,  279,
  302,  296,  297,  273,  299,  300,  297,  302,  299,  274,
  275,  276,  277,  278,   41,   43,  296,  297,   91,  299,
  300,  273,  302,  297,  120,  299,   91,  297,  300,  299,
  296,  297,  120,  299,   61,  296,  297,   41,  299,  274,
  275,  276,  277,  278,  274,  275,  276,  277,  278,   44,
  274,  275,  276,  277,  278,   41,   60,   61,   62,  284,
  285,  286,  287,  288,  289,   41,   42,   43,  300,   45,
   44,   47,   41,   42,   43,   61,   45,   93,   47,   93,
  275,  276,  277,  278,   60,   61,   62,  296,  297,   40,
  299,   60,   61,   62,  274,  275,   41,   42,   43,   40,
   45,  297,   47,   41,   42,   43,  296,   45,   40,   47,
   60,   61,   62,   44,  339,   60,   61,   62,  296,  297,
   41,  299,   60,   61,   62,  296,  297,   43,  299,  274,
  275,  276,  277,  278,   41,  272,   43,   40,   45,   60,
   61,   62,   61,  274,  275,   44,  274,  275,  276,  277,
  278,   44,  298,   60,   61,   62,   42,   43,  154,   45,
  256,   47,  274,  275,  276,   93,   42,   43,  256,   45,
   61,   47,  138,   44,   60,   61,   62,   44,  144,  274,
  275,  276,  277,  278,   60,   61,   62,   42,   43,   93,
   45,  189,   47,   42,   43,    7,   45,    9,   47,   11,
  288,  289,  276,  200,  197,   60,  339,   62,  281,   21,
   22,  147,  300,   41,   42,   43,   28,   45,  285,   47,
  210,  281,  119,  296,  297,   -1,  299,   -1,  224,  225,
  226,  227,  228,  229,  230,  231,   41,   42,   43,   44,
   45,   -1,   47,   -1,   -1,   -1,   -1,  274,  275,  276,
   41,   42,   43,   -1,   45,   -1,   47,   41,   42,   43,
   -1,   45,   -1,   47,   42,   43,   44,   45,   -1,   47,
  274,  275,  276,  277,  278,   41,   42,   43,   -1,   45,
   -1,   47,   42,   43,   -1,   45,   -1,   47,  274,  275,
  276,   -1,  104,   -1,   -1,   -1,   -1,   -1,  274,  275,
  276,  277,  278,   -1,   -1,  274,  275,  276,  277,  278,
  276,   -1,   -1,   -1,  126,   -1,   -1,  129,  130,   41,
   42,   43,   -1,   45,   -1,   47,  276,  277,  278,  274,
  275,  276,  277,  278,   -1,   -1,  274,  275,  276,  277,
  278,   41,   42,   43,   -1,   45,   -1,   47,   -1,   -1,
   -1,   -1,   -1,  274,  275,  276,  277,  278,  324,  325,
   41,   42,   43,   -1,   45,   -1,   47,  274,  275,  276,
  277,  278,  157,   -1,  159,   -1,   -1,   -1,  163,  164,
   41,   42,   43,   -1,   45,   -1,   47,   -1,   -1,  275,
  276,  277,  278,   41,   42,   43,  181,   45,   -1,   47,
  276,  277,  278,   -1,  256,  257,  258,   -1,  260,  261,
  262,  263,  264,  265,  266,  267,  268,   -1,  270,  271,
  272,  273,  277,  278,   -1,   -1,   -1,   -1,   -1,   -1,
  282,  283,  284,   -1,   -1,  287,  288,  289,  290,  291,
  292,  293,  294,  295,   -1,   -1,  298,   -1,  300,  234,
  302,   -1,   -1,  238,  239,  240,  241,  242,  243,  244,
  245,  246,  247,  248,  249,   41,   42,   43,   -1,   45,
   -1,   47,   -1,   -1,  259,  256,  257,  258,   -1,  260,
  261,  262,  263,  264,  265,  266,  267,  268,   -1,  270,
  271,  272,  273,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  282,  283,  284,   -1,   -1,  287,  288,  289,  290,
  291,  292,  293,  294,  295,   -1,   -1,  298,   -1,  300,
   -1,  302,
};
#define YYFINAL 32
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 303
#if YYDEBUG
const char * const yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,"'('","')'","'*'","'+'","','","'-'","'.'","'/'",0,0,0,0,0,0,0,0,0,0,
0,"';'","'<'","'='","'>'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,"'['",0,"']'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'x'","'y'",
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"FILEC","MARKER","LOCUS","TRAIT","RANDOM",
"PEDIGREE","LOG","MODEL","FILTER","LINK","MISSING","FACTOR","BREAK","DOLOOP",
"WHILE","USE","WHERE","ORSYMBOL","ANDSYMBOL","NEQSYMBOL","LEQSYMBOL",
"GEQSYMBOL","NOTSYMBOL","LOGICAL","SHELL","ARRAY","PRINTEXP","INCLUDE",
"LOOP_CLAUSE_START","LOOP_CLAUSE_END","CONSTANT","RSFORMAT","FSFORMAT",
"CENSORED","GROUP","SET","GENDER","AFFECTED","OUTPUT","STRING","VARIABLE",
"ASSIGN","ARRAY_VAR","INTEGER","SYSTEM_VAR","REAL","UMINUS",
};
const char * const yyrule[] = {
"$accept : comfile",
"comfile : command",
"comfile : error",
"comfile : comfile BREAK",
"comfile : comfile BREAK command",
"comfile : comfile BREAK error",
"command : filecommand",
"command : filtercommand",
"command : includecommand",
"command : logcommand",
"command : pedcommand",
"command : locicommand",
"command : rancommand",
"command : linkcommand",
"command : modelcommand",
"command : missingcommand",
"command : factorcommand",
"command : usecommand",
"command : wherecommand",
"command : assigncommand",
"command : realcommand",
"command : integercommand",
"command : arraycommand",
"command : printcommand",
"command : docommand",
"command : whilecommand",
"command : constantcommand",
"command : censorcommand",
"command : affectedcommand",
"command : groupcommand",
"command : setcommand",
"command : defformatcommand",
"command : sexcommand",
"command : outputcommand",
"assigncommand : assignment",
"assignment : ASSIGN '=' expression",
"assignment : ASSIGN '(' expression ')' '=' expression",
"expression : REAL",
"expression : INTEGER",
"expression : STRING",
"expression : single_element",
"expression : expression '+' expression",
"expression : expression '-' expression",
"expression : expression '*' expression",
"expression : expression '/' expression",
"expression : '-' expression",
"expression : '(' expression ')'",
"setcommand : SET SYSTEM_VAR INTEGER",
"setcommand : SET VARIABLE INTEGER",
"docommand : DOLOOP",
"$$1 :",
"includecommand : INCLUDE $$1 complex_string",
"censorcommand : CENSORED single_element WHERE '(' res_condition ')'",
"affectedcommand : AFFECTED WHERE '(' res_condition ')'",
"sexcommand : GENDER single_element expression ',' expression",
"whilecommand : WHILE '(' condition ')'",
"condition : expression",
"condition : condition '=' condition",
"condition : condition NEQSYMBOL condition",
"condition : condition LEQSYMBOL condition",
"condition : condition GEQSYMBOL condition",
"condition : condition '<' condition",
"condition : condition '>' condition",
"condition : condition ORSYMBOL condition",
"condition : condition ANDSYMBOL condition",
"condition : NOTSYMBOL condition",
"printcommand : PRINTEXP printlist",
"printlist : expression",
"printlist : printlist ',' expression",
"defformatcommand : RSFORMAT complex_string",
"defformatcommand : FSFORMAT complex_string",
"arraycommand : ARRAY arraylist",
"arraylist : VARIABLE '(' expression ')'",
"arraylist : arraylist ',' VARIABLE '(' expression ')'",
"realcommand : REAL varlist",
"integercommand : INTEGER varlist",
"usecommand : USE varlist WHERE '(' res_condition ')'",
"usecommand : USE WHERE '(' res_condition ')'",
"wherecommand : WHERE '(' res_condition ')' USE varlist",
"wherecommand : WHERE '(' res_condition ')'",
"res_condition : INTEGER",
"res_condition : REAL",
"res_condition : STRING",
"res_condition : single_element",
"res_condition : res_condition '+' res_condition",
"res_condition : res_condition '-' res_condition",
"res_condition : res_condition '*' res_condition",
"res_condition : res_condition '/' res_condition",
"res_condition : '(' res_condition ')'",
"res_condition : '-' res_condition",
"res_condition : res_condition '=' res_condition",
"res_condition : res_condition NEQSYMBOL res_condition",
"res_condition : res_condition LEQSYMBOL res_condition",
"res_condition : res_condition GEQSYMBOL res_condition",
"res_condition : res_condition '<' res_condition",
"res_condition : res_condition '>' res_condition",
"res_condition : res_condition ORSYMBOL res_condition",
"res_condition : res_condition ANDSYMBOL res_condition",
"res_condition : NOTSYMBOL res_condition",
"filecommand : FILEC filename_string ',' filevarlist",
"filecommand : FILEC '[' formatlist ']' filename_string ',' filevarlist",
"filecommand : FILEC '[' fformatlist ']' filename_string ',' filevarlist",
"filename_string : complex_string",
"filename_string : SHELL '(' complex_string ')'",
"formatlist : format",
"formatlist : formatlist ',' format",
"formatlist : INTEGER '(' formatlist ')'",
"formatlist : formatlist ',' INTEGER '(' formatlist ')'",
"fformatlist : fformat",
"fformatlist : fformatlist ';' fformat",
"fformat : FSFORMAT complex_string",
"fformat : RSFORMAT complex_string",
"format : INTEGER",
"format : INTEGER 'x'",
"format : 'x'",
"format : INTEGER error",
"format : error",
"logcommand : LOG complex_string",
"outputcommand : OUTPUT complex_string",
"missingcommand : MISSING complex_string",
"missingcommand : MISSING complex_string varlist",
"missingcommand : MISSING complex_string ',' varlist",
"pedcommand : PEDIGREE varlist",
"locicommand : MARKER LOCUS locuslist",
"locicommand : TRAIT LOCUS varlist",
"rancommand : RANDOM varlist",
"constantcommand : CONSTANT varlist",
"factorcommand : FACTOR varlist",
"linkcom1 : LINK",
"linkcom1 : LINK '[' 'x' ']'",
"linkcom1 : LINK '[' 'y' ']'",
"linkcommand : linkcom1 varlist",
"linkcommand : linkcom1 complex_string1 ',' varlist",
"linkcommand : linkcom1 complex_string1",
"filtercommand : FILTER complex_string",
"modelcommand : MODEL VARIABLE '=' modellist",
"modelcommand : MODEL ARRAY_VAR '(' expression ')' '=' modellist",
"groupcommand : GROUP single_element",
"modellist : modellist '+' single_vlist",
"modellist : modellist '+' interactionlist",
"modellist : interactionlist",
"modellist : single_vlist",
"interactionlist : single_vlist '*' single_vlist",
"interactionlist : single_vlist '.' single_vlist",
"interactionlist : interactionlist '*' single_vlist",
"interactionlist : interactionlist '.' single_vlist",
"single_vlist : VARIABLE",
"single_vlist : ARRAY_VAR",
"single_vlist : ARRAY_VAR '(' expression ')'",
"single_element : VARIABLE",
"single_element : ARRAY_VAR '(' expression ')'",
"locuslist : locus",
"locuslist : locuslist ',' locus",
"locus : single_element lociclause",
"lociclause : '[' single_element ']'",
"lociclause : '[' single_element ',' single_element ']'",
"lociclause : '[' error ']'",
"simple_varlist : single_vlist",
"simple_varlist : simple_varlist ',' single_vlist",
"open_bracket : '('",
"loop_clause1 : open_bracket simple_varlist ',' BREAK assignment ',' expression ')'",
"loop_clause1 : open_bracket simple_varlist ',' BREAK assignment ',' expression ',' expression ')'",
"loop_clause : loop_clause1 LOOP_CLAUSE_START simple_varlist LOOP_CLAUSE_END",
"loop_clause : loop_clause1",
"fsingle_vlist :",
"fsingle_vlist : VARIABLE",
"fsingle_vlist : ARRAY_VAR",
"fsingle_vlist : ARRAY_VAR '(' expression ')'",
"filevarlist : fsingle_vlist",
"filevarlist : loop_clause",
"filevarlist : filevarlist ',' fsingle_vlist",
"filevarlist : filevarlist ',' loop_clause",
"varlist : single_vlist",
"varlist : loop_clause",
"varlist : varlist ',' single_vlist",
"varlist : varlist ',' loop_clause",
"complex_string1 : STRING",
"complex_string1 : complex_string1 '+' STRING",
"complex_string1 : single_element '+' complex_string1",
"complex_string : STRING",
"complex_string : single_element",
"complex_string : complex_string '+' STRING",
"complex_string : complex_string '+' single_element",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
int yystacksize;
#line 440 "control_parse.y"

static void enter_loop(void)
{	
	if(loop_level<MAX_LOOP)	{
		loop_stat[loop_level]=loop_record;
		loop_ptr[loop_level]=loop_main_ptr;
		loop_level++;
		if(!loop_record) loop_record=1;
	} else yyerror("Too many nested loops\n");
}

static void start_loopclause(void)
{	
	if(loop_level<MAX_LOOP)	{
		loop_stat[loop_level]=loop_record;
		loop_ptr[loop_level]=loop_main_ptr;
		in_loopclause=1;
		loop_level++;
		if(!loop_record) loop_record=1;
	} else yyerror("Too many nested loops\n");
}

static void begin_looping(struct var_element *element, struct express *exp1, struct express *exp2)
{	
	int er=0,i;
	
	if(element && exp1) {
		if(exp1->type==ST_INTEGER) loop_clause_end=(int)exp1->arg.value;
		else er=1;
		if(!er && exp2) {
			if(exp2->type==ST_INTEGER) loop_clause_step=(int)exp2->arg.value;
			else er=1;
		} else loop_clause_step=1;
		if(element->type&ST_INTEGER) {
			i=(int)element->arg.value;
			if(loop_clause_step<0) {
				if(i<loop_clause_end) er= -1;
			} else if(i>loop_clause_end) er= -1;
		} else er=1;
	} else er=2;
	if(er) {
		switch(er) {
		 case 1:
			yyerror("Loop variable not integer type\n");
			break;
		 case 2:
			yyerror("Syntax error\n");
			break;
		}
		loop_record=loop_stat[--loop_level];
		if(!loop_record) loop_main_ptr=loop_level?loop_ptr[loop_level-1]:0;
		in_loopclause=0;
	} else {
		in_loopclause= -1;
		loop_record= -1;
		loop_clause_ptr=loop_main_ptr;
		loop_clause_element=element;
		loop_main_ptr=loop_ptr[loop_level-1];
	}
	if(exp1) free(exp1);
	if(exp2) free(exp2);
}

static int if_true(struct express *express)
{
	int l=0;
	
	if(express)	{
		switch(express->type) {
		 case ST_INTEGER:
			l=(express->arg.value!=0);
			break;
		 case ST_REAL:
			l=(express->arg.rvalue!=0.0);
			break;
		 case ST_STRING:
			if(express->arg.string && express->arg.string[0]) l=1;
		}
	}
	return l;
}

static void do_while_com(struct express *express)
{
	int i;
	
	if(loop_level)	{
		if(!scan_error_n && if_true(express)) {
			loop_record= -1;
			loop_main_ptr=loop_ptr[loop_level-1];
		} else {
			loop_record=loop_stat[--loop_level];
			if(!loop_record) {
				i=loop_main_ptr-1;
				loop_main_ptr=loop_level?loop_ptr[loop_level-1]:0;
				for(;i>=loop_main_ptr;i--)	{
					if(loop_stack[i].token==STRING) free(loop_stack[i].yylval.string);
				}
			}
		}
	} else yyerror("WHILE outside of do loop\n");
}

static void print_exp(struct express *express)
{
	if(!express) return;
	switch(express->type) {
	 case ST_STRING:
		printf("%s",express->arg.string);
		free(express->arg.string);
		break;
	 case ST_INTEGER:
		printf("%ld",express->arg.value);
		break;
	 case ST_REAL:
		printf("%g",express->arg.rvalue);
		break;
	}
}

static void set_sex(struct var_element *elem,struct express *exp1,struct express *exp2)
{
	if(!exp1 || !exp2) yyerror1("Null arguments to sex command\n");
	else if(sex_elem) yyerror1("Error: Multiple sex commands");
	else {
		if(exp1->type != exp2->type) yyerror1("Arguments to sex command of different type\n");
		else if(exp1->type!=ST_INTEGER && exp1->type!=ST_STRING) yyerror1("Arguments to sex command of invalid type\n");
		else {
			sex_elem=elem;
			sex_exp[0]=exp1;
			sex_exp[1]=exp2;
			sex_elem->type|=(ST_SEX|ST_FACTOR|ST_CONSTANT);
			if(exp1->type==ST_INTEGER) sex_elem->type|=ST_INTTYPE;
		}
	}
}

static void set_group(struct var_element *elem)
{
	if(group_elem) yyerror1("Error: Multiple group commands");
	else {
		group_elem=elem;
		group_elem->type|=(ST_GROUP|ST_FACTOR|ST_CONSTANT);
	}
}

static void do_ped_com(struct var_list *vlist)
{
	int i,j,n;
	struct var_list *vlist1;
	
	if(pedflag) {
		yyerror1("Error: Multiple pedigree commands");
		scan_error|=PED_ERR;
	}
	pedflag=1;
	n=0;
	while(vlist) {
		if(vlist->var->vtype&ST_ARRAY) {
			if(vlist->index) {
				if(n<4) pedlist[n]=vlist->var->element+vlist->index-1;
				n++;
			} else for(i=0;i<vlist->var->n_elements;i++)	{
				if(n<4) pedlist[n]=vlist->var->element+i;
				n++;
			}
		} else {
			if(n<4) pedlist[n]=vlist->var->element;
			n++;
		}
		vlist1=vlist->next;
		free(vlist);
		vlist=vlist1;
	}
	if(n!=3 && n!=4) {
		yyerror1("Error: Wrong no. variables for pedigree command (3 or 4 required)");
		scan_error|=PED_ERR;
	} else {
		for(i=1;i<n;i++) {
			for(j=0;j<i;j++) if(pedlist[i]==pedlist[j]) {
				yyerror1("Error: Repeated variables in pedigree command");
				scan_error|=PED_ERR;
			}
		}
		if(n==4) {
			pedlist[0]->type|=(ST_FAMILY|ST_FACTOR|ST_CONSTANT);
			family_id=1;
			i=1;
		} else i=0;
		pedlist[i++]->type|=(ST_ID|ST_FACTOR|ST_CONSTANT);
		pedlist[i++]->type|=(ST_SIRE|ST_FACTOR|ST_CONSTANT);
		pedlist[i++]->type|=(ST_DAM|ST_FACTOR|ST_CONSTANT);
	}
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "alloc_express"
static struct express *alloc_express(void)
{
	struct express *e;
	
	if(!(e=malloc(sizeof(struct express)))) ABT_FUNC(MMsg);
	return e;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "do_logical_op"
static struct express *do_logical_op(struct express *ex1,struct express *ex2,int op)
{
	int i=0,l=0;
	double rv1,rv2;
	char *s1,*s2;
	
	s1=s2=0;
	if(ex1->type&ST_STRING) {
		s1=ex1->arg.string;
		i++;
	}
	if(ex2 && ex2->type&ST_STRING) {
		s2=ex2->arg.string;
		i++;
	}
	if(i==2)	{
		switch(op) {
		 case '=':
			l=mystrcmp(s1,s2)?0:1;
			break;
		 case NEQSYMBOL:
			l=mystrcmp(s1,s2)?1:0;
			break;
		 case '<':
			l=mystrcmp(s1,s2)<0?1:0;
			break;
		 case '>':
			l=mystrcmp(s1,s2)>0?1:0;
			break;
		 case LEQSYMBOL:
			l=mystrcmp(s1,s2)>=0?1:0;
			break;
		 case GEQSYMBOL:
			l=mystrcmp(s1,s2)>=0?1:0;
			break;
		 case ORSYMBOL:
			l=(s1 || s2);
			break;
		 case ANDSYMBOL:
			l=(s1 && s2);
			break;
		 default:
			ABT_FUNC("Internal error - invalid string op\n");
		}
	} else if(i && !ex2)	{
		if(op!=NOTSYMBOL) ABT_FUNC("Internal error - invalid unary string op\n");
		else l=s1?0:1;
	} else if(i) {
		switch(op) {
		 case ORSYMBOL:
			if(ex1->type&ST_STRING) l=(s1 || ex2->arg.value);
			else l=(s2 || ex1->arg.value);
			break;
		 case ANDSYMBOL:
			if(ex1->type&ST_STRING) l=(s1 && ex2->arg.value);
			else l=(s2 && ex1->arg.value);
			break;
		 default:
			ABT_FUNC("Internal error - invalid string op\n");
		}
	} else {
		rv1=rv2=0.0;
		if(ex1->type&ST_INTEGER) rv1=(double)ex1->arg.value;
		else if(ex1->type&ST_REAL) rv1=ex1->arg.rvalue;
		if(ex2) {
			if(ex2->type&ST_INTEGER) rv2=(double)ex2->arg.value;
			else if(ex2->type&ST_REAL) rv2=ex2->arg.rvalue;
		}
		switch(op) {
		 case '=':
			l=(rv1==rv2);
			break;
		 case NEQSYMBOL:
			l=(rv1!=rv2);
			break;
		 case '<':
			l=(rv1<rv2);
			break;
		 case '>':
			l=(rv1>rv2);
			break;
		 case LEQSYMBOL:
			l=(rv1<=rv2);
			break;
		 case GEQSYMBOL:
			l=(rv1>=rv2);
			break;
		 case ORSYMBOL:
			l=(rv1 || rv2);
			break;
		 case ANDSYMBOL:
			l=(rv1 && rv2);
			break;
		 case NOTSYMBOL:
			l=(rv1==0.0);
			break;
		 default:
			ABT_FUNC("Internal error - invalid op\n");
		}
	}
	if(ex2) free(ex2);
	ex1->type=ST_INTEGER;
	ex1->arg.value=l;
	return ex1;
}

static struct express *do_express_op(struct express *ex1,struct express *ex2,int op)
{
	double rv1,rv2;
	int i;
	
	if(ex1->type&ST_STRING)	{
		if(ex2 && ex2->type&ST_STRING) {
			if(op!='+') yyerror("Illegal string operation\n");
			else {
				ex1->arg.string=string_copy(ex1->arg.string,ex2->arg.string);
				free(ex2->arg.string);
			}
		} else yyerror("Can't mix numeric and string expressions\n");
	} else if(ex2 && ex2->type&ST_STRING) yyerror("Can't mix numeric and string expressions\n");
	else {
		rv1=rv2=0.0;
		if(ex1->type&ST_INTEGER) rv1=(double)ex1->arg.value;
		else if(ex1->type&ST_REAL) rv1=ex1->arg.rvalue;
		if(ex2) {
			if(ex2->type&ST_INTEGER) rv2=(double)ex2->arg.value;
			else if(ex2->type&ST_REAL) rv2=ex2->arg.rvalue;
		}
		switch(op) {
		 case '+': 
			rv1+=rv2;
			break;
		 case '-':
			if(ex2) rv1-=rv2;
			else rv1= -rv1;
			break;
		 case '*':
			rv1*=rv2;
			break;
		 case '/':
			if(rv2==0.0) {
				yyerror("Divide by zero error\n");
				rv1=0.0;
			} else rv1/=rv2;
			break;
		}
		i=(int)rv1;
		if((double)i==rv1) {
			ex1->type=ST_INTEGER;
			ex1->arg.value=i;
		} else {
			ex1->type=ST_REAL;
			ex1->arg.rvalue=rv1;
		}
	}
	if(ex2) free(ex2);
	return ex1;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "check_element_add_op"
static void check_element_add_op(struct var_element *element)
{
	switch(element->type&(ST_REAL|ST_INTEGER|ST_STRING)) {
	 case ST_STRING:
		add_operation(string_copy(0,element->arg.string),STRING,0);
		break;
	 case ST_INTEGER:
		add_operation(&element->arg,INTEGER,0);
		break;
	 case ST_REAL:
		add_operation(&element->arg,REAL,0);
		break;
	 case 0:
		add_operation(element,VARIABLE,0);
		break;
	 default:
		ABT_FUNC("Internal error - illegal element type\n");
	}
}

static int check_index(struct scan_var *var,struct express *express)
{
	int i;
	
	if(express->type!=ST_INTEGER) {
		if(in_loopclause<=0) yyerror("Non-integral expression for array index");
	} else if(var->vtype&ST_ARRAY) {
		i=(int)express->arg.value;
		if(i<1 || i>var->n_elements) {
			if(in_loopclause<=0) yyerror("Array index out of bounds");
		} else return i;
	} else	yyerror("Not an array");
	return 0;
}

static struct var_element *get_element(struct scan_var *var,struct express *express)
{
	int i;
	
	if(express) {
		if(!(i=check_index(var,express))) return 0;
		return var->element+i-1;
	} else {
		if(var->vtype&ST_ARRAY)	{
			yyerror("Illegal reference to array");
			return 0;
		}
	}
	return var->element;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "set_array_var"
static void set_array_var(struct scan_var *var,struct express *express)
{
	int i;
	
	if(express->type!=ST_INTEGER) yyerror("Non-integral expression for array size");
	else if((i=(int)express->arg.value)<1) yyerror("Illegal array size");
	else if(var->vtype) yyerror("Can't redefine variable");
	else {
		var->vtype|=ST_ARRAY;
		var->n_elements=i;
		free(var->element);
		if(!(var->element=calloc((size_t)var->n_elements,sizeof(struct var_element)))) ABT_FUNC(MMsg);
	}
}

static int count_var_list(struct var_list *vlist)
{
	int i=0;
	struct scan_var *var;
	
	while(vlist) {
		var=vlist->var;
		if(var && (var->vtype&ST_ARRAY) && !vlist->index) i+=var->n_elements;
		else i++;
		vlist=vlist->next;
	}
	return i;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "assign_var"
static struct var_element *assign_var(struct scan_var *var,struct express *ix,struct express *express)
{
	struct var_element *element;
	
	if(!express) return 0;
	if(!(element=get_element(var,ix))) return 0;
	switch(express->type) {
	 case ST_STRING:
	     element->arg.string=express->arg.string;
		RemBlock=AddRemem(element->arg.string,RemBlock);
		break;
	 case ST_REAL:
	 case ST_INTEGER:
	     element->arg=express->arg;
		break;
	 case 0:
	     yyerror1("Undefined assignment\n");
		element->type=0;
		element->arg.string=0;
		break;
	 default:
		ABT_FUNC(IntErr);
	}
	if(!ix) var->vtype|=ST_SCALAR;
	element->type=express->type;
	return element;
}

void check_vars(struct scan_var *var,int *i,void check_func(struct scan_var *,int *))
{
	if(var->var_left) {
		check_vars(var->var_left,i,check_func);
	}
	check_func(var,i);
	if(var->var_right) {
		check_vars(var->var_right,i,check_func);
	}
}

static void check_vars_1(struct scan_var *var,void check_func(struct scan_var *))
{
	if(var->var_left) {
		check_vars_1(var->var_left,check_func);
	}
	check_func(var);
	if(var->var_right) {
		check_vars_1(var->var_right,check_func);
	}
}

void print_scan_err(char *fmt, ...)
{
	va_list args;
	
	va_start(args,fmt);
	vfprintf(stderr,fmt,args);
	va_end(args);
	if((++scan_error_n)>=max_scan_errors) abt(__FILE__,__LINE__,"Too many errors - aborting\n");
}

void print_scan_warn(char *fmt, ...)
{
	va_list args;
	
	if(scan_warn_n<max_scan_warnings) {
		va_start(args,fmt);
		vfprintf(stderr,fmt,args);
		va_end(args);
	}
	scan_warn_n++;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "add_operation"
static void add_operation(void *arg,int type, int op)
{
	struct operation *o;
	
	if(!arg && type) ABT_FUNC(IntErr);
	if(!(o=malloc(sizeof(struct operation)))) ABT_FUNC(MMsg);
	o->next=Op_List;
	Op_List=o;
	o->type=type;
	o->op=op;
	switch(type) {
	 case VARIABLE:
		o->arg.element= (struct var_element *)arg;
		break;
	 case INTEGER:
		o->arg.value= *(int *)arg;
		break;
	 case REAL:
		o->arg.rvalue= *(double *)arg;
		break;
	 case STRING:
		o->arg.string= (char *)arg;
		break;
	}
}

static void new_command(void)
{
	shell_flag=in_loopclause=0;
	Op_List=0;
	iflag=0;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "add_to_model"
static struct model_list *add_to_model(struct model_list *model,struct var_list *vlist)
{
	struct var_list *vlist1;
	struct scan_var *var;
	struct model_list *m1;
	int i;
	
	if(!(m1=malloc(sizeof(struct model_list)))) ABT_FUNC(MMsg);
	if(vlist) {
		i=count_var_list(vlist);
		if(!(m1->element=malloc(sizeof(void *)*i))) ABT_FUNC(MMsg);
		i=0;
		while(vlist) {
			var=vlist->var;
			if(var->vtype&ST_ARRAY)	{
				if(vlist->index) {
					var->element[vlist->index-1].type|=ST_MODEL;
					var->element[vlist->index-1].index=vlist->index;
					var->element[vlist->index-1].oindex=vlist->index;
					m1->element[i++]=var->element+vlist->index-1;
				} else yyerror("Error - Can't use whole arrays as model parameters");
			} else {
				var->element[0].type|=ST_MODEL;
				var->element[0].index=0;
				m1->element[i++]=var->element;
			}
			vlist1=vlist->next;
			free(vlist);
			vlist=vlist1;
		}
		m1->nvar=i;
	} else ABT_FUNC("Nothing to add...\n");
	m1->next=model;
	return m1;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "string_copy"
static char *string_copy(char *s1,char *s2)
{
	if(s1) {  
		if(!(s1=realloc(s1,strlen(s1)+strlen(s2)+1))) ABT_FUNC(MMsg);
		strcat(s1,s2);
	} else {
		if(!(s1=malloc(strlen(s2)+1))) ABT_FUNC(MMsg);
		strcpy(s1,s2);
	}
	return s1;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "setup_format"
static struct format *setup_format(struct format_clause *fc)
{
	int i,n=0,pp=0;
	struct format_atom **fa;
	struct format *format;
	
	fa=fc->f_atoms;
	for(i=0;i<fc->n_atoms;i++) if(!fa[i]->pos) n++;
	if(!n) {
		if(!(scan_error&FORMAT_ERR)) yyerror("Error - Empty format clause");
		free(fa);
		free(fc);
		scan_error|=FORMAT_ERR;
		scan_error_n++;
		return 0;
	}
	if(!(format=malloc(sizeof(struct format)))) ABT_FUNC(MMsg);
	format->line=lineno;
	if(!(format->f_atoms=malloc(sizeof(struct format_atom)*n))) ABT_FUNC(MMsg);
	for(i=n=0;i<fc->n_atoms;i++) {
		if(!fa[i]->pos) {
			format->f_atoms[n].size=fa[i]->size;
			format->f_atoms[n++].pos=pp;
		}
		pp+=fa[i]->size;
	}
	free(fa);
	format->n_atoms=n;
	f_atom_n=0;
	free(fc);
	return format;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "make_f_atom"
static struct format_atom *make_f_atom(int n,int flag)
{
	if(f_atom_n>=f_atom_size) {
		f_atom_size*=2;
		if(!(f_atom_list=realloc(f_atom_list,sizeof(struct format_atom)*f_atom_size))) ABT_FUNC(MMsg);
	}
	f_atom_list[f_atom_n].size=n;
	f_atom_list[f_atom_n].pos=flag;
	return &f_atom_list[f_atom_n++];
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "add_fformat"
static struct fformat *add_fformat(struct fformat *f1,struct fformat *f2)
{
	if(f2->rs) {
		if(f1->rs) free(f1->rs);
		f1->rs=f2->rs;
	}
	if(f2->fs) {
		if(f1->fs) free(f1->fs);
		f1->fs=f2->fs;
	}
	free(f2);
	return f1;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "create_fformat"
static struct fformat *create_fformat(char *s,int fg)
{
	struct fformat *ff;
	
	if(!(ff=malloc(sizeof(struct fformat)))) ABT_FUNC(MMsg);
	ff->rs=ff->fs=0;
	if(fg==1) ff->rs=s;
	else if(fg==2) ff->fs=s;
	else ABT_FUNC("Internal error - incorrect flag\n");
	return ff;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "add_f_atom"
static struct format_clause *add_f_atom(struct format_clause *fc,struct format_atom *fa)
{
	if(!fc) {
		if(!(fc=malloc(sizeof(struct format_clause)))) ABT_FUNC(MMsg);
		fc->fc_size=16;
		fc->n_atoms=0;
		if(!(fc->f_atoms=malloc(sizeof(struct format_atom *)*fc->fc_size))) ABT_FUNC(MMsg);
	}
	if(fc->n_atoms>=fc->fc_size) {
		fc->fc_size*=2;
		if(!(fc->f_atoms=realloc(fc->f_atoms,sizeof(struct format_atom *)*fc->fc_size))) ABT_FUNC(MMsg);
	}
	fc->f_atoms[fc->n_atoms++]=fa;
	return fc;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "add_f_list"
static struct format_clause *add_f_list(struct format_clause *fc,struct format_clause *fc1,int n)
{
	int sz,i,j;
	
	sz=fc1->n_atoms*n;
	if(!fc) {
		if(!(fc=malloc(sizeof(struct format_clause)))) ABT_FUNC(MMsg);
		fc->fc_size=16;
		if(sz>16) fc->fc_size=sz;
		fc->n_atoms=0;
		if(!(fc->f_atoms=malloc(sizeof(struct format_atom *)*fc->fc_size))) ABT_FUNC(MMsg);
	} else {
		if(sz>(fc->fc_size-fc->n_atoms)) {
			fc->fc_size=sz+fc->n_atoms;
			if(!(fc->f_atoms=realloc(fc->f_atoms,sizeof(struct format_atom *)*fc->fc_size))) ABT_FUNC(MMsg);
		}
	}
	for(i=0;i<n;i++) for(j=0;j<fc1->n_atoms;j++)
	  fc->f_atoms[fc->n_atoms++]=fc1->f_atoms[j];
	free(fc1->f_atoms);
	free(fc1);
	return fc;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "alloc_var"
static struct scan_var *alloc_var(char *p)
{
	struct scan_var *v;
	int i;
	
	if(!(v=malloc(sizeof(struct scan_var)))) ABT_FUNC(MMsg);
	v->var_left=v->var_right=(struct scan_var *)0;
	v->vtype=0;
	i=(int)strlen(p);
	if(!(v->name=malloc((size_t)i+1))) ABT_FUNC(MMsg);
	v->name[i--]=0;
	for(;i>=0;i--) v->name[i]=toupper((int)p[i]);
	v->n_elements=1;
	if(!(v->element=calloc(1,sizeof(struct var_element)))) ABT_FUNC(MMsg);
	v->element->arg.element=0;
	v->balance=0;
	return v;
}

static struct scan_var *rotate_left(struct scan_var *var)
{
	struct scan_var *l,*gc;
	
	l=var->var_left;
	if(l->balance==-1) {
		var->var_left=l->var_right;
		l->var_right=var;
		var->balance=l->balance=0;
		var=l;
	} else {
		gc=l->var_right;
		l->var_right=gc->var_left;
		gc->var_left=l;
		var->var_left=gc->var_right;
		gc->var_right=var;
		switch(gc->balance) {
		 case -1:
			var->balance=1;
			l->balance=0;
			break;
		 case 0:
			var->balance=l->balance=0;
			break;
		 case 1:
			var->balance=0;
			l->balance=-1;
		}
		gc->balance=0;
		var=gc;
	}
	return var;
}

static struct scan_var *rotate_right(struct scan_var *var)
{
	struct scan_var *r,*gc;
	
	r=var->var_right;
	if(r->balance==1) {
		var->var_right=r->var_left;
		r->var_left=var;
		var->balance=r->balance=0;
		var=r;
	} else {
		gc=r->var_left;
		r->var_left=gc->var_right;
		gc->var_right=r;
		var->var_right=gc->var_left;
		gc->var_left=var;
		switch(gc->balance) {
		 case -1:
			var->balance=0;
			r->balance=1;
			break;
		 case 0:
			var->balance=r->balance=0;
			break;
		 case 1:
			var->balance=-1;
			r->balance=0;
		}
		gc->balance=0;
		var=gc;
	}
	return var;
}

static struct scan_var *find_var(char *p,struct scan_var *var,struct scan_var **var1,int *balanced)
{
	int i;

	if((i=strcasecmp(p,var->name))) {
		if(i<0) {
			if(var->var_left) {
				var->var_left=find_var(p,var->var_left,var1,balanced);
			} else {
				*var1=var->var_left=alloc_var(p);
				*balanced=0;
			}
			if(!(*balanced)) {
				switch(var->balance) {
				 case -1:
					var=rotate_left(var);
					*balanced=1;
					break;
				 case 0:
					var->balance=-1;
					break;
				 case 1:
					var->balance=0;
					*balanced=1;
				}
			}
		} else {
			if(var->var_right) {
				var->var_right=find_var(p,var->var_right,var1,balanced);
			} else {
				*var1=var->var_right=alloc_var(p);
				*balanced=0;
			}
			if(!(*balanced)) {
				switch(var->balance) {
				 case -1:
					var->balance=0;
					*balanced=1;
					break;
				 case 0:
					var->balance=1;
					break;
				 case 1:
					var=rotate_right(var);
					*balanced=1;
				}
			}
		}
	} else {
		*var1=var;
		*balanced=1;
	}
	return var;
}

void free_var(struct scan_var *var)
{
	if(var->var_left) free_var(var->var_left);
	if(var->var_right) free_var(var->var_right);
	if(var->name) free(var->name);
	if(var->element) free(var->element);
	free(var);
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "Check_var"
static void Check_var(struct scan_var *var)
{
	int i;
	struct var_element *element;
	char *nbuf;
	
	if(var->var_left) Check_var(var->var_left);
	i=strlen(var->name)+4+log((double)(var->n_elements+1))/log(10.0);
	if(!(nbuf=malloc((size_t)i))) ABT_FUNC(MMsg);
	for(i=0;i<var->n_elements;i++) {
		if(var->vtype&ST_ARRAY) sprintf(nbuf,"%s(%d)",var->name,i+1);
		else sprintf(nbuf,"%s",var->name);
		element=var->element+i;
		if(!(element->type&(ST_DATA|ST_TRAITLOCUS|ST_LINKED))) {
			if(element->type&(ST_MODEL|ST_SEX|ST_ID|ST_FAMILY|ST_SIRE|ST_DAM|ST_TRAIT|ST_GROUP))
			  print_scan_err("Error: No data for variable %s\n",nbuf);
		}
		if((element->type&ST_DATA) && (element->type&(ST_TRAITLOCUS|ST_MARKER)))
		  print_scan_err("Error: Variable %s can not have data\n",nbuf);
		if((element->type&ST_LINKED) && !(element->type&(ST_TRAITLOCUS|ST_MARKER)))
		  print_scan_err("Error: Variable %s is not a locus and so can not be linked\n",nbuf);
		if((element->type&ST_TRAIT) && (element->type&(ST_GROUP|ST_MARKER|ST_TRAITLOCUS|ST_ID|ST_FAMILY|ST_SIRE|ST_DAM|ST_HAPLO|ST_LINKED|ST_STRING|ST_REAL|ST_INTEGER)))
		  print_scan_err("Error: Variable %s inappropriate type for trait\n",nbuf);
		else if((element->type&ST_TRAITLOCUS) && (element->type&(ST_SEX|ST_GROUP|ST_CENSORED|ST_RANDOM|ST_MARKER|ST_ID|ST_FAMILY|ST_SIRE|ST_DAM|ST_HAPLO|ST_STRING|ST_REAL|ST_INTEGER|ST_REALTYPE|ST_INTTYPE)))
		  print_scan_err("Error: Variable %s inappropriate type for trait locus\n",nbuf);
		else if((element->type&ST_MARKER) && (element->type&(ST_SEX|ST_GROUP|ST_CENSORED|ST_REAL|ST_INTEGER|ST_RANDOM|ST_ID|ST_FAMILY|ST_SIRE|ST_DAM|ST_HAPLO|ST_STRING|ST_REAL|ST_INTEGER)))
		  print_scan_err("Error: Variable %s inappropriate type for marker\n",nbuf);
		else if((element->type&ST_HAPLO) && (element->type&(ST_SEX|ST_GROUP|ST_CENSORED|ST_REAL|ST_RANDOM|ST_ID|ST_FAMILY|ST_SIRE|ST_DAM|ST_STRING|ST_REAL|ST_INTEGER)))
		  print_scan_err("Error: Variable %s inappropriate type for haplotype\n",nbuf);
		else if((element->type&ST_RANDOM) && (element->type&(ST_SEX|ST_GROUP|ST_STRING|ST_REAL|ST_INTEGER|ST_REAL)))
		  print_scan_err("Error: Variable %s inappropriate type to be random\n",nbuf);
		else if((element->type&(ST_INTTYPE|ST_REALTYPE)) && (element->type&(ST_STRING|ST_REAL|ST_INTEGER)))
		  print_scan_err("Error: Type collision for variable %s\n",nbuf);
		else if((element->type&ST_INTTYPE) && (element->type&ST_REALTYPE))
		  print_scan_err("Error: Real variable %s can not also be integer type\n",nbuf);
		else if((element->type&(ST_STRING|ST_REAL|ST_INTEGER)) && (element->type&(ST_SEX|ST_ID|ST_FAMILY|ST_SIRE|ST_DAM)))
		  print_scan_err("Error: Variable %s can not be a pedigree or sex variable\n",nbuf);
		else if((element->type&ST_REAL) && (element->type&(ST_ID|ST_FAMILY|ST_SIRE|ST_DAM|ST_SEX)))
		  print_scan_err("Error: Real variable %s can not be a pedigree or sex variable\n",nbuf);
		else if((element->type&ST_FACTOR) && (element->type&ST_REAL))
		  print_scan_err("Error: Real variable %s can not be a factor\n",nbuf);
		else if(element->type&(ST_SEX|ST_ID|ST_FAMILY|ST_SIRE|ST_DAM|ST_TRAITLOCUS|ST_GROUP|ST_LINKED|ST_MODEL|ST_TRAIT))
		  element->type|=ST_REQUIRED;
		else if(element->type&ST_HAPLO) {
			if(element->arg.element && element->arg.element->type&ST_LINKED) {
				element->type|=ST_REQUIRED;
				if(!(element->type&ST_DATA)) print_scan_err("Error: No data for variable %s\n",nbuf);
			}
		}
		if((element->type&(ST_MARKER|ST_REQUIRED)) == (ST_MARKER|ST_REQUIRED)) n_markers++;
		if(element->type&(ST_REQUIRED|ST_RESTRICT))	{
			if(!(element->type&ST_HAPLO)) element->arg.var=var;
		} else element->type=0;
	}
	free(nbuf);
	if(var->var_right) Check_var(var->var_right);
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "symbol_lookup"
int symbol_lookup(char *p,int fg)
{
	static char *Coms[] = {"FILE","LOCUS","MARKER","DISCRETE","MODEL","PEDIGREE","LOG",
		"FILTER","MISSING","MODEL","LINK","RANDOM","TRAIT","WHERE","USE",
		"REAL","INTEGER","SHELL","ARRAY","PRINT","DO","WHILE","CONSTANT",
		"CENSORED","GROUP","SET","SEX","AFFECTED","OUTPUT","INCLUDE",(char *)0};
	static int Com_token[] = {FILEC,LOCUS,MARKER,FACTOR,MODEL,PEDIGREE,LOG,
		FILTER,MISSING,MODEL,LINK,RANDOM,TRAIT,WHERE,USE,
		REAL,INTEGER,SHELL,ARRAY,PRINTEXP,DOLOOP,WHILE,CONSTANT,
		CENSORED,GROUP,SET,GENDER,AFFECTED,OUTPUT,INCLUDE,
		SYSTEM_VAR,VARIABLE,ARRAY_VAR};
	static char *Syst[] = {"PRUNE_OPTION","RECODE_OPTION","NO_EXTRA_ALLELE",
		"PEEL_OPTION","TRACE_RESTRICT","TRACE_CENSORED","TRACE_AFFECTED",
		"CORRECT_ERRORS","TRACE_PEEL",(char *)0};
	int i=0,j=0,k;
	
	while(Coms[i])	{
		if(!strcasecmp(Coms[i],p)) break;
		i++;
	}
	at_file=0;
	if(Com_token[i]==FILEC || Com_token[i]==LINK) at_file=1;
	if(Com_token[i]==SYSTEM_VAR) {
		i++;
		while(Syst[j])	{
			if(!strcasecmp(Syst[j],p))	{
				yylval.value=j;
				i--;
				break;
			}
			j++;
		}
	}
	if(Com_token[i]==VARIABLE) {
		if(fg && begin_comm) {
			begin_comm=0;
			return BREAK;
		}
		if(!root_var) yylval.var=root_var=alloc_var(p);
		else {
			root_var=find_var(p,root_var,&yylval.var,&k);
		}
		if(yylval.var->vtype&ST_ARRAY) i++;
		if(fg) {
			begin_comm=1;
			strcpy(linebuf1,linebuf);
			lineno1=lineno;
		}
	} else if(begin_comm && Com_token[i]!=SYSTEM_VAR && Com_token[i]!=LOCUS && Com_token[i]!=SHELL 
				 && !(at_use==1 && Com_token[i]==WHERE) && !(at_use==2 && Com_token[i]==USE))	{
		begin_comm=0;
		at_use=0;
		return BREAK;
	} else {
		begin_comm=1;
		strcpy(linebuf1,linebuf);
		lineno1=lineno;
		if(Com_token[i]==MODEL) at_model=1;
		else at_model=0;
		if(Com_token[i]==USE || Com_token[i]==CENSORED || Com_token[i]==AFFECTED) at_use|=1;
		else if(Com_token[i]==WHERE) at_use|=2;
		else at_use=0;
	}
	return Com_token[i];
}

static struct var_list *add_to_var_list(struct var_list *vlist,struct scan_var *var,struct express *express)
{
	struct var_list *vlist1,*vlist2;
	int i;
	
	if(express) i=check_index(var,express);
	else {
		i=0;
		if(var && !(var->vtype&ST_ARRAY)) var->vtype|=ST_SCALAR;
	}
	if(!(vlist1=malloc(sizeof(struct var_list)))) ABT_FUNC(MMsg);
	vlist1->next=0;
	vlist1->var=var;
	vlist1->index=i;
	vlist2=vlist;
	if(vlist2) {
		while(vlist2->next) vlist2=vlist2->next;
		vlist2->next=vlist1;
	} else vlist=vlist1;
	return vlist;
}

struct var_list *add_var_lists(struct var_list *vlist,struct var_list *vlist1)
{
	struct var_list *vlist2;
	
	vlist2=vlist;
	if(vlist2) {
		while(vlist2->next) vlist2=vlist2->next;
		vlist2->next=vlist1;
	} else vlist=vlist1;
	return vlist;
}

static void set_locus_element(struct var_element *element)
{
	element->type|=ST_MARKER;
	if(hap_list[0]) {
		if(hap_list[0]->arg.element && hap_list[0]->arg.element!=element)	{
			yyerror1("Haplotype vector (left) used twice");
			hap_list[0]->arg.element=0;
		} else hap_list[0]->arg.element=element;
	}
	if(hap_list[1]) {
		if(hap_list[1]->arg.element && hap_list[1]->arg.element!=element)	{
			yyerror1("Haplotype vector (right) used twice");
			hap_list[1]->arg.element=0;
		} else hap_list[1]->arg.element=element;
	}
	hap_list[0]=hap_list[1]=0;
}

static void set_haplo_element(struct var_element *element,struct var_element *element1)
{
	element->type|=(ST_HAPLO|ST_FACTOR|ST_CONSTANT);
	if(element1) element1->type|=(ST_HAPLO|ST_FACTOR|ST_CONSTANT);
	hap_list[0]=element;
	hap_list[1]=element1;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "do_file_com"
static void do_file_com(char *fname,struct format_clause *fc,struct fformat *ff,struct var_list *vlist)
{
	int i,j;
	struct InFile *file;
	struct format *format;
	struct var_list *vlist1;
	struct var_element *element;
	
	if(!vlist) {
		yyerror1("No variables listed for FILE command\n");
		return;
	} else if(!fname) {
		free_vlist(vlist);
		return;
	} else if(!fname[0]) {
		yyerror1("Zero length filename for FILE command\n");
		free_vlist(vlist);
		return;
	}
	file=Infiles;
	if(!(Infiles=calloc(1,sizeof(struct InFile)))) ABT_FUNC(MMsg);
	Infiles->next=file;
	Infiles->nvar=count_var_list(vlist);
	if(!(Infiles->element=malloc(sizeof(void *)*Infiles->nvar))) ABT_FUNC(MMsg);
	i=0;
	while(vlist) {
		if(vlist->var) {
			if(vlist->var->vtype&ST_ARRAY) {
				if(vlist->index) {
					element=vlist->var->element+vlist->index-1;
					element->type|=ST_DATA;
					Infiles->element[i++]=element;
				} else {
					for(j=0;j<vlist->var->n_elements;j++) {
						element=vlist->var->element+j;
						element->type|=ST_DATA;
						Infiles->element[i++]=element;
					}
				}
			} else {
				element=vlist->var->element;
				element->type|=ST_DATA;
				Infiles->element[i++]=element;
			}
		} else Infiles->element[i++]=0;
		vlist1=vlist->next;
		free(vlist);
		vlist=vlist1;
	}
	if(fc) {
		format=setup_format(fc);
		Infiles->format=format;
		if(!(scan_error&FORMAT_ERR)) {
			if(format->n_atoms<i) {
				printf("format->n_atoms = %d\n",format->n_atoms);
				printf("i = %d\n",i);
				print_scan_err("Line %d: Error - Too many variables for format clause\n",format->line);
				scan_error|=FORMAT_ERR;
			} else if(format->n_atoms>i)
				  print_scan_warn("Line %d: Warning - Too few variables for format clause\n",format->line);
		}
	} else if(ff) Infiles->fformat=ff;
	Infiles->name=fname;
	Infiles->shell_flag=shell_flag;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "change_type"
static void change_type(int type,struct var_list *vlist)
{
	int j;

	while(vlist) {
		if(vlist->var->vtype&ST_ARRAY) {
			if(vlist->index) vlist->var->element[vlist->index-1].type|=type;
			else for(j=0;j<vlist->var->n_elements;j++)
			  vlist->var->element[j].type|=type;
		} else vlist->var->element[0].type|=type;
		vlist=vlist->next;
	}
}

static void free_vlist(struct var_list *vlist)
{
	struct var_list *vlist1;
	
	while(vlist) {
		vlist1=vlist->next;
		free(vlist);
		vlist=vlist1;
	}
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "do_link_com"
static void do_link_com(char *s,int type,struct var_list *vlist)
{
	struct Link *l,*l1;
	struct var_list *vlist1;
	struct var_element *element;
	int i,j;
	
	if(!vlist) ABT_FUNC("Zero var_list pointer\n");
	if(!(l1=malloc(sizeof(struct Link)))) ABT_FUNC(MMsg);
	l1->next=0;
	l=links;
	if(l) {
		while(l->next) l=l->next;
		l->next=l1;
	} else links=l1;
	if(!s) {
		if(vlist->var->vtype&ST_ARRAY && vlist->index) {
			element=vlist->var->element+vlist->index-1;
			if(element->type&ST_STRING) {
				s=element->arg.string;
				vlist1=vlist->next;
				free(vlist);
				vlist=vlist1;
			}
		} else {
			element=vlist->var->element;
			if(element->type&ST_STRING) {
				s=element->arg.string;
				vlist1=vlist->next;
				free(vlist);
				vlist=vlist1;
			}
		}
	}
	l1->name=s;
	if(!vlist) {
		if(s) print_scan_err("Error: No variable list for LINK command (%s)\n",s);
		else print_scan_err("Error: No variable list for LINK command\n");
		scan_error|=LINK_ERR;
		return;
	}
	i=count_var_list(vlist);
	l1->n_loci=i;
	l1->type=type;
	if(!(l1->element=malloc(sizeof(void *)*i))) ABT_FUNC(MMsg);
	i=0;
	while(vlist) {
		if(vlist->var->vtype&ST_ARRAY) {
			if(vlist->index) {
				element=vlist->var->element+vlist->index-1;
				if(element->type&ST_LINKED) {
					print_scan_err("Error: %s(%d) appears in multiple linkage groups\n",vlist->var->name,vlist->index);
					scan_error|=LINK_ERR;
				} else element->type|=ST_LINKED;
				l1->element[i++]=element;
			} else {
				if(vlist->var->vtype&ST_LINKED) {
					print_scan_err("Error: %s appears in multiple linkage groups\n",vlist->var->name);
					scan_error|=LINK_ERR;
				} else {
					for(j=0;j<vlist->var->n_elements;j++) {
						element=vlist->var->element+j;
						element->type|=ST_LINKED;
						l1->element[i++]=element;
					}
					vlist->var->vtype|=ST_LINKED;
				}
			}
		} else {
			element=vlist->var->element;
			if(element->type&ST_LINKED) {
				print_scan_err("Error: %s appears in multiple linkage groups\n",vlist->var->name);
				scan_error|=LINK_ERR;
			}
			element->type|=ST_LINKED;
			l1->element[i++]=element;
		}
		vlist1=vlist->next;
		free(vlist);
		vlist=vlist1;
	}
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "do_missing_com"
static void do_missing_com(char *s,struct var_list *vlist)
{
	struct var_list *vlist1;
	struct scan_var *var;
	struct Miss *m;
	int i,j;
	
	m=Miss;
	if(!(Miss=malloc(sizeof(struct Miss)))) ABT_FUNC(MMsg);
	Miss->Missing=s;
	Miss->next=m;
	if((i=count_var_list(vlist))) {
		if(!(Miss->element=malloc(sizeof(void *)*i))) ABT_FUNC(MMsg);
		i=0;
		while(vlist) {
			var=vlist->var;
			if(var->vtype&ST_ARRAY) {
				if(vlist->index) Miss->element[i++]=var->element+vlist->index-1;
				else for(j=0;j<var->n_elements;j++) Miss->element[i++]=var->element+j;
			} else Miss->element[i++]=var->element;
			vlist1=vlist->next;
			free(vlist);
			vlist=vlist1;
		}
	} else Miss->element=0;
	Miss->nvar=i;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "do_model_com"
static void do_model_com(struct model_list *mlist,struct scan_var *var,struct express *express)
{
	struct model *model;
	struct var_element *element;
	
	if(Models) {
		print_scan_err("Error: Mutiple models not currently supported\n");
		return;
	}
	if(!(model=malloc(sizeof(struct model)))) ABT_FUNC(MMsg);
     model->next=Models;
	Models=model;
	model->trait=var;
	if(!express) {
		model->index=0;
		var->element[0].type|=ST_TRAIT;
	} else {
		element=get_element(var,express);
		if(element) {
			model->index=(int)express->arg.value;
			element->index=element->oindex=model->index;
			element->type|=ST_TRAIT;
		} else model->trait=0;
	}
	model->model_list=mlist;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "add_censored"
static void add_censored(struct var_element *element,const int fg)
{
	struct operation *ops,*ops1,*ops2;
	struct Censor *cen;
	
	if(fg && !element) {
		print_scan_err("Error: Nothing to censor!\n");
		return;
	}
	ops=Op_List;
	/* Reverse list order (really return list to orignal order! */
	if(ops) {
		ops1=ops->next;
		while(ops1) {
			ops2=ops1->next;
			ops1->next=ops;
			ops=ops1;
			ops1=ops2;
		}
		Op_List->next=0;
		Op_List=ops;
	}
	if(fg) {
		if(!(cen=malloc(sizeof(struct Censor)))) ABT_FUNC(MMsg);
		cen->next=Censored;
		Censored=cen;
		cen->Op_List=ops;
		cen->element=element;
		element->type|=ST_CENSORED;
	} else {
		if(Affected) print_scan_warn("Warning - new affected statement overrules previous statement\n");
		Affected=ops;
	}
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "add_restriction"
static void add_restriction(struct var_list *vlist)
{
	struct operation *ops,*ops1,*ops2;
	struct Restrict *res;
	struct var_list *vlist1;
	int i,j;
	struct scan_var *var;
	
	ops=Op_List;
	/* Reverse list order (really return list to original order! */
	if(ops) {
		ops1=ops->next;
		while(ops1) {
			ops2=ops1->next;
			ops1->next=ops;
			ops=ops1;
			ops1=ops2;
		}
		Op_List->next=0;
		Op_List=ops;
	}
	if(!(res=malloc(sizeof(struct Restrict)))) ABT_FUNC(MMsg);
	res->next=Restrictions;
	Restrictions=res;
	res->Op_List=ops;
	if((res->nvar=count_var_list(vlist))) {
		if(!(res->element=malloc(sizeof(void *)*res->nvar))) ABT_FUNC(MMsg);
		i=0;
		change_type(ST_RESTRICTED,vlist);
		while(vlist) {
			var=vlist->var;
			if(var->vtype&ST_ARRAY) {
				if(vlist->index) res->element[i++]=var->element+vlist->index-1;
				else for(j=0;j<var->n_elements;j++) res->element[i++]=var->element+j;
			} else res->element[i++]=var->element;
			vlist1=vlist->next;
			free(vlist);
			vlist=vlist1;
		}	
	} else res->element=0;
	while(ops) {
		if(ops->type==VARIABLE) ops->arg.element->type|=ST_RESTRICT;
		ops=ops->next;
	}
}

static void find_markers(struct scan_var *var,int *i)
{
	int j;
	
	for(j=0;j<var->n_elements;j++) if(var->element[j].type&ST_MARKER) {
		markers[*i].element=var->element+j;
		markers[*i].var=var;
		markers[(*i)++].index=j+1;
	}
}

static void find_trait_loci(struct scan_var *var,int *i)
{
	int j;
	
	for(j=0;j<var->n_elements;j++) if(var->element[j].type&ST_TRAITLOCUS) {
		if(traitlocus) {
			traitlocus[*i].element=var->element+j;
			traitlocus[*i].var=var;
			traitlocus[(*i)].index=j+1;
		}
		(*i)++;
	}
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "ReadControl"
static void find_haplo(struct scan_var *var)
{
	int j,k;
	
	for(k=0;k<var->n_elements;k++) if(var->element[k].type&ST_HAPLO) {
		for(j=0;j<n_markers;j++) if(var->element[k].arg.element==markers[j].element) {
			if(!markers[j].hap_element[0]) markers[j].hap_element[0]=var->element+k;
			else if(!markers[j].hap_element[1])	markers[j].hap_element[1]=var->element+k;
			else {
				if(markers[j].index) print_scan_err("Error: marker %s(%d) has >2 haplotype vectors associated with it\n",markers[j].var->name,markers[j].index);
				else print_scan_err("Error: marker %s has >2 haplotype vectors associated with it\n",markers[j].var->name);
			}
			break;
		}
		if(j==n_markers) ABT_FUNC("Internal error: can not find marker for haplotype vector\n");
	}
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "ReadControl"
int ReadControl(FILE *fptr,char *cname)
{
	int i,j,k;
	void yy_cleanup(void);
	struct InFile *infile,**infile_p;
	struct Restrict *res,*res1,**res_p;
	struct Censor *cen,**cen_p;
	struct var_element *elem;
	struct Link *linkp;
	struct operation *ops;
	
	yyin=fptr;
	fname_list[0]=cname;
	list_ptr=0;
	if(!(f_atom_list=malloc(sizeof(struct format_atom)*f_atom_size))) ABT_FUNC(MMsg);
	for(i=0;i<2;i++) syst_var[i]=2;
	for(;i<NUM_SYSTEM_VAR;i++) syst_var[i]=0;
	yyparse();
	yy_cleanup();
	/* Sanity check! */
	if(!scan_error_n)	{
		if(!Infiles) print_scan_err("Error: No input files specified\n");
		if(!pedflag) print_scan_err("Error: No pedigree variables specified\n");
		else {
			for(i=0;i<3;i++) if(pedlist[i+family_id]->type&ST_INTTYPE) break;
			if(i<3) for(i=0;i<3;i++) pedlist[i+family_id]->type|=ST_INTTYPE;
		}
		if(root_var) Check_var(root_var);
		/* Flag variables used as the operands to a restriction statement *whose result is used* as ST_REQUIRED */
		res=0;
		while(res!=Restrictions) {	
			res1=Restrictions;
			while(res1->next!=res) res1=res1->next;
			for(i=j=0;i<res1->nvar;i++) if(res1->element[i]->type&ST_REQUIRED) {
				j=1;
				break;
			}
			if(!res1->nvar || j) {
				ops=res1->Op_List;
				while(ops) {
					if(ops->type==VARIABLE) ops->arg.element->type|=ST_REQUIRED;
					ops=ops->next;
				}
			}
			res=res1;
		}
		/* Delete restrict structures that are not used */
		res=Restrictions;
		res_p= &Restrictions;
		while(res) {
			for(i=j=0;i<res->nvar;i++) if(res->element[i]->type&ST_REQUIRED) {
				j=1;
				break;
			}
			if(res->nvar && !j) {
				*res_p=res->next;
				free_restrict(res);
				res= *res_p;
			} else {
				res_p= &res->next;
				res=res->next;
			}
		}
		/* Flag variables used in censored statements as required.  Delete unused censored statements */
		cen=Censored;
		cen_p= &Censored;
		while(cen) {
			if(cen->element->type&ST_TRAIT) {
				ops=cen->Op_List;
				while(ops) {
					if(ops->type==VARIABLE) ops->arg.element->type|=ST_REQUIRED;
					ops=ops->next;
				}
				cen_p= &cen->next;
				cen=cen->next;
			} else {
				*cen_p=cen->next;
				free_op(cen->Op_List);
				free(cen);
				cen= *cen_p;
			}
		}
		if((ops=Affected)) {
			while(ops) {
				if(ops->type==VARIABLE) ops->arg.element->type|=ST_REQUIRED;
				ops=ops->next;
			}
		}
		/* Check file structures - remove ones that aren't needed */
		infile=Infiles;
		infile_p= &Infiles;
		while(infile) {
			infile->ncol=0;
			for(i=0;i<infile->nvar;i++) {
				elem=infile->element[i];
			   if(elem) elem->type&=~ST_FLAG;
			}
			for(i=j=0;i<infile->nvar;i++)	{
				elem=infile->element[i];
				if(elem && elem->type&ST_REQUIRED) {
					if(elem->type&ST_FLAG) {
						print_scan_err("Error: Duplicate variables for file %s\n",infile->name);
						break;
					}
					elem->type|=ST_FLAG;
					if(elem->type&ST_ID)	{
						j|=1;
						infile->id_col=infile->ncol;
					} else if(elem->type&ST_FAMILY) {
						j|=2;
						infile->family_col=infile->ncol;
					}
					infile->ncol++;
				} else infile->element[i]=0;
			}
			for(i=0;i<infile->nvar;i++) if(infile->element[i]) infile->element[i]->type&=~ST_FLAG;
			if(!(j&1)) print_scan_err("Error: No id column for file %s\n",infile->name);
			else if(family_id && j!=3) print_scan_err("Error: No family column for file %s\n",infile->name);
			if(infile->ncol==1) {
				*infile_p=infile->next;
				free_infile(infile);
				infile= *infile_p;
			} else {
				infile_p= &infile->next;
				infile=infile->next;
			}
		}
		if(!Infiles) print_scan_err("Error: No input files with data\n");
		free(f_atom_list);
		/* Count markers and link up with haplotype vectors */
		if(n_markers) {
			if(!(markers=calloc((size_t)n_markers,sizeof(struct Marker)))) ABT_FUNC(MMsg);
			for(i=0;i<n_markers;i++) {
				markers[i].allele_trans=0;
			   markers[i].order=0;
				markers[i].o_size=0;
			}
			i=0;
			if(root_var) {
				check_vars(root_var,&i,find_markers);
				check_vars_1(root_var,find_haplo);
			}
			for(i=0;i<n_markers;i++) {
				linkp=links;
				j=0;
				while(linkp) {
					j++;
					for(k=0;k<linkp->n_loci;k++) {
						if(linkp->element[k]==markers[i].element) {
							markers[i].link=j;
							break;
						}
					}
					if(k<linkp->n_loci) break;
					linkp=linkp->next;
				}
				if(!linkp) {
					if(markers[i].var->vtype&ST_ARRAY)
					  abt(__FILE__,__LINE__,"%s(): No linkage group specified for candidate gene %s(%d)\n",FUNC_NAME,markers[i].var->name,markers[i].index);
					else abt(__FILE__,__LINE__,"%s(): No linkage group specified for candidate gene %s\n",FUNC_NAME,markers[i].var->name);
				}
				if(!markers[i].hap_element[0]) ABT_FUNC("Internal error: null haplotype vector\n");
				if(markers[i].hap_element[0]->type&ST_INTTYPE) markers[i].hap_element[1]->type|=ST_INTTYPE; 
				if(markers[i].hap_element[1] && markers[i].hap_element[1]->type&ST_INTTYPE) markers[i].hap_element[0]->type|=ST_INTTYPE;
			}
		}
		i=0;
		if(root_var) check_vars(root_var,&i,find_trait_loci);
		if(i) {
			if(i>1) print_scan_err("Error: multiple trait loci indicated\n");
			else {
				if(!(traitlocus=calloc(1,sizeof(struct Marker)))) ABT_FUNC(MMsg);
				traitlocus->order=0;
				traitlocus->o_size=0;
				i=0;
				check_vars(root_var,&i,find_trait_loci);
			}
		}
	}		
	return scan_error_n;
}
#line 2453 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack()
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    i = yyssp - yyss;
    newss = yyss ? (short *)realloc(yyss, newsize * sizeof *newss) :
      (short *)malloc(newsize * sizeof *newss);
    if (newss == NULL)
        return -1;
    yyss = newss;
    yyssp = newss + i;
    newvs = yyvs ? (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs) :
      (YYSTYPE *)malloc(newsize * sizeof *newvs);
    if (newvs == NULL)
        return -1;
    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab

#ifndef YYPARSE_PARAM
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG void
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif	/* ANSI-C/C++ */
#else	/* YYPARSE_PARAM */
#ifndef YYPARSE_PARAM_TYPE
#define YYPARSE_PARAM_TYPE void *
#endif
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG YYPARSE_PARAM_TYPE YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL YYPARSE_PARAM_TYPE YYPARSE_PARAM;
#endif	/* ANSI-C/C++ */
#endif	/* ! YYPARSE_PARAM */

int
yyparse (YYPARSE_PARAM_ARG)
    YYPARSE_PARAM_DECL
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate])) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(lint) || defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(lint) || defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 1:
#line 147 "control_parse.y"
{ new_command(); }
break;
case 2:
#line 148 "control_parse.y"
{ new_command(); }
break;
case 4:
#line 150 "control_parse.y"
{ new_command(); }
break;
case 5:
#line 151 "control_parse.y"
{ new_command(); }
break;
case 34:
#line 184 "control_parse.y"
{}
break;
case 35:
#line 187 "control_parse.y"
{ yyval.element=assign_var(yyvsp[-2].var,0,yyvsp[0].express); if(yyvsp[0].express) free(yyvsp[0].express);}
break;
case 36:
#line 188 "control_parse.y"
{ yyval.element=assign_var(yyvsp[-5].var,yyvsp[-3].express,yyvsp[0].express); if(yyvsp[-3].express) free(yyvsp[-3].express); if(yyvsp[0].express) free(yyvsp[0].express);}
break;
case 37:
#line 191 "control_parse.y"
{ yyval.express=alloc_express(); yyval.express->type=ST_REAL; yyval.express->arg.rvalue=yyvsp[0].rvalue; }
break;
case 38:
#line 192 "control_parse.y"
{ yyval.express=alloc_express(); yyval.express->type=ST_INTEGER; yyval.express->arg.value=yyvsp[0].value; }
break;
case 39:
#line 193 "control_parse.y"
{ yyval.express=alloc_express(); yyval.express->type=ST_STRING; yyval.express->arg.string=yyvsp[0].string; }
break;
case 40:
#line 194 "control_parse.y"
{ yyval.express=alloc_express(); yyval.express->type=yyvsp[0].element->type; if(yyvsp[0].element->type==ST_STRING) yyval.express->arg.string=string_copy(0,yyvsp[0].element->arg.string);
		  else yyval.express->arg=yyvsp[0].element->arg; }
break;
case 41:
#line 196 "control_parse.y"
{ yyval.express=do_express_op(yyvsp[-2].express,yyvsp[0].express,'+'); }
break;
case 42:
#line 197 "control_parse.y"
{ yyval.express=do_express_op(yyvsp[-2].express,yyvsp[0].express,'-'); }
break;
case 43:
#line 198 "control_parse.y"
{ yyval.express=do_express_op(yyvsp[-2].express,yyvsp[0].express,'*'); }
break;
case 44:
#line 199 "control_parse.y"
{ yyval.express=do_express_op(yyvsp[-2].express,yyvsp[0].express,'/'); }
break;
case 45:
#line 200 "control_parse.y"
{ yyval.express=do_express_op(yyvsp[0].express,0,'-'); }
break;
case 46:
#line 201 "control_parse.y"
{ yyval.express=yyvsp[-1].express; }
break;
case 47:
#line 204 "control_parse.y"
{ syst_var[yyvsp[-1].value]=yyvsp[0].value; }
break;
case 48:
#line 205 "control_parse.y"
{ yyerror("Unrecognized system variable"); }
break;
case 49:
#line 208 "control_parse.y"
{ enter_loop(); }
break;
case 50:
#line 211 "control_parse.y"
{iflag=1;}
break;
case 51:
#line 211 "control_parse.y"
{include_control_file(yyvsp[0].string);}
break;
case 52:
#line 214 "control_parse.y"
{ add_censored(yyvsp[-4].element,1); at_use=0;}
break;
case 53:
#line 217 "control_parse.y"
{ add_censored(0,0); at_use=0;}
break;
case 54:
#line 220 "control_parse.y"
{ set_sex(yyvsp[-3].element,yyvsp[-2].express,yyvsp[0].express); }
break;
case 55:
#line 223 "control_parse.y"
{ do_while_com(yyvsp[-1].express); if(yyvsp[-1].express) free(yyvsp[-1].express);}
break;
case 57:
#line 227 "control_parse.y"
{yyval.express=do_logical_op(yyvsp[-2].express,yyvsp[0].express,'=');}
break;
case 58:
#line 228 "control_parse.y"
{yyval.express=do_logical_op(yyvsp[-2].express,yyvsp[0].express,NEQSYMBOL);}
break;
case 59:
#line 229 "control_parse.y"
{yyval.express=do_logical_op(yyvsp[-2].express,yyvsp[0].express,LEQSYMBOL);}
break;
case 60:
#line 230 "control_parse.y"
{yyval.express=do_logical_op(yyvsp[-2].express,yyvsp[0].express,GEQSYMBOL);}
break;
case 61:
#line 231 "control_parse.y"
{yyval.express=do_logical_op(yyvsp[-2].express,yyvsp[0].express,'<');}
break;
case 62:
#line 232 "control_parse.y"
{yyval.express=do_logical_op(yyvsp[-2].express,yyvsp[0].express,'>');}
break;
case 63:
#line 233 "control_parse.y"
{yyval.express=do_logical_op(yyvsp[-2].express,yyvsp[0].express,ORSYMBOL);}
break;
case 64:
#line 234 "control_parse.y"
{yyval.express=do_logical_op(yyvsp[-2].express,yyvsp[0].express,ANDSYMBOL);}
break;
case 65:
#line 235 "control_parse.y"
{yyval.express=do_logical_op(yyvsp[0].express,0,NOTSYMBOL);}
break;
case 66:
#line 238 "control_parse.y"
{ printf("\n"); }
break;
case 67:
#line 241 "control_parse.y"
{ print_exp(yyvsp[0].express); if(yyvsp[0].express) free(yyvsp[0].express); }
break;
case 68:
#line 242 "control_parse.y"
{ print_exp(yyvsp[0].express); if(yyvsp[0].express) free(yyvsp[0].express); }
break;
case 69:
#line 245 "control_parse.y"
{if(rsformat) free(rsformat); rsformat=yyvsp[0].string;}
break;
case 70:
#line 246 "control_parse.y"
{if(fsformat) free(fsformat); fsformat=yyvsp[0].string;}
break;
case 72:
#line 251 "control_parse.y"
{set_array_var(yyvsp[-3].var,yyvsp[-1].express); if(yyvsp[-1].express) free(yyvsp[-1].express); }
break;
case 73:
#line 252 "control_parse.y"
{set_array_var(yyvsp[-3].var,yyvsp[-1].express); if(yyvsp[-1].express) free(yyvsp[-1].express); }
break;
case 74:
#line 255 "control_parse.y"
{change_type(ST_REALTYPE,yyvsp[0].var_list); free_vlist(yyvsp[0].var_list);}
break;
case 75:
#line 258 "control_parse.y"
{change_type(ST_INTTYPE,yyvsp[0].var_list); free_vlist(yyvsp[0].var_list);}
break;
case 76:
#line 261 "control_parse.y"
{add_restriction(yyvsp[-4].var_list);at_use=0;}
break;
case 77:
#line 262 "control_parse.y"
{add_restriction(0);at_use=0;}
break;
case 78:
#line 264 "control_parse.y"
{add_restriction(yyvsp[0].var_list);at_use=0;}
break;
case 79:
#line 265 "control_parse.y"
{add_restriction(0);at_use=0;}
break;
case 80:
#line 267 "control_parse.y"
{add_operation(&(yyvsp[0].value),INTEGER,0);}
break;
case 81:
#line 268 "control_parse.y"
{add_operation(&(yyvsp[0].rvalue),REAL,0);}
break;
case 82:
#line 269 "control_parse.y"
{add_operation(yyvsp[0].string,STRING,0);}
break;
case 83:
#line 270 "control_parse.y"
{if(yyvsp[0].element) check_element_add_op(yyvsp[0].element);}
break;
case 84:
#line 271 "control_parse.y"
{add_operation(0,0,'+');}
break;
case 85:
#line 272 "control_parse.y"
{add_operation(0,0,'-');}
break;
case 86:
#line 273 "control_parse.y"
{add_operation(0,0,'*');}
break;
case 87:
#line 274 "control_parse.y"
{add_operation(0,0,'/');}
break;
case 89:
#line 276 "control_parse.y"
{add_operation(0,0,UMINUS);}
break;
case 90:
#line 277 "control_parse.y"
{add_operation(0,0,'=');}
break;
case 91:
#line 278 "control_parse.y"
{add_operation(0,0,NEQSYMBOL);}
break;
case 92:
#line 279 "control_parse.y"
{add_operation(0,0,LEQSYMBOL);}
break;
case 93:
#line 280 "control_parse.y"
{add_operation(0,0,GEQSYMBOL);}
break;
case 94:
#line 281 "control_parse.y"
{add_operation(0,0,'<');}
break;
case 95:
#line 282 "control_parse.y"
{add_operation(0,0,'>');}
break;
case 96:
#line 283 "control_parse.y"
{add_operation(0,0,ORSYMBOL);}
break;
case 97:
#line 284 "control_parse.y"
{add_operation(0,0,ANDSYMBOL);}
break;
case 98:
#line 285 "control_parse.y"
{add_operation(0,0,NOTSYMBOL);}
break;
case 99:
#line 288 "control_parse.y"
{ do_file_com(yyvsp[-2].string,0,0,yyvsp[0].var_list); }
break;
case 100:
#line 289 "control_parse.y"
{ do_file_com(yyvsp[-2].string,yyvsp[-4].format_clause,0,yyvsp[0].var_list); }
break;
case 101:
#line 290 "control_parse.y"
{ do_file_com(yyvsp[-2].string,0,yyvsp[-4].fformat,yyvsp[0].var_list); }
break;
case 102:
#line 293 "control_parse.y"
{ yyval.string=yyvsp[0].string; }
break;
case 103:
#line 294 "control_parse.y"
{ yyval.string=yyvsp[-1].string; shell_flag=1; }
break;
case 104:
#line 297 "control_parse.y"
{yyval.format_clause=add_f_atom(0,yyvsp[0].f_atom); }
break;
case 105:
#line 298 "control_parse.y"
{yyval.format_clause=add_f_atom(yyvsp[-2].format_clause,yyvsp[0].f_atom); }
break;
case 106:
#line 299 "control_parse.y"
{yyval.format_clause=add_f_list(0,yyvsp[-1].format_clause,yyvsp[-3].value); }
break;
case 107:
#line 300 "control_parse.y"
{yyval.format_clause=add_f_list(yyvsp[-5].format_clause,yyvsp[-1].format_clause,yyvsp[-3].value); }
break;
case 109:
#line 304 "control_parse.y"
{yyval.fformat=add_fformat(yyvsp[-2].fformat,yyvsp[0].fformat); }
break;
case 110:
#line 307 "control_parse.y"
{yyval.fformat=create_fformat(yyvsp[0].string,2); }
break;
case 111:
#line 308 "control_parse.y"
{yyval.fformat=create_fformat(yyvsp[0].string,1); }
break;
case 112:
#line 311 "control_parse.y"
{yyval.f_atom=make_f_atom(yyvsp[0].value,0);}
break;
case 113:
#line 312 "control_parse.y"
{yyval.f_atom=make_f_atom(yyvsp[-1].value,1);}
break;
case 114:
#line 313 "control_parse.y"
{yyval.f_atom=make_f_atom(1,1);}
break;
case 115:
#line 314 "control_parse.y"
{yyval.f_atom=make_f_atom(0,1); scan_error|=FORMAT_ERR; }
break;
case 116:
#line 315 "control_parse.y"
{yyval.f_atom=make_f_atom(0,1); scan_error|=FORMAT_ERR; }
break;
case 117:
#line 319 "control_parse.y"
{ LogFile=yyvsp[0].string; }
break;
case 118:
#line 322 "control_parse.y"
{ OutputFile=yyvsp[0].string; }
break;
case 119:
#line 324 "control_parse.y"
{ do_missing_com(yyvsp[0].string,0); }
break;
case 120:
#line 325 "control_parse.y"
{ do_missing_com(yyvsp[-1].string,yyvsp[0].var_list); }
break;
case 121:
#line 326 "control_parse.y"
{ do_missing_com(yyvsp[-2].string,yyvsp[0].var_list); }
break;
case 122:
#line 328 "control_parse.y"
{ do_ped_com(yyvsp[0].var_list); }
break;
case 124:
#line 332 "control_parse.y"
{ change_type(ST_TRAITLOCUS,yyvsp[0].var_list); free_vlist(yyvsp[0].var_list);}
break;
case 125:
#line 335 "control_parse.y"
{ change_type(ST_RANDOM|ST_FACTOR,yyvsp[0].var_list); free_vlist(yyvsp[0].var_list);}
break;
case 126:
#line 338 "control_parse.y"
{ change_type(ST_CONSTANT,yyvsp[0].var_list); free_vlist(yyvsp[0].var_list);}
break;
case 127:
#line 341 "control_parse.y"
{ change_type(ST_FACTOR,yyvsp[0].var_list); free_vlist(yyvsp[0].var_list); }
break;
case 128:
#line 343 "control_parse.y"
{ yyval.value=LINK_AUTO; }
break;
case 129:
#line 344 "control_parse.y"
{ yyval.value=LINK_X; }
break;
case 130:
#line 345 "control_parse.y"
{ yyval.value=LINK_Y; }
break;
case 131:
#line 348 "control_parse.y"
{ do_link_com(0,yyvsp[-1].value,yyvsp[0].var_list); }
break;
case 132:
#line 349 "control_parse.y"
{ do_link_com(yyvsp[-2].string,yyvsp[-3].value,yyvsp[0].var_list); }
break;
case 133:
#line 350 "control_parse.y"
{ yyerror1("No variable list for LINK command\n"); }
break;
case 134:
#line 353 "control_parse.y"
{
	if(Filter) print_scan_warn("Line %d: Warning - Filter defined twice\n",lineno);
	Filter=yyvsp[0].string; }
break;
case 135:
#line 358 "control_parse.y"
{do_model_com(yyvsp[0].model_list,yyvsp[-2].var,0);}
break;
case 136:
#line 359 "control_parse.y"
{do_model_com(yyvsp[0].model_list,yyvsp[-5].var,yyvsp[-3].express); if(yyvsp[-3].express) free(yyvsp[-3].express); }
break;
case 137:
#line 362 "control_parse.y"
{set_group(yyvsp[0].element);}
break;
case 138:
#line 365 "control_parse.y"
{ yyval.model_list=add_to_model(yyvsp[-2].model_list,yyvsp[0].var_list); }
break;
case 139:
#line 366 "control_parse.y"
{ yyval.model_list=add_to_model(yyvsp[-2].model_list,yyvsp[0].var_list); }
break;
case 140:
#line 367 "control_parse.y"
{ yyval.model_list=add_to_model(0,yyvsp[0].var_list); }
break;
case 141:
#line 368 "control_parse.y"
{ yyval.model_list=add_to_model(0,yyvsp[0].var_list); }
break;
case 142:
#line 371 "control_parse.y"
{ yyval.var_list=add_var_lists(yyvsp[-2].var_list,yyvsp[0].var_list); }
break;
case 143:
#line 372 "control_parse.y"
{ yyval.var_list=add_var_lists(yyvsp[-2].var_list,yyvsp[0].var_list); }
break;
case 144:
#line 373 "control_parse.y"
{ yyval.var_list=add_var_lists(yyvsp[-2].var_list,yyvsp[0].var_list); }
break;
case 145:
#line 374 "control_parse.y"
{ yyval.var_list=add_var_lists(yyvsp[-2].var_list,yyvsp[0].var_list); }
break;
case 146:
#line 377 "control_parse.y"
{ yyval.var_list=add_to_var_list(0,yyvsp[0].var,0); }
break;
case 147:
#line 378 "control_parse.y"
{ yyval.var_list=add_to_var_list(0,yyvsp[0].var,0); }
break;
case 148:
#line 379 "control_parse.y"
{ yyval.var_list=add_to_var_list(0,yyvsp[-3].var,yyvsp[-1].express); if(yyvsp[-1].express) free(yyvsp[-1].express); }
break;
case 149:
#line 382 "control_parse.y"
{ yyval.element=get_element(yyvsp[0].var,0); }
break;
case 150:
#line 383 "control_parse.y"
{ yyval.element=get_element(yyvsp[-3].var,yyvsp[-1].express); if(yyvsp[-1].express) free(yyvsp[-1].express); }
break;
case 153:
#line 388 "control_parse.y"
{ if(yyvsp[-1].element) set_locus_element(yyvsp[-1].element); }
break;
case 154:
#line 391 "control_parse.y"
{ if(yyvsp[-1].element) set_haplo_element(yyvsp[-1].element,0); }
break;
case 155:
#line 392 "control_parse.y"
{ if(yyvsp[-3].element) set_haplo_element(yyvsp[-3].element,yyvsp[-1].element); }
break;
case 158:
#line 397 "control_parse.y"
{ yyval.var_list=add_var_lists(yyvsp[-2].var_list,yyvsp[0].var_list); }
break;
case 159:
#line 400 "control_parse.y"
{ start_loopclause(); }
break;
case 160:
#line 403 "control_parse.y"
{ free_vlist(yyvsp[-6].var_list); begin_looping(yyvsp[-3].element,yyvsp[-1].express,0); }
break;
case 161:
#line 404 "control_parse.y"
{ free_vlist(yyvsp[-8].var_list); begin_looping(yyvsp[-5].element,yyvsp[-3].express,yyvsp[-1].express); }
break;
case 162:
#line 407 "control_parse.y"
{ yyval.var_list=yyvsp[-1].var_list; in_loopclause=0; }
break;
case 163:
#line 408 "control_parse.y"
{ yyval.var_list=0; in_loopclause=0; }
break;
case 164:
#line 411 "control_parse.y"
{ yyval.var_list=add_to_var_list(0,0,0); }
break;
case 165:
#line 412 "control_parse.y"
{ yyval.var_list=add_to_var_list(0,yyvsp[0].var,0); }
break;
case 166:
#line 413 "control_parse.y"
{ yyval.var_list=add_to_var_list(0,yyvsp[0].var,0); }
break;
case 167:
#line 414 "control_parse.y"
{ yyval.var_list=add_to_var_list(0,yyvsp[-3].var,yyvsp[-1].express); if(yyvsp[-1].express) free(yyvsp[-1].express); }
break;
case 170:
#line 419 "control_parse.y"
{ yyval.var_list=add_var_lists(yyvsp[-2].var_list,yyvsp[0].var_list); }
break;
case 171:
#line 420 "control_parse.y"
{ yyval.var_list=add_var_lists(yyvsp[-2].var_list,yyvsp[0].var_list); }
break;
case 174:
#line 425 "control_parse.y"
{ yyval.var_list=add_var_lists(yyvsp[-2].var_list,yyvsp[0].var_list); }
break;
case 175:
#line 426 "control_parse.y"
{ yyval.var_list=add_var_lists(yyvsp[-2].var_list,yyvsp[0].var_list); }
break;
case 176:
#line 429 "control_parse.y"
{ yyval.string = yyvsp[0].string; }
break;
case 177:
#line 430 "control_parse.y"
{ yyval.string = string_copy(yyvsp[-2].string,yyvsp[0].string); free(yyvsp[0].string); }
break;
case 178:
#line 431 "control_parse.y"
{ if(yyvsp[-2].element && (yyvsp[-2].element->type&ST_STRING)) yyval.string = string_copy(yyvsp[0].string,yyvsp[-2].element->arg.string); else yyval.string=yyvsp[0].string; }
break;
case 179:
#line 434 "control_parse.y"
{ yyval.string = yyvsp[0].string; }
break;
case 180:
#line 435 "control_parse.y"
{ if(yyvsp[0].element && (yyvsp[0].element->type&ST_STRING)) yyval.string = string_copy(0,yyvsp[0].element->arg.string); else yyval.string=0; }
break;
case 181:
#line 436 "control_parse.y"
{ yyval.string = string_copy(yyvsp[-2].string,yyvsp[0].string); free(yyvsp[0].string); }
break;
case 182:
#line 437 "control_parse.y"
{ if(yyvsp[0].element && (yyvsp[0].element->type&ST_STRING)) yyval.string = string_copy(yyvsp[-2].string,yyvsp[0].element->arg.string); else yyval.string=yyvsp[-2].string; }
break;
#line 3211 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
