/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *                      Simon Heath - MSKCC                                 *
 *                                                                          *
 *                          August 2000                                     *
 *                                                                          *
 * loki_dump.c:                                                             *
 *                                                                          *
 * Routines for reading and writing dump files                              *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/times.h>
#include <time.h>
#include <errno.h>
#include <sys/wait.h>

#include "utils.h"
#include "loki.h"
#include "loki_peel.h"
#include "loki_output.h"
#include "sample_rand.h"
#include "loki_dump.h"

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "read_dump"
int read_dump(int *lp,int *lp1,long *old_pos)
{
	int i=0,j,k,k1,k2,nl=0,type,nrec,v2[4];
	unsigned int ui;
	char *tmp,*tmp1;
	double v1[4],z;
	FILE *fdump;

	k=k1=k2=0;
	if(Filter) {
		errno=0;
		i=child_open(READ,Dumpfile,Filter);
		fdump=fdopen(i,"r");
		if(errno && errno!=ESPIPE) i=__LINE__;
		else i=0;
		errno=0;
	} else fdump=fopen(Dumpfile,"r");
	if(!fdump || i) fprintf(stderr,"[%s:%d] %s(): File Error.  Couldn't open '%s' for input\n",__FILE__,__LINE__,FUNC_NAME,Dumpfile);
	else {
		fputs("[Parameters] ",stdout);
		fflush(stdout);	
		if(!(tmp=fget_line(fdump))) i=__LINE__;
		if(!i && strncmp(tmp,"Loki.dump:",10)) i=__LINE__;
		if(!i) { 
			ui=strtoul(tmp+10,&tmp1,16);
			if(ui!=RunID) {
				i=__LINE__;
				fputs("!Run ID mismatch! ",stdout);
			} else for(j=0;!i && j<4;j++) {
				if(*tmp1++!=',') i=__LINE__;
				else v2[j]=strtol(tmp1,&tmp,16);
				tmp1=tmp;
			}
		}
		if(!i) {
			*lp=(int)v2[0];
			nl=(int)v2[1];
			n_cov_columns=(int)v2[3];
			k=3;
			if(polygenic_flag) k++;
			for(j=0;!i && j<k;j++) {
				if(*tmp1++!=',') i=__LINE__;
				if(txt_get_double(tmp1,&tmp,v1+j)) i=__LINE__;
				tmp1=tmp;
			}
			for(j=0;!i && j<n_random;j++) {
				if(*tmp1++!=',') i=__LINE__;
				if(txt_get_double(tmp1,&tmp,c_var+j)) i=__LINE__;
				tmp1=tmp;
			}
		}
		if(!i) {
			residual_var=v1[0];
			tau=v1[1];
			grand_mean=v1[2];
			if(polygenic_flag) additive_var=v1[3];
		}
		if(*tmp!='\n') i=__LINE__;
		else if(!(tmp=fget_line(fdump))) i=__LINE__;
		for(j=0;!i && j<model.n_terms;j++) {
			type=model.term[j].vars[0].type;
			if(type&(ST_TRAITLOCUS|ST_ID)) continue;
			for(k=0;!i && k<model.term[j].df;k++) {
				if(txt_get_double(tmp,&tmp1,model.term[j].eff+k)) i=__LINE__;
				if(*tmp1=='\n') {
					if(!(tmp=fget_line(fdump))) i=__LINE__;
				} else {
					if(*tmp1++!=',') i=__LINE__;
					tmp=tmp1;
				}
			}
		}
		*lp1=*lp;
		*old_pos=-1;
		if(!i) {
			if(strncmp(tmp,"LKST:",5)) {
				*lp1=strtol(tmp,&tmp1,16);
				if(*tmp1==',') {
					*old_pos=strtol(++tmp1,&tmp,10);
					tmp1=tmp;
				}
				if(*tmp1!='\n') i=__LINE__;
				if(!i && !(tmp=fget_line(fdump))) i=__LINE__;
			}
		}
		if(!i && strncmp(tmp,"LKST:",5)) i=__LINE__;
		else if(!i) {
			j=(int)strtol(tmp+5,&tmp1,16);
			if(j!=N_MOVE_STATS) i=__LINE__;
		}
		for(j=0;!i && j<N_MOVE_STATS;j++) {
			if(*tmp1++!=',') i=__LINE__;
			else move_stats[j].success=(int)strtol(tmp1,&tmp,16);
			if(!i && *tmp++!=',') i=__LINE__;
			else move_stats[j].n=(int)strtol(tmp,&tmp1,16);
		}
		if(!i) {
			if(*tmp1!='\n') i=__LINE__;
			else if(!(tmp=fget_line(fdump))) i=__LINE__;
		}
		if(!i && strncmp(tmp,"LKQT:",5)) i=__LINE__;
		else {
			j=(int)strtol(tmp+5,&tmp1,16);
			if(!i && (*tmp1!='\n' || j!=nl)) i=__LINE__;
		}
		for(j=0;!i && j<nl;j++) {
			if(!(tmp=fget_line(fdump))) i=__LINE__;
			else {
				k1=(int)strtol(tmp,&tmp1,16);
				if(k1!=2) ABT_FUNC("Illegal no. alleles for QTL\n");
				k=get_new_traitlocus(k1);
				if(*tmp1++!=',') i=__LINE__;
				else {
					tlocus[k].link_group=(int)strtol(tmp1,&tmp,10);
					if(*tmp++!=',') i=__LINE__;
					else tlocus[k].flag=(int)strtol(tmp,&tmp1,16);
				}
				if(k<0) ABT_FUNC("Internal error - Couldn't get trait locus\n");
			}
			for(k2=0;!i && k2<2;k2++) {
				if(*tmp1++!=',') i=__LINE__;
				else {
					if(txt_get_double(tmp1,&tmp,tlocus[k].pos+k2)) i=__LINE__;
					else tmp1=tmp;
				}
			}
			if(!i) {
				k1=k1*(k1+1)/2-1;
				for(k2=0;!i && k2<k1;k2++) {
					if(*tmp1++!=',') i=__LINE__;
					else {
						if(txt_get_double(tmp1,&tmp,tlocus[k].eff+k2)) i=__LINE__;
						else tmp1=tmp;
					}
				}
			}
			for(k1=0;!i && k1<n_genetic_groups;k1++) {
				z=1.0;
				for(k2=0;!i && k2<tlocus[j].n_alleles-1;k2++) {
					if(*tmp1++!=',') i=__LINE__;
					else {
						if(txt_get_double(tmp1,&tmp,tlocus[k].freq[k1]+k2)) i=__LINE__;
						else {
							z-=tlocus[k].freq[k1][k2];
							tmp1=tmp;
						}
					}
				}
				if(!i) tlocus[k].freq[k1][k2]=z;
			}
			if(!i && *tmp1!='\n') i=__LINE__;
		}
		if(!i && !(tmp=fget_line(fdump))) i=__LINE__;
		else {
			if(!i && strncmp(tmp,"LKMK:",5)) i=__LINE__;
			else {
				j=(int)strtol(tmp+5,&tmp1,16);
				if(!i && (*tmp1!='\n' || j!=n_markers)) i=__LINE__;
			}
		}
		for(j=0;!i && j<n_markers;j++) {
			if(!(tmp=fget_line(fdump))) i=__LINE__;
			k1=(int)strtol(tmp,&tmp1,16);
			if(*tmp1++!=',' || k1!=marker[j].n_alleles) i=__LINE__;
			else {
				marker[j].link_group=(int)strtol(tmp1,&tmp,10);
				if(*tmp++!=',') i=__LINE__;
				else marker[j].flag=((int)strtol(tmp,&tmp1,16))&~RFMASK_OK;
			}
			for(k=0;!i && k<2;k++) {
				if(*tmp1++!=',') i=__LINE__;
				else {
					if(txt_get_double(tmp1,&tmp,marker[j].pos+k)) i=__LINE__;
					else tmp1=tmp;
				}
			}
			for(k=0;!i && k<n_genetic_groups;k++) {
				z=1.0;
				for(k2=0;!i && k2<k1-1;k2++) {
					if(*tmp1++!=',') i=__LINE__;
					else {
						if(txt_get_double(tmp1,&tmp,marker[j].freq[k]+k2)) i=__LINE__;
						else {
							z-=marker[j].freq[k][k2];
							tmp1=tmp;
						}
					}
				}
				if(!i) marker[j].freq[k][k2]=z;
			}
			if(!i && *tmp1!='\n') i=__LINE__;
		}
		if(!i && n_markers+nl) {
			fputs("[Genotypes] ",stdout);
			fflush(stdout);
			if(nl) {
				if(!(tmp=fget_line(fdump))) i=__LINE__;
				else if(strncmp(tmp,"LKQS:",5)) i=__LINE__;
				else k2=(int)strtol(tmp+5,&tmp1,16);
				if(!i && *tmp1++!=',') i=__LINE__;
				else {
					k=(int)strtol(tmp1,&tmp,16);
					if(*tmp!='\n' || k!=ped_size || k2>=k) i=__LINE__;
				}
				if(k2) for(j=0;!i && j<n_tloci;j++) if(tlocus[j].flag) {
					if(!(tmp=fget_line(fdump))) i=__LINE__;
					for(k=0;!i && k<ped_size;k++) if(id_array[k].sire || id_array[k].dam) {
						k1=(int)*tmp++;
						if(k1>='0'&&k1<='9') k1-='0';
						else if(k1>='a'&&k1<='f') k1-='a'-10;
						else i=__LINE__;
						if(!i) {
							tlocus[j].seg[1][k]=(k1&3)-2;
							tlocus[j].seg[0][k]=(k1>>2)-2;
						}
					}
					if(!i && *tmp!='\n') i=__LINE__;
				}
				if(!(tmp=fget_line(fdump))) i=__LINE__;
				else if(strncmp(tmp,"LKQG\n",5)) i=__LINE__;
				for(k=0;!i && k<ped_size;k++) {
					if(!(tmp=fget_line(fdump))) i=__LINE__;
					for(k1=j=0;!i && j<n_tloci;j++) if(tlocus[j].flag) {
						if(k1++ && *tmp++!=',') i=__LINE__;
						else {
							tlocus[j].gt[k]=(int)strtol(tmp,&tmp1,16);
							tmp=tmp1;
						}
					}
					if(!i && *tmp1!='\n') i=__LINE__;
				}
			}
			if(!i && n_markers) {
				if(!(tmp=fget_line(fdump))) i=__LINE__;
				else if(strncmp(tmp,"LKMS:",5)) i=__LINE__;
				else k2=(int)strtol(tmp+5,&tmp1,16);
				if(!i && *tmp1++!=',') i=__LINE__;
				else {
					k=(int)strtol(tmp1,&tmp,16);
					if(*tmp!='\n' || k!=n_markers) i=__LINE__;
				}
				if(k2) for(j=0;!i && j<n_markers;j++) {
					if(!(tmp=fget_line(fdump))) i=__LINE__;
					for(k=0;!i && k<ped_size;k++) if(id_array[k].sire || id_array[k].dam) {
						k1=(int)*tmp++;
						if(k1>='0'&&k1<='9') k1-='0';
						else if(k1>='a'&&k1<='f') k1-='a'-10;
						else i=__LINE__;
						if(!i) {
							marker[j].seg[1][k]=(k1&3)-2;
							marker[j].seg[0][k]=(k1>>2)-2;
						}
					}
					if(!i && *tmp!='\n') i=__LINE__;
				}
				for(k=j=0;!i && j<n_markers;j++) if(marker[j].mterm) k++;
				if(k) {
					if(!(tmp=fget_line(fdump))) i=__LINE__;
					else if(strncmp(tmp,"LKMG:",5)) i=__LINE__;
					else k2=(int)strtol(tmp+5,&tmp1,16);
					if(*tmp1++!=',' || k2!=k) i=__LINE__;
					else {
						k2=(int)strtol(tmp1,&tmp,16);	
						if(k2!=ped_size) i=__LINE__;
					}
					for(k1=0;!i && k1<k;k1++) {
						if(*tmp++!=',') i=__LINE__;
						else k2=(int)strtol(tmp,&tmp1,16);
						if(k2<0 || k2>=n_markers || !marker[k2].mterm) i=__LINE__;
						else tmp=tmp1;
					}
					if(!i && *tmp++!='\n') i=__LINE__;
					for(k=0;!i && k<ped_size;k++) {
						if(!(tmp=fget_line(fdump))) i=__LINE__;
						for(k1=j=0;!i && j<n_markers;j++) if(marker[j].mterm) {
							if(k1++ && *tmp++!=',') i=__LINE__;
							else {
								marker[j].gt[k]=(int)strtol(tmp,&tmp1,16);
								tmp=tmp1;
							}
						}
						if(!i && *tmp!='\n') i=__LINE__;
					}
				}
			}
		}
		if(!i) {
			fputs("[Residuals] ",stdout);
			fflush(stdout);
			if(!(tmp=fget_line(fdump))) i=__LINE__;
			else if(strncmp(tmp,"LKRS:",5)) i=__LINE__;
			else k2=(int)strtol(tmp+5,&tmp1,16);
			if(*tmp1++!='\n' || k2!=censored_flag) i=__LINE__;
		}
		for(j=0;!i && j<ped_size;j++) {
			if(!i && polygenic_flag) {
				if(!(tmp=fget_line(fdump))) i=__LINE__;
				if(txt_get_double(tmp,&tmp1,&id_array[j].bv)) i=__LINE__;
				if(!i && *tmp1++!='\n') i=__LINE__;
			}
			if(!id_array[j].res) continue;
			if(!(tmp=fget_line(fdump))) i=__LINE__;
			nrec=id_array[j].n_rec;
			if(!nrec) nrec=1;
			for(k=0;!i && k<nrec;k++) {
				if(k && *tmp++!=',') i=__LINE__;
				else if(txt_get_double(tmp,&tmp1,id_array[j].res+k)) i=__LINE__;
				if(!i && use_student_t) {
					if(*tmp1++!=',') i=__LINE__;
					else if(txt_get_double(tmp1,&tmp,id_array[j].vv+k)) i=__LINE__;
					tmp1=tmp;
				}
				if(!i && censored_flag && id_array[j].cens) {
					if(*tmp1++!=',') i=__LINE__;
					else if(txt_get_double(tmp1,&tmp,id_array[j].cens+k)) i=__LINE__;
				} else tmp=tmp1;
			}
			if(!i && *tmp!='\n') i=__LINE__;
		}
		if(!i) {
			fputs("[Seed] ",stdout);
			fflush(stdout);
			if(!(tmp=fget_line(fdump))) i=__LINE__;
			else if(strncmp(tmp,"LKSD:",5)) i=__LINE__;
			if(!i && binreadseed(fdump,tmp+5)) i=__LINE__;
		}
		if(!i) {
			fputs("[Time",stdout);
			fflush(stdout);
			if(!(tmp=fget_line(fdump))) i=__LINE__;
			else {
				if(strncmp(tmp,"LKTM:",5)) {
					fputs(" !Not found!] ",stdout);
					fflush(stdout);
				} else {
					fputs("] ",stdout);
					fflush(stdout);
					if(txt_get_double(tmp+5,&tmp1,&extra_time)) i=__LINE__;
					if(!i && *tmp1++!=',') i=__LINE__;
					if(!i && txt_get_double(tmp1,&tmp,&extra_stime)) i=__LINE__;
					if(!i && *tmp++!=',') i=__LINE__;
					if(!i && txt_get_double(tmp,&tmp1,&extra_utime)) i=__LINE__;
					if(!i && *tmp1!='\n') i=__LINE__;
					if(!i && !(tmp=fget_line(fdump))) i=__LINE__;
				}
				if(!i && strncmp(tmp,"Ldmp.end\n",9)) i=__LINE__;
			}
		}
	}
	if(fdump) fclose(fdump);
	if(i) fprintf(stderr,"[%s:%d] Error in %s() ",__FILE__,i,FUNC_NAME);
	return -i;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "write_dump"
void write_dump(int lp,int lp1,long old_pos)
{
	int i=0,j,k,k1,k2,nx,nl,type,nrec;
	double ex1,ex2;
	struct tms tbuf;
	long tps;
	FILE *fdump;
	
	fputs("Dumping program state: ",stdout);
	fflush(stdout);
	if(!Dumpfile) Dumpfile=make_file_name("dump");
	if(Dumpfile) {
		j=syst_var[SYST_BACKUPS].flag?syst_var[SYST_BACKUPS].data.value:1;
		if(j) i=mkbackup(Dumpfile,j);
		if(!i) {
			if(Filter) {
				errno=0;
				i=child_open(WRITE,Dumpfile,Filter);
				fdump=fdopen(i,"w");
				if(errno && errno!=ESPIPE) i=1;
				else i=0;
				errno=0;
			} else fdump=fopen(Dumpfile,"w");
			if(!fdump || i) fprintf(stderr,"[%s:%d] %s(): File Error.  Couldn't open '%s' for output\n",__FILE__,__LINE__,FUNC_NAME,Dumpfile);
			else {
				fputs("[Parameters] ",stdout);
				fflush(stdout);
				for(nx=j=0;j<model.n_terms;j++) {
					type=model.term[j].vars[0].type;
					if(type&(ST_TRAITLOCUS|ST_ID)) continue;
					nx+=model.term[j].df;
				}
				for(nl=j=0;j<n_tloci;j++) if(tlocus[j].flag) nl++;
				if(!i && fprintf(fdump,"Loki.dump:%x,%x,%x,%x,%x,",RunID,lp,nl,nx,n_cov_columns)<0) i=1;
				if(!i) i=txt_print_double(residual_var,fdump);
				if(!i && fputc(',',fdump)<0) i=1;
				if(!i) i=txt_print_double(tau,fdump);
				if(!i && fputc(',',fdump)<0) i=1;
				if(!i) i=txt_print_double(grand_mean,fdump);
				if(!i && polygenic_flag) {
					if(!i && fputc(',',fdump)<0) i=1;
					i=txt_print_double(additive_var,fdump);
				}
				for(j=0;!i && j<n_random;j++) {
					if(!i && fputc(',',fdump)<0) i=1;
					i=txt_print_double(c_var[j],fdump);
				}
				if(!i && fputc('\n',fdump)<0) i=1;
				for(j=0;!i && j<model.n_terms;j++) {
					type=model.term[j].vars[0].type;
					if(type&(ST_TRAITLOCUS|ST_ID)) continue;
					for(k=0;!i && k<model.term[j].df;k++) {
						i=txt_print_double(model.term[j].eff[k],fdump);	
						if(!i && fputc('\n',fdump)<0) i=1;
					}
				}
				if(!i && fprintf(fdump,"%x,%ld",lp1,old_pos)<0) i=1;
				if(!i && fprintf(fdump,"\nLKST:%x",N_MOVE_STATS)<0) i=1;
				for(j=0;!i && j<N_MOVE_STATS;j++) if(fprintf(fdump,",%x,%x",move_stats[j].success,move_stats[j].n)<0) i=1;
				if(!i && fprintf(fdump,"\nLKQT:%x\n",nl)<0) i=1;
				for(j=0;!i && j<n_tloci;j++) if(tlocus[j].flag) {
					if(fprintf(fdump,"%x,%d,%x",tlocus[j].n_alleles,tlocus[j].link_group,tlocus[j].flag)<0) i=1;
					for(k=0;!i && k<2;k++) {
						if(fputc(',',fdump)<0) i=1;
						else i=txt_print_double(tlocus[j].pos[k],fdump);	
					}
					k=tlocus[j].n_alleles;
					k1=k*(k+1)/2-1;
					for(k=0;!i && k<k1;k++) {
						if(fputc(',',fdump)<0) i=1;
						else i=txt_print_double(tlocus[j].eff[k],fdump);
					}
					for(k=0;!i && k<n_genetic_groups;k++) {
						for(k1=0;!i && k1<tlocus[j].n_alleles-1;k1++) {
							if(fputc(',',fdump)<0) i=1;
							else i=txt_print_double(tlocus[j].freq[k][k1],fdump);
						}
					}
					if(!i && fputc('\n',fdump)<0) i=1;
				}
				if(!i && fprintf(fdump,"LKMK:%x\n",n_markers)<0) i=1;
				for(j=0;!i && j<n_markers;j++) {
					if(fprintf(fdump,"%x,%d,%x",marker[j].n_alleles,marker[j].link_group,marker[j].flag)<0) i=1;
					for(k=0;!i && k<2;k++) {
						if(fputc(',',fdump)<0) i=1;
						else i=txt_print_double(marker[j].pos[k],fdump);	
					}
					for(k=0;!i && k<n_genetic_groups;k++) {
						for(k1=0;!i && k1<marker[j].n_alleles-1;k1++) {
							if(fputc(',',fdump)<0) i=1;
							else i=txt_print_double(marker[j].freq[k][k1],fdump);
						}
					}
					if(!i && fputc('\n',fdump)<0) i=1;
				}
				if(!i && n_markers+nl) {
					fputs("[Genotypes] ",stdout);
					fflush(stdout);
					for(k2=k=0;k<ped_size;k++) if(id_array[k].sire || id_array[k].dam) k2++;
					if(nl) {
						if(k2) {
							if(fprintf(fdump,"LKQS:%x,%x\n",k2,ped_size)<0) i=1;
							for(j=0;!i && j<n_tloci;j++) if(tlocus[j].flag) {
								for(k=0;!i && k<ped_size;k++) if(id_array[k].sire || id_array[k].dam) {
									if(fprintf(fdump,"%x",((tlocus[j].seg[0][k]+2)<<2)|(tlocus[j].seg[1][k]+2))<0) i=1;
								}
								if(!i && fputc('\n',fdump)<0) i=1;
							}
						}
						if(!i && fputs("LKQG\n",fdump)<0) i=1;
						for(k=0;!i && k<ped_size;k++) {
							for(k1=j=0;!i && j<n_tloci;j++) if(tlocus[j].flag) {
								if(k1++ && fputc(',',fdump)<0) i=1;
								if(!i && fprintf(fdump,"%x",tlocus[j].gt[k])<0) i=1;
							}
							if(!i && fputc('\n',fdump)<0) i=1;
						}
					}
					if(n_markers) {
						if(k2) {
							if(fprintf(fdump,"LKMS:%x,%x\n",k2,n_markers)<0) i=1;
							for(j=0;!i && j<n_markers;j++) {
								for(k=0;!i && k<ped_size;k++) if(id_array[k].sire || id_array[k].dam) {
									if(fprintf(fdump,"%x",((marker[j].seg[0][k]+2)<<2)|(marker[j].seg[1][k]+2))<0) i=1;
								}
								if(!i && fputc('\n',fdump)<0) i=1;
							}
						}
						for(k=j=0;!i && j<n_markers;j++) if(marker[j].mterm) k++;
						if(k) {
							if(fprintf(fdump,"LKMG:%x,%x",k,ped_size)<0) i=1;
							for(j=0;!i && j<n_markers;j++) if(marker[j].mterm && fprintf(fdump,",%x",j)<0) i=1;
							if(!i && fputc('\n',fdump)<0) i=1;
							for(k=0;!i && k<ped_size;k++) {
								for(k1=j=0;!i && j<n_markers;j++) if(marker[j].mterm) {
									if(k1++ && fputc(',',fdump)<0) i=1;
									if(!i && fprintf(fdump,"%x",marker[j].gt[k])<0) i=1;
								}
								if(!i && fputc('\n',fdump)<0) i=1;
							}
						}
					}
				}
				if(!i) {
					fputs("[Residuals] ",stdout);
					fflush(stdout);
					if(fprintf(fdump,"LKRS:%x\n",censored_flag)<0) i=1;
				}
				for(j=0;!i && j<ped_size;j++) {
					if(polygenic_flag) {
						i=txt_print_double(id_array[j].bv,fdump);
						if(!i && fputc('\n',fdump)<0) i=1;
					}
					if(!id_array[j].res) continue;
					nrec=id_array[j].n_rec;
					if(!nrec) nrec=1;
					for(k=0;!i && k<nrec;k++) {
						if(k && fputc(',',fdump)<0) i=1;
						else i=txt_print_double(id_array[j].res[k],fdump);
						if(!i && use_student_t) {
							if(fputc(',',fdump)<0) i=1;
							else i=txt_print_double(id_array[j].vv[k],fdump);
						}
						if(!i && censored_flag && id_array[j].cens) {
							if(fputc(',',fdump)<0) i=1;
							else i=txt_print_double(id_array[j].cens[k],fdump);
						}
					}
					if(!i && fputc('\n',fdump)<0) i=1;
				}
				if(!i) {
					fputs("[Seed] ",stdout);
					fflush(stdout);
					if(fputs("LKSD:",fdump)<0) i=1;
					if(!i) i=bindumpseed(fdump);
				}
				if(!i) {
					fputs("[Time] ",stdout);
					fflush(stdout);
					if(fputs("LKTM:",fdump)<0) i=1;
					if(!i) i=txt_print_double(extra_time+difftime(time(0),start_time),fdump);
					if(!i) {
						ex1=extra_stime;
						ex2=extra_utime;
						tps=sysconf (_SC_CLK_TCK);
						errno=0;
						(void)times(&tbuf);
						if(errno) perror("print_end_time():");
						else {
							ex1+=(double)tbuf.tms_stime/(double)tps;
							ex2+=(double)tbuf.tms_utime/(double)tps;
						}
					}
					if(!i && fputc(',',fdump)<0) i=1;
					if(!i) i=txt_print_double(ex1,fdump);
					if(!i && fputc(',',fdump)<0) i=1;
					if(!i) i=txt_print_double(ex2,fdump);
					if(!i && fputc('\n',fdump)<0) i=1;
				}
				if(!i && fputs("Ldmp.end\n",fdump)<0) i=1;
			}
			if(fdump) {
				fclose(fdump);
				if(i) unlink(Dumpfile);
			}
		}
	} else i=1;
	if(i) fputs("FAILED\n",stdout);
	else fputs("OK\n",stdout);
	do j=wait(&i); while(j>0); /* Wait for child processes to finish */
}

