/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *             Simon Heath - Rockefeller University                         *
 *                                                                          *
 *                       February 1998                                      *
 *                                                                          *
 * loki_ibd.c:                                                              *
 *                                                                          *
 * Routines for estimaing pariwise IBD matrices                             *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <math.h>
#include <stdio.h>
#include <float.h>
#include <stdlib.h>
#include <strings.h>

#include "ranlib.h"
#include "utils.h"
#include "loki.h"
#include "loki_peel.h"
#include "loki_ibd.h"

double ***ibd;
int *loci;
char *seg[2];

void free_IBD(void)
{
	if(ibd) {
		if(ibd[0]) {
			if(ibd[0][0]) free(ibd[0][0]);
			free(ibd[0]);
		}
		free(ibd);
	}
	if(loci) free(loci);
	if(seg[0]) free(seg[0]);
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "sample_segs"
void sample_segs(void) 
{
	int i,i1,j,k,comp,link,locus,nloci,nkids,par_flag;
	int *loci,s,s1,kid;
	double *p[2],pp[2],z,*recom[2],x,x1,*r;
	
	if(!(loci=malloc(sizeof(int)*n_markers))) ABT_FUNC(MMsg);
	if(!(p[0]=malloc(sizeof(double)*n_markers*4))) ABT_FUNC(MMsg);
	p[1]=p[0]+n_markers;
	recom[0]=p[1]+n_markers;
	recom[1]=recom[0]+n_markers;
	for(i1=comp=0;comp<n_comp;comp++) {
		if(comp_size[comp]>1) {
			for(link=0;link<n_links;link++) {
				for(nloci=k=0;k<linkage[link].n_markers;k++) {
					locus=linkage[link].mk_index[k];
					if(marker[locus].n_all1[comp]>=2) loci[nloci++]=locus;
				}
				if(!nloci) continue;
				gnu_qsort(loci,(size_t)nloci,(size_t)sizeof(int),cmp_loci_pos);
				for(par_flag=0;par_flag<2;par_flag++) {
					x=marker[loci[0]].pos[par_flag];
					for(k=1;k<nloci;k++) {
						x1=marker[loci[1]].pos[par_flag];
						recom[par_flag][k-1]=.5*(1.0-exp(0.02*(x-x1)));
						x=x1;
					}
				}
				for(i=i1;i<i1+comp_size[comp];i++) if(id_array[i].nkids) {
					par_flag=2-id_array[i].sex;
					r=recom[par_flag];
					nkids=id_array[i].nkids;
					for(j=0;j<nkids;j++) {
						kid=id_array[i].kids[j];
						locus=loci[0];
						s=marker[locus].seg[par_flag][kid];
						if(s<0) {
							p[0][0]=p[1][0]=.5;
						} else {
							p[s][0]=1.0;
							p[s^1][0]=0.0;
						}
						for(k=1;k<nloci;k++) {
							locus=loci[k];
							s=marker[locus].seg[par_flag][kid];
							if(s<0) {
								pp[0]=.5*(p[0][k-1]*(1.0-r[k-1])+p[1][locus]*r[k-1]);
								pp[1]=.5*(p[1][k-1]*(1.0-r[k-1])+p[0][locus]*r[k-1]);
								z=pp[0]+pp[1];
								p[0][k]=pp[0]/z;
								p[1][k]=pp[1]/z;
							} else {
								p[s][k]=1.0;
								p[s^1][k]=0.0;
							}
						}
						s=ranf()<p[0][nloci-1]?0:1;
						marker[locus].seg[par_flag][kid]=s;
						for(k=nloci-2;k>=0;k--) {
							locus=loci[k];
							s1=marker[locus].seg[par_flag][kid];
							if(s1<0) {
								pp[s]=p[s][k]*(1.0-r[k]);
								pp[s^1]=p[s^1][k]*r[k];
								z=pp[0]+pp[1];
								s1=ranf()*z<pp[0]?0:1;
								marker[locus].seg[par_flag][kid]=s1;
							}
							s=s1;
						}
					}
				}
			}
		}
		i1+=comp_size[comp];
	}
	free(loci);
	free(p[0]);
	return;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "SetupIBD"
int SetupIBD(void)
{
	int i,j,k,k1;
	double *tp;
	
	for(i=j=0;i<n_links;i++) if(linkage[i].ibd_list) j+=linkage[i].ibd_list->idx;
	if(j) { /* Allocate memory for j triangular matrices */
		for(k=i=0;i<n_comp;i++) k+=comp_size[i]*(comp_size[i]+1)/2;
		if(!(ibd=malloc(sizeof(void *)*j))) ABT_FUNC(MMsg);
		if(!(ibd[0]=malloc(sizeof(void *)*j*n_comp))) ABT_FUNC(MMsg);
		if(!(ibd[0][0]=tp=malloc(sizeof(double)*j*k))) ABT_FUNC(MMsg);
		for(i=0;i<j*k;i++) tp[i]=0.0;
		for(i=1;i<j;i++) {
			ibd[i]=ibd[i-1]+n_comp;
			ibd[i][0]=ibd[i-1][0]+k;
		}
		for(i=1;i<n_comp;i++) {
			k1=comp_size[i-1]*(comp_size[i-1]+1)/2;
			for(k=0;k<j;k++) ibd[k][i]=ibd[k][i-1]+k1;
		}
	}
	if(!(loci=malloc((n_markers+2*ped_size)*sizeof(int)))) ABT_FUNC(MMsg);
	if(!(seg[0]=malloc((size_t)(2*ped_size)))) ABT_FUNC(MMsg);
	seg[1]=seg[0]+ped_size;
	return j;
}

void Set_Trans(double *pos,const int *perm,int idx,int n_loci,int comp)
{
	int i,k,k1,k2,ids,idd;
	double theta,Mtp[2],Ptp[2],z,z1;
	
	for(i=comp_start[comp];i<comp_start[comp]+comp_size[comp];i++) {
		ids=id_array[i].sire;
		idd=id_array[i].dam;
		if(ids || idd) {
			Mtp[0]=Mtp[1]=Ptp[0]=Ptp[1]=1.0;
			for(k=idx-1;k>=0;k--) {
				k1=perm[k];
				if(k1<0)	{
					if(id_array[i].pruned_flag[n_markers] || id_array[ids-1].pruned_flag[n_markers] || !(tlocus[-1-k1].flag&MARKER_SAMPLED)) continue;
					k2=tlocus[-1-k1].seg[X_MAT][i];
					if(k2<0) continue;
					theta=.5*(1.0-exp(-0.02*(pos[X_MAT]-tlocus[-1-k1].pos[X_MAT])));
				} else {
					if(marker[k1].n_all1[comp]<2) continue;
					if(id_array[i].pruned_flag[k1] || id_array[ids-1].pruned_flag[k1] || !(marker[k1].flag&MARKER_SAMPLED)) continue;
					k2=marker[k1].seg[X_MAT][i];
					if(k2<0) continue;
					theta=.5*(1.0-exp(-0.02*(pos[X_MAT]-marker[k1].pos[X_MAT])));
				}
				Mtp[k2]=1.0-theta;
				Mtp[1-k2]=theta;
				break;
			}
			for(k=idx-1;k>=0;k--) {
				k1=perm[k];
				if(k1<0)	{
					if(id_array[i].pruned_flag[n_markers] || id_array[ids-1].pruned_flag[n_markers] || !(tlocus[-1-k1].flag&MARKER_SAMPLED)) continue;
					k2=tlocus[-1-k1].seg[X_PAT][i];
					if(k2<0) continue;
					theta=.5*(1.0-exp(-0.02*(pos[X_PAT]-tlocus[-1-k1].pos[X_PAT])));
				} else {
					if(marker[k1].n_all1[comp]<2) continue;
					if(id_array[i].pruned_flag[k1] || id_array[ids-1].pruned_flag[k1] || !(marker[k1].flag&MARKER_SAMPLED)) continue;
					k2=marker[k1].seg[X_PAT][i];
					if(k2<0) continue;
					theta=.5*(1.0-exp(-0.02*(pos[X_PAT]-marker[k1].pos[X_PAT])));
				}
				Ptp[k2]=1.0-theta;
				Ptp[1-k2]=theta;
				break;
			}
			for(k=idx;k<n_loci;k++) {
				k1=perm[k];
				if(k1<0)	{
					if(id_array[i].pruned_flag[n_markers] || id_array[ids-1].pruned_flag[n_markers] || !(tlocus[-1-k1].flag&MARKER_SAMPLED)) continue;
					k2=tlocus[-1-k1].seg[X_MAT][i];
					if(k2<0) continue;
					theta=.5*(1.0-exp(-0.02*(tlocus[-1-k1].pos[X_MAT]-pos[X_MAT])));
				} else {
					if(marker[k1].n_all1[comp]<2) continue;
					if(id_array[i].pruned_flag[k1] || id_array[ids-1].pruned_flag[k1] || !(marker[k1].flag&MARKER_SAMPLED)) continue;
					k2=marker[k1].seg[X_MAT][i];
					if(k2<0) continue;
					theta=.5*(1.0-exp(-0.02*(marker[k1].pos[X_MAT]-pos[X_MAT])));
				}
				Mtp[k2]*=1.0-theta;
				Mtp[1-k2]*=theta;
				break;
			}
			for(k=idx;k<n_loci;k++) {
				k1=perm[k];
				if(k1<0)	{
					if(id_array[i].pruned_flag[n_markers] || id_array[ids-1].pruned_flag[n_markers] || !(tlocus[-1-k1].flag&MARKER_SAMPLED)) continue;
					k2=tlocus[-1-k1].seg[X_PAT][i];
					if(k2<0) continue;
					theta=.5*(1.0-exp(-0.02*(tlocus[-1-k1].pos[X_PAT]-pos[X_PAT])));
				} else {
					if(marker[k1].n_all1[comp]<2) continue;
					if(id_array[i].pruned_flag[k1] || id_array[ids-1].pruned_flag[k1] || !(marker[k1].flag&MARKER_SAMPLED)) continue;
					k2=marker[k1].seg[X_PAT][i];
					if(k2<0) continue;
					theta=.5*(1.0-exp(-0.02*(marker[k1].pos[X_PAT]-pos[X_PAT])));
				}
				Ptp[k2]*=1.0-theta;
				Ptp[1-k2]*=theta;
				break;
			}
			z=Mtp[0]+Mtp[1];
			z1=Ptp[0]+Ptp[1];
			for(k=0;k<2;k++) {
				Mtp[k]/=z;
				Ptp[k]/=z1;
				id_array[i].tpp[X_MAT][k]=Mtp[k];
				id_array[i].tpp[X_PAT][k]=Ptp[k];
			}
			id_array[i].tp[X_MM_PM]=Mtp[X_MAT]*Ptp[X_MAT];
			id_array[i].tp[X_MM_PP]=Mtp[X_MAT]*Ptp[X_PAT];
			id_array[i].tp[X_MP_PM]=Mtp[X_PAT]*Ptp[X_MAT];
			id_array[i].tp[X_MP_PP]=Mtp[X_PAT]*Ptp[X_PAT];
		} else {	
			for(k=0;k<2;k++) id_array[i].tpp[X_MAT][k]=id_array[i].tpp[X_PAT][k]=0.5;
			for(k=0;k<4;k++) id_array[i].tp[k]=0.25;
		}
	}
}

void Handle_IBD(void)
{
	int i,i1,i2,i3,i4,j,k,k1,k2,k3,ix=0,*genes[2],nl,s,par,comp;
	double x,z,**ibd1,ps[2],tp[2];
	
	genes[0]=loci+n_markers;
	genes[1]=genes[0]+ped_size;
	for(i=0;i<n_links;i++) {
		if(linkage[i].ibd_list)	{
			get_locuslist(loci,i,&nl);
			gnu_qsort(loci,(size_t)nl,(size_t)sizeof(int),cmp_loci_pos);
			for(j=0;j<linkage[i].ibd_list->idx;j++) {
				ibd1=ibd[ix++];
				x=linkage[i].ibd_list->pos[j];
				for(k=0;k<nl;k++) if(marker[loci[k]].pos[0]>=x) break;
				/* Should extend this to allow for sex-specific maps later */
				ps[0]=ps[1]=x;
				for(i1=0,comp=0;comp<n_comp;comp++) {
					Set_Trans(ps,loci,k,nl,comp);
					for(i2=0;i2<comp_size[comp];i2++) if(id_array[i1+i2].sire) {
						for(k1=0;k1<2;k1++) {
							tp[0]=id_array[i1+i2].tpp[k1][0];
							tp[1]=id_array[i1+i2].tpp[k1][1];
							z=tp[0]+tp[1];
							if(tp[0] && ranf()*z<=tp[0]) seg[k1][i1+i2]=0;
							else seg[k1][i1+i2]=1;
						}
					}
					i1+=comp_size[comp];
				}
				for(i1=0,comp=0;comp<n_comp;comp++) {
					for(k1=i2=i3=0;i2<comp_size[comp];i2++,i1++) {
						par=id_array[i1].sire;
						if(par) {
							s=seg[X_PAT][i1];
							genes[X_PAT][i1]=genes[s][par-1];
							par=id_array[i1].dam;
							s=seg[X_MAT][i1];
							genes[X_MAT][i1]=genes[s][par-1];
						} else {	
							genes[X_MAT][i1]= ++k1;
							genes[X_PAT][i1]= ++k1;
						}
						for(i4=comp_start[comp];i4<i1;i4++)	{
							z=0.0;
							for(k2=0;k2<2;k2++) for(k3=0;k3<2;k3++) if(genes[k2][i1]==genes[k3][i4]) z+=.5;
							ibd1[comp][i3++]+=z;
						}
						z=1.0;
						if(genes[0][i1]==genes[1][i1]) z+=1.0;
						ibd1[comp][i3++]+=z;
					}
				}
			}
		}
	}
	return;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "score_ibd"
double score_ibd(int locus,int *k5,int comp,int affn,int *aff,double *pairs)
{
	int i,i1,i2,j,k2,k3,k4,**genes;
	double l=0.0,z;
	
	*k5=k4=0;
 	if(marker[locus].n_all1[comp]<2) return l;
	genes=marker[locus].genes;
	for(i=0;i<affn;i++) {
		i1=aff[i];
		for(j=0;j<=i;j++) {
			i2=aff[j];
			z=0.0;
			for(k2=0;k2<2;k2++) for(k3=0;k3<2;k3++) if(genes[k2][i1]==genes[k3][i2]) z+=.25;
			if(pairs[k4]>0.0) {
 				l+=z/pairs[k4];
				(*k5)++;
			} else if(z>0.0) {
				fprintf(stderr,"[%d  %d %d  %d %d  (%d %d) (%d %d)  %d]\n",comp,i,j,i1,i2,k4,genes[0][i1],genes[1][i1],genes[0][i2],genes[1][i2]);
				ABT_FUNC("OOOK!\n");
			}
			k4++;
		}
	}
 	return l; 
}

