/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *             Simon Heath - Rockefeller University                         *
 *                                                                          *
 *                       October 1997                                       *
 *                                                                          *
 * loki_init.c:                                                             *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <float.h>

#include "utils.h"
#include "loki.h"
#include "loki_peel.h"
#include "sample_rand.h"

int censored_flag,censor_mode;
struct move_stats move_stats[N_MOVE_STATS];
struct Family *family;

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "LokiSetup"
void LokiSetup(void)
{
	int i,j,k=-1,fx,k1,k2,l,er=0,n_all,comp,sire,dam,*temp_p,grp,*perm,*perm1;
	unsigned long a;
	double p,min,max,*temp_dp,**temp_dpp;
	struct Variable *group_var=0;
	static char *sexstr[2]={"female","male"};

	if(sex_map) {
		for(j=i=0;i<ped_size;i++) {
			if(id_array[i].sex<1 || id_array[i].sex>2) {
				print_orig_id(stderr,i+1);
				fputs(" has invalid sex information\n",stderr);
				er=1;
			}
		}
	}
	for(i=0;i<n_id_records;i++) if(id_variable[i].type&ST_GROUP) {
		group_var=id_variable+i;
		break;
	}
	/* Check for trait loci in model */
	for(i=0;i<model.n_terms;i++) {
		for(k=0;k<model.term[i].n_vars;k++)
		  if(model.term[i].vars[k].type&ST_TRAITLOCUS) break;
		if(k<model.term[i].n_vars) break;
	}
	if(i==model.n_terms) n_tloci=0;
	else {
		/* This is just enough for starters.  If we need more then we
		 * can realloc this no? */
		if(n_tloci>max_tloci) n_tloci=max_tloci;
		if(n_tloci) {
			if(!(tlocus=malloc(sizeof(struct TraitLocus)*n_tloci))) ABT_FUNC(MMsg);
			for(i=0;i<n_tloci;i++) {
				tlocus[i].freq=0;
				tlocus[i].flag=0;
				tlocus[i].seg[0]=0;
				tlocus[i].gt=0;
			}
		}
	}
	for(i=j=0;i<ped_size;i++) {
		id_array[i].nkids=0;
		sire=id_array[i].sire;
		dam=id_array[i].dam;
		if(sire)	{
			id_array[sire-1].nkids++;
			j++;
		}
		if(dam) {
			id_array[dam-1].nkids++;
			j++;
		}
		id_array[i].flag=0;
	}
	if(j) {
		if(!(temp_p=malloc(sizeof(int)*j))) ABT_FUNC(MMsg);
		RemBlock=AddRemem(temp_p,RemBlock);
		for(i=0;i<ped_size;i++) {
			j=id_array[i].nkids;
			id_array[i].kids=temp_p;
			if(j) {
				temp_p+=j;
				id_array[i].nkids=0;
			}
		}
		for(i=0;i<ped_size;i++) {
			sire=id_array[i].sire;
			dam=id_array[i].dam;
			if(sire) {
				j=id_array[sire-1].nkids++;
				id_array[sire-1].kids[j]=i;
			}
			if(dam) {
				j=id_array[dam-1].nkids++;
				id_array[dam-1].kids[j]=i;
			}
		}
	}
	for(k=0,comp=0;comp<n_comp;comp++)	{
		for(k1=0;k1<comp_size[comp];k1++) {
			if(!id_array[k].sire) comp_ngenes[comp]+=2;
			if(!id_array[k++].dam) comp_ngenes[comp]+=2;
		}
	}
	if(n_markers) {
		if(!(temp_p=malloc(sizeof(int)*2*n_markers*ped_size))) ABT_FUNC(MMsg);
		RemBlock=AddRemem(temp_p,RemBlock);
		for(i=0;i<n_markers;i++) for(j=0;j<2;j++) {
			marker[i].genes[j]=temp_p;
			temp_p+=ped_size;
		}
		if(!(temp_dp=calloc((size_t)(n_markers*n_comp),sizeof(double)))) ABT_FUNC(MMsg);
		RemBlock=AddRemem(temp_dp,RemBlock);
		for(i=0;i<n_markers;i++) {
			marker[i].lk_store=temp_dp;
			temp_dp+=n_comp;
		}
		for(k=k2=i=0;i<n_markers;i++) {
			if(!marker[i].n_alleles) continue;
			for(k1=grp=0;grp<n_genetic_groups;grp++) if(marker[i].count_flag[i]) k1++;
			if(k1) {
				marker[i].counts=(void *)-1;
				k2+=k1*marker[i].n_alleles;
				k++;
			} else marker[i].counts=0;
		}
		if(k) {
			if(!(temp_dpp=malloc(sizeof(void *)*k*n_genetic_groups))) ABT_FUNC(MMsg);
			RemBlock=AddRemem(temp_dpp,RemBlock);
			if(!(temp_dp=malloc(sizeof(double)*k2))) ABT_FUNC(MMsg);
			RemBlock=AddRemem(temp_dp,RemBlock);
			for(i=0;i<n_markers;i++) if(marker[i].counts) {
				marker[i].counts=temp_dpp;
				temp_dpp+=n_genetic_groups;
				for(grp=0;grp<n_genetic_groups;grp++) {
					if(marker[i].count_flag[i]) {
						marker[i].counts[grp]=temp_dp;
						temp_dp+=marker[i].n_alleles;
					} else marker[i].counts[grp]=0;
				}
			}
		}
	}
	for(i=0;i<n_markers;i++) if(marker[i].n_alleles) {
		j=marker[i].lumped;
		if(!extra_allele_flag) {
			for(grp=0;grp<n_genetic_groups;grp++)
			  if(marker[i].freq_set[grp][j] && marker[i].freq[grp][j]>0.0) break;
			if(grp==n_genetic_groups) {
				/* Lumped allele not used so remove */
				a=~(1<<j);
				n_all= --marker[i].n_alleles;
				for(k=comp=0;comp<n_comp;comp++) {
					if(marker[i].n_all1[comp]>n_all) {
						for(k1=0;k1<comp_size[comp];k1++) {
							if(id_array[k+k1].pruned_flag[i]) continue;
							req_set[0][i][k+k1]&=a;
							req_set[1][i][k+k1]&=a;
							for(l=0;l<n_all;l++) all_set[i][k+k1][l]&=a;
							all_set[i][k+k1][l]=0;
						}
						marker[i].n_all1[comp]--;
					}
					k+=comp_size[comp];
				}
			}
		}
		for(grp=0;grp<n_genetic_groups;grp++) {
			if(marker[i].count_flag[grp]) {
				for(p=0.0,j=0;j<marker[i].n_alleles;j++) {
					if(marker[i].freq_set[grp][j]) marker[i].counts[grp][j]=marker[i].freq[grp][j];
					else marker[i].counts[grp][j]=1.0;
					p+=marker[i].counts[grp][j];
				}
				for(j=0;j<marker[i].n_alleles;j++) {
					marker[i].freq[grp][j]=marker[i].counts[grp][j]/p;
					marker[i].freq_set[grp][j]=0;
				}
			} else {
				for(p=0.0,j=0;j<marker[i].n_alleles;j++) {
					if(!marker[i].freq_set[grp][j]) marker[i].freq[grp][j]=0.1;
					p+=marker[i].freq[grp][j];
				}
				if(fabs(p-1.0)>0.0001) {
					for(k=j=0;j<marker[i].n_alleles;j++) if(!marker[i].freq_set[grp][j]) k++;
					if(k<j) {
						if(p<1.0) {
							for(j=0;j<marker[i].n_alleles;j++) if(!marker[i].freq_set[grp][j]) marker[i].freq[grp][j]=(1.0-p)/(double)k;
							p=1.0;
						} else {
							printf("Rescaling frequencies for marker %s",marker[i].name);
							if(marker[i].index) printf("(%d)",marker[i].index);
							if(group_var) {
								fputs(" in genetic group ",stdout);
								if(group_var->rec_flag==ST_STRING) fputs(group_var->recode[grp].string,stdout);
								else printf("%d",group_var->recode[grp].value);
							}
							for(j=0;j<marker[i].n_alleles;j++) if(!marker[i].freq_set[grp][j]) marker[i].freq[grp][j]=0.1;
							p+=.1*(double)k;
							fputc('\n',stdout);
						}
					}
				}
				for(j=0;j<marker[i].n_alleles;j++) marker[i].freq[grp][j]/=p;
			}
		}
		if(!marker[i].pos_set) {
			er=1;
			fprintf(stderr,"Position not set for marker %s",marker[i].name);
			if(marker[i].index) fprintf(stderr,"(%d)",marker[i].index);
			fputc('\n',stderr);
		}
	}
	if(!er && n_links) {
		for(i=0;i<n_links;i++) {
			linkage[i].sample_pos=0;
			fx=0;
			for(j=0;j<n_markers;j++) if(marker[j].link_group==i) {
				if(marker[j].pos_set==2) {
					linkage[i].sample_pos=1;
					break;
				}
			}
			for(k=0;k<1+sex_map;k++) {
				if(!linkage[i].range_set[k]) {
					min=DBL_MAX;
					max=-DBL_MAX;
					for(j=0;j<n_markers;j++) if(marker[j].link_group==i) {
						if(marker[j].pos_set==1) fx=1;
						if(marker[j].pos[k]<min) min=marker[j].pos[k];
						if(marker[j].pos[k]>max) max=marker[j].pos[k];
					}
					if(min==-INT_MAX) min=max=0.0;
					linkage[i].r1[k]=min;
					linkage[i].r2[k]=max;
					if(sex_map) printf("Map range (%s) for linkage group '%s' set to %g-%gcM\n",sexstr[k],linkage[i].name,linkage[i].r1[k],linkage[i].r2[k]);
					else {
						printf("Map range for linkage group '%s' set to %g-%gcM\n",linkage[i].name,linkage[i].r1[0],linkage[i].r2[0]);
						linkage[i].r1[1]=linkage[i].r1[0];
						linkage[i].r2[1]=linkage[i].r2[0];
					}
				} else {
					for(j=0;j<n_markers;j++) if(marker[j].link_group==i) {
						if(marker[j].pos_set==1) fx=1;
						k1=0;
						if(marker[j].pos[k]<linkage[i].r1[k]) k1=1;
						if(marker[j].pos[k]>linkage[i].r2[k]) k1=1;
						if(k1) {
							fprintf(stderr,"Marker %s",marker[j].name);
							if(marker[j].index) fprintf(stderr,"(%d)",marker[j].index);
							if(sex_map) fprintf(stderr," located outside of %s linkage group '%s'\n",sexstr[k],linkage[i].name);
							else fprintf(stderr," located outside of linkage group '%s'\n",linkage[i].name);
							er=1;
						}
					}
				}
			}
			if(!fx) { /* If no marker in linkage group has a fixed position, arbitrarily fix first marker */
				min=DBL_MAX;
				k1=-1;
				for(j=0;j<n_markers;j++) if(marker[j].link_group==i) {
					if(marker[j].pos[0]<min) {
						min=marker[j].pos[0];
						k1=j;
					}
				}
				printf("nmk = %d, nlnk = %d, lnk = %d\n",n_markers,n_links,i);
				if(k1<0) ABT_FUNC("Internal error - marker not found\n");
				marker[k1].pos_set=1;
				printf("Position for marker %s",marker[k1].name);
				if(marker[k1].index) printf("(%d)",marker[k1].index);
				printf(" fixed at %g",marker[k1].pos[0]);
				if(sex_map) printf(",%g",marker[k1].pos[1]);
				fputc('\n',stdout);
			}
		}
		for(k=0;k<1+sex_map;k++) {
			p=0.0;
			for(i=0;i<n_links;i++) p+=linkage[i].r2[k]-linkage[i].r1[k];
			if(total_maplength[k]<0.0) {
				if(sex_map) printf("No total %s map length set - no unlinked loci will be allowed\n",sexstr[k]);
				else printf("No total map length set - no unlinked loci will be allowed\n");
				total_maplength[k]=p;
			} else if(p>=total_maplength[k]) {
				if(sex_map) printf("Total %s map length <= sum of linkage group sizes - no unlinked loci will be allowed\n",sexstr[k]);
				else printf("Total map length <= sum of linkage group sizes - no unlinked loci will be allowed\n");
				total_maplength[k]=p;
			}
			if(!sex_map) total_maplength[1]=total_maplength[0];
		}
	}
	if(!er && sex_map) {
		for(j=i=0;i<n_links;i++) if(linkage[i].n_markers>j) j=linkage[i].n_markers;
		if(j) {
			if(!(perm=malloc(sizeof(int)*2*j))) ABT_FUNC(MMsg);
			perm1=perm+j;
			for(i=0;i<n_links;i++) {
				get_locuslist(perm,i,&j);
				for(k=0;k<j;k++) perm1[k]=perm[k];
				sort_sex=0;
				gnu_qsort(perm,(size_t)j,(size_t)sizeof(int),cmp_loci_pos);
				sort_sex=1;
				gnu_qsort(perm1,(size_t)j,(size_t)sizeof(int),cmp_loci_pos);
				for(k=0;k<j;k++) if(perm[k]!=perm1[k]) {
					fprintf(stderr,"Male and female marker maps for linkage group %s have different orders\n",linkage[i].name);
					er=1;
					break;
				}
				k=perm[0];
				p=marker[k].pos[0];
				k1=(p>linkage[i].r1[0])?1:0;
				p=marker[k].pos[1];
				k1^=(p>linkage[i].r1[1])?1:0;
				k=perm[j-1];
				p=marker[k].pos[0];
				k1|=(p<linkage[i].r2[0])?2:0;
				p=marker[k].pos[1];
				k1^=(p<linkage[i].r2[1])?2:0;
				if(!er && k1) {
					fprintf(stderr,"Male and female marker maps for linkage group %s have different numbers of intervals\n",linkage[i].name);
					er=1;
					break;
				}
			}
			free(perm);
		}
	}
	if(er) exit(EXIT_FAILURE);
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "InitValues"
void InitValues(void)
{
	int i,j,k,k1,type,n=0;
	double y,s=0.0,mu=0.0;
	struct id_data *data;
	struct Id_Record *id;
	
	type=model.var.type;
	j=model.var.var_index;
	if(!(type&ST_FACTOR)) {
		if(!(type&ST_CONSTANT)) {
			id=id_array;
			for(i=0;i<ped_size;i++,id++) {
				id->bv=0.0;
				if(id->data1) {
					for(k=0;k<id->n_rec;k++)	{
						data=id->data1[k]+j;
						if(data && data->flag) {
							if(data->flag&ST_INTTYPE) y=(double)data->data.value;
							else y=data->data.rvalue;
							mu+=y;
							n++;
						} else {
							k1=id->n_rec-1;
							if(k<k1) id->data1[k]=id->data1[k1];
							id->data1[k1]=0;
							id->n_rec=k1;
							if(!k1) id->res=0;
							k--;
						}
					}
				}
			}
			if(n) {
				if(grand_mean_set) mu=grand_mean;
				else mu/=(double)n;
				id=id_array;
				for(i=0;i<ped_size;i++,id++) if(id->res) {
					for(k=0;k<id->n_rec;k++) {
						data=id->data1[k]+j;
						if(data->flag&ST_INTTYPE) y=(double)data->data.value;
						else y=data->data.rvalue;
						y-=mu;
						s+=y*y;
						id->res[k]=y;
						if(use_student_t) id->vv[k]=1.0;
						if((type&ST_CENSORED)&&(data->flag&2)) {
							id->cens[k]=0.0;
							censored_flag=1;
						}
					}
				}
			}
		} else {
			id=id_array;
			for(i=0;i<ped_size;i++,id++) {
				id->bv=0.0;
				if(id->data) {
					data=id->data+j;
					if(data && data->flag) {
						if(data->flag&ST_INTTYPE) y=(double)data->data.value;
						else y=data->data.rvalue;
						mu+=y;
						n++;
					} else {
						id->data=0;
						id->res=0;
					}
				}
			}
			if(n) {
				if(grand_mean_set) mu=grand_mean;
				else mu/=(double)n;
				id=id_array;
				for(i=0;i<ped_size;i++,id++) if(id->res) {
					data=id_array[i].data+j;
					if(data->flag&ST_INTTYPE) y=(double)data->data.value;
					else y=data->data.rvalue;
					y-=mu;
					s+=y*y;
					if((type&ST_CENSORED)&&(data->flag&2)) {
						id->cens[0]=0.0;
						censored_flag=1;
					}
					id->res[0]=y;
					if(use_student_t) id->vv[0]=1.0;
				}
			}
		}
		if(!res_var_set) {
			residual_var=n?s/(double)n:1.0;
			if(residual_var<residual_var_limit) residual_var=residual_var_limit;
		} else if(residual_var<residual_var_limit) {
			residual_var=residual_var_limit;
			fprintf(stderr,"Warning - residual variance reset to limit (%g)\n",residual_var);
		}
		if(!add_var_set) {
			additive_var=residual_var;
			if(additive_var<additive_var_limit) additive_var=additive_var_limit;
		} else if(additive_var<additive_var_limit) {
			additive_var=additive_var_limit;
			fprintf(stderr,"Warning - additive variance reset to limit (%g)\n",additive_var);
		}
	}
	if(!grand_mean_set) grand_mean=mu;
	if(syst_var[SYST_NO_OVERDOMINANT].flag && syst_var[SYST_NO_OVERDOMINANT].data.value) no_overdominant=1;
	if(syst_var[SYST_SI_MODE].flag && syst_var[SYST_SI_MODE].data.value) si_mode=1;
	if(syst_var[SYST_CENSOR_MODE].flag && syst_var[SYST_CENSOR_MODE].data.value) censor_mode=1;
	if(syst_var[SYST_TAU_MODE].flag) {
		if(syst_var[SYST_TAU_MODE].flag==ST_REAL) tau_mode=(int)syst_var[SYST_TAU_MODE].data.rvalue;
		else tau_mode=syst_var[SYST_TAU_MODE].data.value;
	}
	if(syst_var[SYST_DEBUG_LEVEL].flag)	{
		if(syst_var[SYST_DEBUG_LEVEL].flag==ST_REAL) debug_level=(int)syst_var[SYST_DEBUG_LEVEL].data.rvalue;
		else debug_level=syst_var[SYST_DEBUG_LEVEL].data.value;
	}
	if(syst_var[SYST_PEEL_TRACE].flag) {
		if(syst_var[SYST_PEEL_TRACE].flag==ST_REAL) peel_trace=(int)syst_var[SYST_PEEL_TRACE].data.rvalue;
		else peel_trace=syst_var[SYST_PEEL_TRACE].data.value;
	}
	if(syst_var[SYST_LM_RATIO].flag)	{
		if(syst_var[SYST_LM_RATIO].flag==ST_REAL) lm_ratio=syst_var[SYST_LM_RATIO].data.rvalue;
		else lm_ratio=(double)syst_var[SYST_LM_RATIO].data.value;
	}
	if(syst_var[SYST_L_PROB].flag) {
		if(syst_var[SYST_L_PROB].flag==ST_REAL) l_prob=syst_var[SYST_L_PROB].data.rvalue;
		else l_prob=(double)syst_var[SYST_L_PROB].data.value;
	}
	if(syst_var[SYST_L_CYCLES].flag) {
		if(syst_var[SYST_L_CYCLES].flag==ST_REAL) l_cycles=(int)syst_var[SYST_L_CYCLES].data.rvalue;
		else l_cycles=syst_var[SYST_L_CYCLES].data.value;
	}
	if(syst_var[SYST_TAU].flag) {
		if(syst_var[SYST_TAU].flag==ST_REAL) tau_beta=syst_var[SYST_TAU].data.rvalue;
		else tau_beta=(double)syst_var[SYST_TAU].data.value;
	} else if(tau_mode==1) tau_beta=residual_var;
	switch(tau_mode) {
	 case 0:
		tau_beta*=residual_var;
	 case 1:
		tau=tau_beta;
		break;
	 case 2:
		tau=residual_var*tau_beta;
		break;
	}
	if(debug_level&2) for(i=0;i<N_MOVE_STATS;i++) move_stats[i].success=move_stats[i].n=0;
	init_rand();
}
