/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *             Simon Heath - University of Washington                       *
 *                                                                          *
 *                        August 1997                                       *
 *                                                                          *
 * loki_peel.c:                                                             *
 *                                                                          *
 * Perform peeling calculation                                              *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <math.h>
#include <stdio.h>

#include "loki.h"
#include "loki_peel.h"

void penetrance(double *val,int id,int locus,int n_all,int n_bits)
{
	int i,j,k,mtype,rec,nrec,censflag,idx;
	double p,y,m,*eff,kon1,kon2,sd,wt;

	if(!id_array[id].res) return;
	if(locus<0) eff=tlocus[-1-locus].eff;
	else eff=marker[locus].mterm->eff;
	if(n_bits) j=1<<(n_bits+n_bits);
	else j=n_all*n_all;
	for(p=0.0,i=0;i<j;i++) p+=val[i];
	if(p<=0.0) {
		fprintf(stderr,"penetrance() called with zero function - ");
		print_orig_id(stderr,id+1);
		fputc('\n',stderr);
		return;
	}
	kon1=1.0/(2.0*residual_var);
	kon2=sqrt(kon1/M_PI);
	sd=sqrt(residual_var*2.0);
	idx=model.var.var_index;
	mtype=model.var.type;
	nrec=id_array[id].n_rec;
	if(!nrec) nrec=1;
	for(rec=0;rec<nrec;rec++) {
		censflag=0;
		y=id_array[id].res[rec];
		wt=use_student_t?1.0/id_array[id].vv[rec]:1.0;
		if(!censor_mode && (mtype&ST_CENSORED)) {
			if(mtype&ST_CONSTANT) {
				if(id_array[id].cens) censflag=1;
			} else {
				if(id_array[id].data1[rec][idx].flag&2) censflag=1;
			}
		}
		/* Add on current genotype effect */
		if(locus<0)	{
			if(tlocus[-1-locus].flag&MARKER_SAMPLED) {
				k=tlocus[-1-locus].gt[id]-1;
				if(k) y+=eff[k-1];
			}
		} else if(marker[locus].flag&MARKER_SAMPLED)	{
			k=marker[locus].gt[id]-1;
			if(k) y+=eff[k-1];
		}
		if(censflag) {
			y-=id_array[id].cens[rec];
			p=.5*erfc(y*sqrt(wt)/sd);
		} else p=kon2*exp(-y*y*kon1*wt);
		val[0]*=p;
		if(!n_bits) { /* Non-sparse representation */
			for(k=0,i=1;i<n_all;i++) for(j=0;j<=i;j++) {
  				m=y-eff[k++];
				if(censflag) p=.5*erfc(m*sqrt(wt)/sd);
				else p=kon2*exp(-m*m*wt*kon1);
				val[i*n_all+j]*=p;
				if(i!=j) val[j*n_all+i]*=p;
			}
			j=n_all*n_all;
		} else {
			for(k=0,i=1;i<n_all;i++) for(j=0;j<=i;j++) { /* Sparse representation */
  				m=y-eff[k++];
				if(censflag) p=.5*erfc(m*sqrt(wt)/sd);
				else p=kon2*exp(-m*m*wt*kon1);
				val[(i<<n_bits)|j]*=p;
				if(i!=j) val[(j<<n_bits)|i]*=p;
			}
			j=1<<(n_bits+n_bits);
		}
	}
}
