/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *             Simon Heath - University of Washington                       *
 *                                                                          *
 *                        October 1997                                      *
 *                                                                          *
 * meiosis_scan.c:                                                          *
 *                                                                          *
 * Perform update of segregation indicators using meiosis sampler           *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ranlib.h"
#include "utils.h"
#include "loki.h"
#include "loki_peel.h"

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "meiosis_scan"
void meiosis_scan(int link)
{
	int i,i1,j,k,k1,n_loci,par_flag,fg,s,s1,locus,comp,r_flag,nkids,kid;
	double x,x1,xx,xx1;
	static double *pp[2],*pen[2],*lks[2],*recom,rr;
	static int *loci,*m_flag;

	if(link<0) { /* Free space and prevent reuse */
		if(pp[0]) {
			free(pp[0]);
			pp[0]=0;
		}
		if(loci) {
			free(loci);
			loci=0;
		}
		return;
	}
	n_loci=linkage[link].n_markers;
	if(!n_loci) return;
	if(!pp[0]) { /* Allocate space on first time through */
		for(j=i=0;i<n_links;i++) {
			k=linkage[i].n_markers;
			if(k>j) j=k;
		}
		if(!(pp[0]=malloc(sizeof(double)*(7*j-1)))) ABT_FUNC(MMsg);
		pp[1]=pp[0]+j;
		pen[0]=pp[1]+j;
		pen[1]=pen[0]+j;
		lks[0]=pen[1]+j;
		lks[1]=lks[0]+j;
		recom=lks[1]+j;
		if(!(loci=malloc(sizeof(int)*j*2))) ABT_FUNC(MMsg);
		m_flag=loci+j;
	}
	for(i=comp=0;comp<n_comp;comp++) {
		/* Pick markers in this component where no. alleles >=2 */
		for(k1=k=0;k<linkage[link].n_markers;k++) {
			locus=linkage[link].mk_index[k];
			if(marker[locus].n_all1[comp]>=2) loci[k1++]=locus;
		}
		if(!k1) {
			i+=comp_size[comp];
			continue;
		}
		n_loci=k1;
		gnu_qsort(loci,(size_t)n_loci,(size_t)sizeof(int),cmp_loci_pos);
		for(i1=0;i1<comp_size[comp];i1++,i++) if(id_array[i].nkids) {
			par_flag=2-id_array[i].sex;
			nkids=id_array[i].nkids;
			r_flag=id_array[i].sire?0:1; /* Is this a founder? */
			if(nkids==1) r_flag|=2;
			if(r_flag^3) for(j=0;j<nkids;j++)	{
				kid=id_array[i].kids[j];
				if(n_loci>1) {
					s=marker[loci[0]].seg[par_flag][kid];
					x=marker[loci[0]].pos[par_flag];
					for(k=1;k<n_loci;k++) {
						x1=marker[loci[k]].pos[par_flag];
						recom[k-1]=.5*(1.0-exp(0.02*(x-x1)));
						x=x1;
					}
				}
				/* marker[locus].m_flag[i] is a flag where the following bits mean:
				 *  0  i is homozygous at locus
				 *  1  maternal seg is forced by the data
				 *  2  paternal seg is forced by the data
				 * 
				 * Note: this is all based on the original marker data */
				for(k=0;k<n_loci;k++) { /* Calculate penetrances */
					locus=loci[k];	
					m_flag[k]=0;
					s=marker[locus].seg[par_flag][kid];
					if(s<0) ABT_FUNC("Illegal seg.\n");
					/* Check if seg. is forced by data */
					if((k1=marker[locus].m_flag[kid])) {
						if((par_flag==X_PAT && (k1&2)) || (par_flag==X_MAT && (k1&4)))	{
							s=marker[locus].seg[par_flag][kid];
							m_flag[k]=2;
							pen[s][k]=1.0;
							pen[s^1][k]=0.0;
							continue;
						}
					}
					if(marker[locus].m_flag[i]&1) { /* Parent is homozygous */
						m_flag[k]=1;
						pen[0][k]=pen[1][k]=0.5;
						continue;
					}
					s=(marker[locus].seg[par_flag][kid]^=1);
					lks[s^1][k]=marker[locus].lk_store[comp];
 					if(pass_founder_genes1(locus,kid,par_flag)) {
						xx=seg_pen(locus,comp,&fg,0);
						lks[s][k]=xx;
						if(fg) {
							pen[s][k]=0.0;
							pen[s^1][k]=1.0;
						} else {
							xx1=lks[s^1][k];
							if(xx>xx1) {
								pen[s][k]=1.0;
								pen[s^1][k]=exp(xx1-xx);
							} else {
								pen[s^1][k]=1.0;
								pen[s][k]=exp(xx-xx1);
							}
						}
					} else {
						pen[s][k]=pen[s^1][k]=0.5;
						lks[s][k]=lks[s^1][k];
					}
				}
				xx=pp[0][0]=pen[0][0];
				xx+=pp[1][0]=pen[1][0];
				pp[0][0]/=xx;
				pp[1][0]/=xx;
				for(k=1;k<n_loci;k++) {
					rr=recom[k-1];
					xx=pp[0][k]=pen[0][k]*(rr*pp[1][k-1]+(1.0-rr)*(pp[0][k-1]));
					xx+=pp[1][k]=pen[1][k]*(rr*pp[0][k-1]+(1.0-rr)*(pp[1][k-1]));
					pp[0][k]/=xx;
					pp[1][k]/=xx;
				}
				k=n_loci-1;
				locus=loci[k];
				if(m_flag[k]==2) s=marker[locus].seg[par_flag][kid];
				else {
					s1=marker[locus].seg[par_flag][kid];
					s=(pp[0][k]>0.0 && ranf()<=pp[0][k])?0:1;
					marker[locus].seg[par_flag][kid]=s;
					if(!m_flag[k]) marker[locus].lk_store[comp]=lks[s][k];
					if(s!=s1) pass_founder_genes1(locus,kid,par_flag);
				}
				for(k=n_loci-2;k>=0;k--) {
					s1=s;
					locus=loci[k];
					if(m_flag[k]==2) s=marker[locus].seg[par_flag][kid];
					else {
						rr=recom[k];
						xx=(pp[s][k]*=(1.0-rr));
						xx+=(pp[s^1][k]*=rr);
						s=(pp[0][k]>0.0 && xx*ranf()<=pp[0][k])?0:1;
						s1=marker[locus].seg[par_flag][kid];
						marker[locus].seg[par_flag][kid]=s;
						if(!m_flag[k]) marker[locus].lk_store[comp]=lks[s][n_loci-1];
						if(s!=s1) pass_founder_genes1(locus,kid,par_flag);
					}
				}
			}
		}
	}
}
