/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *                      Simon Heath - MSKCC                                 *
 *                                                                          *
 *                          August 2000                                     *
 *                                                                          *
 * sample_cens.c:                                                           *
 *                                                                          *
 * Routines for sampling censored data                                      *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "loki.h"
#include "sample_cens.h"

/* Sample truncated (censored) data points */
void Sample_Censored(void)
{
	int i,j,type,idx,er;
	double sd,y,z;
	
	sd=sqrt(residual_var);
	type=model.var.type;
	idx=model.var.var_index;
	if(type&ST_CONSTANT)	{
		for(i=0;i<ped_size;i++) if(id_array[i].res && id_array[i].cens) {
			y=id_array[i].res[0]-id_array[i].cens[0];
			z=sd*trunc_normal(y/sd,0.0,2,&er); /* Sample from normal left truncated at y */
			if(!er) {
				id_array[i].cens[0]=z-y;
				id_array[i].res[0]=z;
			}
		}
	} else for(i=0;i<ped_size;i++) if(id_array[i].res && id_array[i].cens) {
		for(j=0;j<id_array[i].n_rec;j++) {
			if(id_array[i].data1[j][idx].flag&2) {
				y=id_array[i].res[j]-id_array[i].cens[j];
				z=sd*trunc_normal(y/sd,0.0,2,&er);
				if(!er) {
					id_array[i].cens[j]=z-y;
					id_array[i].res[j]=z;
				}
			}
		}
	}
}

