/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *                   Simon Heath - MSKCC                                    *
 *                                                                          *
 *                       August 2000                                        *
 *                                                                          *
 * sample_nu.c:                                                             *
 *                                                                          *
 * Sampling routines connected to t-distribution of residuals               *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <math.h>
#include <stdio.h>
#include <float.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include "ranlib.h"
#include "version.h"
#include "utils.h"
#include "loki.h"
#include "sample_nu.h"

double res_nu;
int use_student_t;
static double nu_list[]={1,1.5,2,5,10,25,50,100,-1};
static double *nu_prob;
static int n_nu;

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "init_sample_nu"
void init_sample_nu(void)
{
	int i,type,nrec;
	double n,v;
	
	n_nu=0;
	while(nu_list[n_nu]>=0.0) n_nu++;	
	if(n_nu) {
		res_nu=nu_list[n_nu-1];
		if(!(nu_prob=malloc(sizeof(double)*2*n_nu))) ABT_FUNC(MMsg);
	}
	n=0.0;
	type=model.var.type;
	for(i=0;i<ped_size;i++) if(id_array[i].res) {
		if(type&ST_CONSTANT) nrec=1;
		else nrec=id_array[i].n_rec;
		n+=(double)nrec;
	}
	for(i=0;i<n_nu;i++) {
		v=nu_list[i];
		nu_prob[i]=lgamma((v+n)*.5)-lgamma(.5*v)-log(v*M_PI*residual_var)*.5*n;
	}
	res_nu=.25;
}

void free_sample_nu(void)
{
	if(nu_prob) free(nu_prob);
	nu_prob=0;
}

void sample_weights(void) 
{
	int i,j,type,nrec;
	double y,n,s,kk;
  
	type=model.var.type;
	if(res_nu<1.0e-6) {
		for(i=0;i<ped_size;i++) if(id_array[i].res) {
			if(type&ST_CONSTANT) nrec=1;
			else nrec=id_array[i].n_rec;
			for(j=0;j<nrec;j++) id_array[i].vv[j]=1.0;
		}
	} else {
		kk=residual_var/res_nu;
		n=(1.0/res_nu)+1.0;
		for(i=0;i<ped_size;i++) if(id_array[i].res) {
			if(type&ST_CONSTANT) nrec=1;
			else nrec=id_array[i].n_rec;
			for(j=0;j<nrec;j++) {
				y=id_array[i].res[j];
				s=y*y;
				y=(s+kk)/(sgamma(n*0.5)*2.0);
				id_array[i].vv[j]=y/residual_var;
			}
		}
	}
}

void sample_nu(void)
{
	int i,j,type,nrec;
	double y,n,s,s1,v,z,nu,np,np1;

	res_nu=.25;
	sample_weights();
	return;
	
	nu=ranf();
	if(nu>=1.0e-6 && nu<=1.0) {
		type=model.var.type;
		n=0.0;
		np=np1=s1=0.0;
		for(i=0;i<ped_size;i++) if(id_array[i].res) {
			if(type&ST_CONSTANT) nrec=1;
			else nrec=id_array[i].n_rec;
			for(j=0;j<nrec;j++) {
				n++;
				y=id_array[i].res[j];
				s=y*y/residual_var;
				np+=s*res_nu;
				np1+=s*nu;
				s1+=s;
			}
		}
		v=1.0/res_nu;
		z=lgamma((v+n)*.5)-lgamma(.5*v)-log(v*M_PI*residual_var)*.5*n;
		np=z-.5*(v+n)*log(1.0+np);
		v=1.0/nu;
		z=lgamma((v+n)*.5)-lgamma(.5*v)-log(v*M_PI*residual_var)*.5*n;
		np1=z-.5*(v+n)*log(1.0+np1);
		s1=-.5*(n*log(2.0*M_PI*residual_var)+s1);
		if(np1>np) z=1.0;
		else z=exp(np1-np);
/*  		printf("%g %g  %g %g %g %g",res_nu,np,nu,np1,s1,z);*/
		if(ranf()<=z) {
			res_nu=nu;
/*			fputs(" S\n",stdout); */
		} /* else fputs(" F\n",stdout); */
	}
	sample_weights();
}

