/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *                   Simon Heath - MSKCC                                    *
 *                                                                          *
 *                       August 2000                                        *
 *                                                                          *
 * sample_rand.c:                                                           *
 *                                                                          *
 * Sampling routine for uncorrelated random variance components             *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <math.h>
#include <stdio.h>
#include <float.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include "ranlib.h"
#include "version.h"
#include "utils.h"
#include "sparse.h"
#include "loki.h"
#include "sample_rand.h"

static double *work;
double *c_var;
int n_random;


#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "init_rand"
void init_rand(void)
{
	int i,k,type,df;

	for(n_random=i=k=0;k<model.n_terms;k++) {
		type=model.term[k].vars[0].type;
		if(type&ST_RANDOM) {
			n_random++;
			df=model.term[k].df;
			if(df>i) i=df;
		}
	}
	if(n_random) {
		if(!(c_var=malloc(sizeof(double)*n_random))) ABT_FUNC(MMsg);
		for(k=0;k<n_random;k++) c_var[k]=residual_var;
	}
	if(polygenic_flag) if(!(work=malloc(sizeof(double)*ped_size))) ABT_FUNC(MMsg);
}

void free_rand(void)
{
	if(c_var) free(c_var);
	if(work) free(work);
	c_var=0;
	work=0;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "sample_rand"
void sample_rand(void)
{
	int i,k,r,type,df;
	double *eff,y,ss,n;
	
	for(r=k=0;k<model.n_terms && r<n_random;k++) {
		type=model.term[k].vars[0].type;
		if(type&ST_RANDOM) {
			if(!(type&ST_FACTOR)) ABT_FUNC("Can't fit random effect with continuous covariates\n");
			df=model.term[k].df;
			eff=model.term[k].eff;
			ss=0.0;
			for(i=0;i<df;i++) {
				y=eff[i];
				ss+=y*y;
			}
			n=(double)df+RES_PRIOR_VC0;
			ss+=RES_PRIOR_VC0*RES_PRIOR_SC0;
			c_var[r++]=ss/(sgamma(n*.5)*2.0);
		}
	}
}

void sample_additive_var(void)
{
	int i,id,j,k,comp;
	double y,n,z,ss;
	struct SparseMatRec *AI;
	
	ss=0.0;
	for(id=comp=0;comp<n_comp;comp++) {
		AI=AIMatrix[comp];
		for(i=0;i<comp_size[comp];i++) {
			y=id_array[i+id].bv;
			work[i]=y*AI[i].val;
			for(j=AI[i].x;j<AI[i+1].x;j++) {
				k=AI[j].x;
				z=AI[j].val;
				work[k]+=y*z;
				work[i]+=id_array[k+id].bv*z;
			}
		}
		for(i=0;i<comp_size[comp];i++) ss+=id_array[i+id].bv*work[i];
		id+=i;
	}
	n=(double)ped_size+RES_PRIOR_VA0;
	ss+=RES_PRIOR_VA0*RES_PRIOR_SA0;
	additive_var=ss/(sgamma(n*.5)*2.0);
}
