/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *             Simon Heath - University of Washington                       *
 *                                                                          *
 *                       March 1997                                         *
 *                                                                          *
 * scan.h:                                                                  *
 *                                                                          *
 ****************************************************************************/

#ifdef USE_DMALLOC
#include <dmalloc.h>
#endif

#include "ranlib.h"
#include "loki_struct.h"

#define DataFileError(a) New_DFE(__FILE__,__LINE__,a)

extern int symbol_lookup(char *,int);
extern void print_scan_warn(char *, ...);
extern void print_scan_err(char *, ...);
extern void ReadData(void);

union arg_type {
	long value;
	double rvalue;
	struct scan_var *var;
	struct var_element *element;
	char *string;
};

struct express {
	union arg_type arg;
	int type;
};

struct var_element {
	union arg_type arg;
	int n_levels,index,oindex,type;
};

struct scan_var {
	struct scan_var *var_left,*var_right;
	char *name;
	int balance;
	int n_elements; /* 1 for scalars, >=1 for arrays */
	int vtype;
	struct var_element *element;
};

struct var_list {
	struct var_list *next;
	struct scan_var *var;
	int index;
};

struct label_node {
	struct label_node *left,*right;
	int type,index,balance;
	union	{
		char *string;
		long value;
	} data;
};

struct InFile {
	struct InFile *next;
	char *name;
	int shell_flag;
	int nvar,ncol,n_records,id_col,family_col;
	struct var_element **element;
	struct format *format;
	struct fformat *fformat;
	struct DataBlock *data;
};

struct Id_Record {
	int sire,dam;
	int nfam,nkids;
	int fam_code;
	int sex,affected;
	int *kids,*famlist;
	int family;
	int component;
	int group;
	int *haplo[2];
	int *mkflag;
	int flag,nrec,order;
	int fc_flag;
	int nhaps[2],ngens,ngens1;
	int allele[2];
	struct id_data *data,**data1;
};

struct Family {
	int sire,dam;
	int nkids;
	int *kids;
};

struct format {
	int n_atoms,line;
	struct format_atom *f_atoms;
};

struct Marker {
	int index;
	int link;
	int **order,*o_size;
	int **allele_trans;
	struct scan_var *var;
	struct var_element *element,*hap_element[2];
};

struct format_clause {
	int n_atoms,fc_size;
	struct format_atom **f_atoms;
};

struct fformat {
	char *fs;
	char *rs;
};

struct format_atom {
	int size,pos;
};

struct model {
	struct model *next;
	struct scan_var *trait;
	struct model_list *model_list;
	int index;
};

struct model_list {
	struct model_list *next;
	int nvar;
	struct var_element **element;
};
		
struct Link {
	struct Link *next;
	int n_loci;
	int type;
	char *name;
	struct var_element **element;
};

struct Miss {
	struct Miss *next;
	char *Missing;
	int nvar;
	struct var_element **element;
};

struct op_stack {
	int type;
	union arg_type arg;
};

struct operation {
	struct operation *next;
	int type,op;
	union arg_type arg;
};

struct Restrict {
	struct Restrict *next;
	struct operation *Op_List;
	int nvar;
	int flag;
	struct var_element **element;
};

struct Censor {
	struct Censor *next;
	struct operation *Op_List;
	struct var_element *element;
};

union DataRec {
	long value;
	double rvalue;
	struct label_node *node;
};

struct DataBlock {
	struct DataBlock *next;
	int blocksize,record_ptr;
	unsigned char *type;
	union DataRec *records;
};

struct recode_table_tag {
	int index;
	struct label_node *node;
};

extern void WriteData(void);
extern void free_nodes(void);
extern void setup_pedigree(int,struct recode_table_tag *);
extern void recode_factors(int,struct recode_table_tag *);
extern void recode_marker_data(int,struct recode_table_tag *);
extern int check_missing(int,int,int,struct DataBlock *);
extern int Calculate_NRM(void);
extern void check_vars(struct scan_var *,int *,void(struct scan_var *,int *));
extern void free_infile(struct InFile *);
extern void prune_pedigree(void);
extern void count_components(void);
extern void free_restrict(struct Restrict *);
extern void affected_data(void);
extern void restrict_data(void);
extern void print_orig_alleles(FILE *,const int,const int,const int);
extern void censored_data(void);
extern void cleanup_unused(void);
extern void Output_Data(void);
extern void print_orig_id(FILE *,const int,const int);
extern void print_orig_id1(FILE *,const int,const int);
extern void print_orig_family(FILE *,const int,const int);
extern void print_orig_triple(FILE *,const int);
extern void match_records(void);
extern void free_var(struct scan_var *);
extern void WriteReport(void);
extern void New_DFE(const char *,const int,const char *);
extern int Genotype_Elimination(int);
extern void InitFamilies(void);
extern void check_ymark(void);
extern void free_op(struct operation *);
extern void yyerror(char *);
extern void yyerror1(char *);
extern void check_sex(void),count_loops(void),handle_groups(void),check_inbreeding(void);
extern struct label_node *find_node(const void *,int,int);

extern struct InFile *Infiles;
extern struct scan_var *root_var;
extern struct Link *links;
extern struct Miss *Miss;
extern struct Restrict *Restrictions;
extern struct Censor *Censored;
extern struct operation *Affected;
extern struct model *Models;
extern struct Marker *markers,*traitlocus;
extern struct Id_Record *id_array;
extern struct var_element **var_factors;
extern struct label_node **ped_recode,**family_recode,***factor_recode;
extern struct Family *family;
extern struct remember *RemBlock;

extern int *rec_tab,*rec_tab1;
extern char linebuf[],linebuf1[],*Filter,*rsformat,*fsformat,*OutputFile;
extern int nfiles,scan_error,scan_error_n,scan_warn_n;
extern int lineno,lineno1,at_file,at_model,at_use,begin_comm;
extern int max_scan_errors,max_scan_warnings,nrm_flag;
extern int ped_size,pruned_ped_size,*ped_recode1,n_factors,n_markers,n_id_records,n_nonid_records;
extern int n_comp,*comp_size,n_orig_families,n_families,verbose,family_id;
extern struct express *sex_exp[2];
extern struct var_element *sex_elem,*group_elem;

extern unsigned int RunID;

#define MAX_LOOP 16
#define MAX_INCLUDE 5

extern int loop_level,loop_ptr[MAX_LOOP],loop_stat[MAX_LOOP],loop_record,loop_stack_size;
extern int loop_main_ptr,in_loopclause,loop_clause_end,loop_clause_step,loop_clause_ptr;
extern struct var_element *loop_clause_element,**id_elements,**nonid_elements;

extern FILE *yyin;

#define SCAN_ERR 1
#define PED_ERR 8
#define FILE_ERR 16
#define FORMAT_ERR 32
#define LINK_ERR 64

#define NUM_SYSTEM_VAR 9

extern int syst_var[NUM_SYSTEM_VAR];

#define PRUNE_OPTION 0
#define RECODE_OPTION 1
#define NO_EXTRA_ALLELE 2
#define PEEL_OPTION 3
#define TRACE_RESTRICT 4
#define TRACE_CENSORED 5
#define TRACE_AFFECTED 6
#define CORRECT_ERRORS 7
#define PEEL_TRACE 8
